#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#       easyIstr.py
#
#   15/04/16    新規作成 ver1.0
#      04/26    塑性解析を追加 ver1.1
#      04/28    並列処理追加   ver1.11
#
#
# generated by wxGlade 0.6.7+ on Thu Apr  2 17:28:32 2015
#

import wx
import os
import glob
import csv
import re
import pyFistr
import abaqus2fistr

version = "1.11"
currDir = ""
sourceMeshFile = ""
istrDataFileName = "easyIstr_data"
istrData = []
materialDBName = ""
anaType = ""            #解析の種類
selPairName = ""        #選択しているcontactのPairName
materialDBCont =[]
meshGroup = []          #meshグループ名list:[EGRP, SGRP, NGRP]
matDefine = []          #材料定義:[[EGRP名, 材料], ...]
bndDefine = []          #BOUNDARYグループ名（変位）
cloadDefine = []        #CLOADグループ名（荷重）
dloadDefine = []        #DLOADグループ名（面圧）
vloadDefine = []        #VLOADグループ名（体積力）
centDefine = []         #CENTグループ名（遠心力）
tempDefine = []         #TEMPERATURE
gravDefine = []         #GRAV
contactDefine = []      #CONTACT
stepDefine = []         #STEP

treeData = []
treeRoot = [] 
treeFistrModel = []
treeSolution = []
treeMaterial = []
treeBoundary = []
treeStep = []
treeSolver = []
treePost = []

def analysisTypes():
    anaTypes = [_(u"線形弾性静解析"), _(u"非線形静解析"), _(u"動解析")]
    return anaTypes

def initialTreeParts():
    root = [_(u"FrontISTR anarysis")] 
    fistrModel = [_(u"FistrModel.msh")]
    solution =[[_(u"解析の種類"),[_(u"設定")]]]
    material = [_(u"材料物性値")]
    boundary = [[_(u"境界条件"), [_(u"BOUNDARY（変位）"), _(u"CLOAD（荷重）"),
                                  _(u"DLOAD（圧力）"), _(u"VLOAD（体積力）"),
                                  _(u"GRAV（重力）"), _(u"CENT（遠心力）"),
                                  _(u"TEMPERATURE（温度）"),
                                  _(u"CONTACT（接触）")]]]
    step = [_(u"ステップ解析")]
    solver = [[_(u"solver"), [_(u"出力")]]]
    post = [_(u"post")]
    return [root, fistrModel, solution, material, boundary, step, solver, post]

materialType = [
    ["ELASTIC", [], []],
    ["PLASTIC",
        ["MISES", "MOHR-COULOMB", "DRUCKER-PRANGER"],
        ["BILINEAR", "MULTILINEAR", "SWIFT", "RAMBERG-OSGOOD", "KINEMATIC", 
          "COMBINED"]],
    ["HYPERELASTIC",
        ["NEOHOOKE", "MOONY-RIVLIN", "ARRUDA-BOYCE"], []],
    ["VISCOELASTIC", [], []],
    ["CREEP", ["NORTON"], []]
    ]

matGrpHeader = ["!ITEM", "!ELASTIC", "!DENSITY", "!EXPANSION_COEFF", "!PLASTIC", "!HYPERELASTIC", "!VISCOELASTIC", "!CREEP"]


solveMethod = ["CG", "BiCGSTAB", "GMRES", "GPBiCG", "DIRECT"]

def outputItems():
    items = [_(u"変位:DISPLACEMENT"), _(u"節点反力:REACTION"),
        _(u"節点ひずみ:NSTRAIN"),  _(u"節点応力:NSTRESS"), _(u"節点Mises応力:NMISES"),
        _(u"要素ひずみ:ESTRAIN"), _(u"要素応力:ESTRESS"), _(u"要素Mises応力:EMISES"),
        _(u"積分点ひずみ:ISTRAIN"), _(u"積分点応力:ISTRESS"),
        _(u"積分点塑性ひずみ:PL_ISTRAIN"), _(u"節点熱ひずみ:TH_NSTRAIN"),
        _(u"要素熱ひずみ:TH_ESTRAIN"), _(u"速度:VEL"), _(u"加速度:ACC")]
    return items

#defailtの出力item
def iniOutputItems():
    items = [_(u"変位:DISPLACEMENT"), _(u"節点応力:NSTRESS"),
        _(u"節点Mises応力:NMISES"), _(u"要素応力:ESTRESS"), _(u"要素Mises応力:EMISES")]
    return items

#dynamicSolution
def defaultDynamicSolutionCont():
    #全6行、21ヶのデータ
    data = ["1", "1",
            "0.0", "1.0", "500", "1e-5",
            "0.5", "0.25",
            "1", "1", "0.0", "0.0",
            "100", "", "1",
            "0", "0", "0", "0", "0", "0"]
    return data

# begin wxGlade: dependencies
import gettext
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


class MyFrame(wx.Frame):
    def __init__(self, *args, **kwds):
        # begin wxGlade: MyFrame.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)
        self.label_1 = wx.StaticText(self, wx.ID_ANY, _("easyIstr for FronfISTR"))
        self.label_2 = wx.StaticText(self, wx.ID_ANY, _(u"設定項目"))
        self.tree_ctrl_1 = wx.TreeCtrl(self, wx.ID_ANY, style=wx.TR_HAS_BUTTONS | wx.TR_NO_LINES | wx.TR_DEFAULT_STYLE | wx.SUNKEN_BORDER)
        self.notebook_1 = wx.Notebook(self, wx.ID_ANY, style=0)
        self.notebook_1_pane_1 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.radio_btn_1 = wx.RadioButton(self.notebook_1_pane_1, wx.ID_ANY, _("unv2fistr"))
        self.radio_btn_2 = wx.RadioButton(self.notebook_1_pane_1, wx.ID_ANY, _("abaqus2fistr"))
        self.label_3 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _(u"ファイル名:"))
        self.text_ctrl_1 = wx.TextCtrl(self.notebook_1_pane_1, wx.ID_ANY, "")
        self.button_1 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"参照..."))
        self.button_2 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"変換"))
        self.sizer_7_staticbox = wx.StaticBox(self.notebook_1_pane_1, wx.ID_ANY, _(u"ファイル変換"))
        self.label_4 = wx.StaticText(self.notebook_1_pane_1, wx.ID_ANY, _("label_4"))
        self.button_4 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"mesh読込"))
        self.button_47 = wx.Button(self.notebook_1_pane_1, wx.ID_ANY, _(u"形状確認"))
        self.sizer_10_staticbox = wx.StaticBox(self.notebook_1_pane_1, wx.ID_ANY, _(u"メッシュ内容"))
        self.notebook_1_pane_2 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_30 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"材料DB（mat.csv）"))
        self.text_ctrl_22 = wx.TextCtrl(self.notebook_1_pane_2, wx.ID_ANY, "")
        self.button_3 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"参照..."))
        self.label_37 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"elGroup名"))
        self.list_box_5 = wx.ListBox(self.notebook_1_pane_2, wx.ID_ANY, choices=[])
        self.button_17 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"選択>>"))
        self.button_18 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"<<戻す"))
        self.label_43 = wx.StaticText(self.notebook_1_pane_2, wx.ID_ANY, _(u"物性値を定義するgroup"))
        self.list_box_6 = wx.ListBox(self.notebook_1_pane_2, wx.ID_ANY, choices=[])
        self.sizer_82_staticbox = wx.StaticBox(self.notebook_1_pane_2, wx.ID_ANY, _(u"材料を設定"))
        self.button_20 = wx.Button(self.notebook_1_pane_2, wx.ID_ANY, _(u"設定"))
        self.notebook_1_pane_3 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_5 = wx.StaticText(self.notebook_1_pane_3, wx.ID_ANY, _(u"境界条件の設定"))
        self.notebook_1_pane_4 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_41 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"設定するgroup名を選択する"))
        self.label_42 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"group名:"))
        self.list_box_14 = wx.ListBox(self.notebook_1_pane_4, wx.ID_ANY, choices=[])
        self.button_42 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"選択>>"))
        self.button_43 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"<<戻す"))
        self.label_67 = wx.StaticText(self.notebook_1_pane_4, wx.ID_ANY, _(u"設定するgroup"))
        self.list_box_15 = wx.ListBox(self.notebook_1_pane_4, wx.ID_ANY, choices=[])
        self.button_44 = wx.Button(self.notebook_1_pane_4, wx.ID_ANY, _(u"設定"))
        self.sizer_155_staticbox = wx.StaticBox(self.notebook_1_pane_4, wx.ID_ANY, _("parentName"))
        self.notebook_1_pane_5 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_6 = wx.StaticText(self.notebook_1_pane_5, wx.ID_ANY, _(u"node group名:"))
        self.checkbox_1 = wx.CheckBox(self.notebook_1_pane_5, wx.ID_ANY, _("x"))
        self.text_ctrl_2 = wx.TextCtrl(self.notebook_1_pane_5, wx.ID_ANY, "")
        self.checkbox_2 = wx.CheckBox(self.notebook_1_pane_5, wx.ID_ANY, _("y"))
        self.text_ctrl_3 = wx.TextCtrl(self.notebook_1_pane_5, wx.ID_ANY, "")
        self.checkbox_3 = wx.CheckBox(self.notebook_1_pane_5, wx.ID_ANY, _("z"))
        self.text_ctrl_4 = wx.TextCtrl(self.notebook_1_pane_5, wx.ID_ANY, "")
        self.button_9 = wx.Button(self.notebook_1_pane_5, wx.ID_ANY, _(u"設定"))
        self.sizer_14_staticbox = wx.StaticBox(self.notebook_1_pane_5, wx.ID_ANY, _(u"節点拘束・変位"))
        self.notebook_1_pane_6 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_8 = wx.StaticText(self.notebook_1_pane_6, wx.ID_ANY, _(u"node group名:"))
        self.label_7 = wx.StaticText(self.notebook_1_pane_6, wx.ID_ANY, _("x"))
        self.text_ctrl_5 = wx.TextCtrl(self.notebook_1_pane_6, wx.ID_ANY, "")
        self.label_9 = wx.StaticText(self.notebook_1_pane_6, wx.ID_ANY, _("y"))
        self.text_ctrl_6 = wx.TextCtrl(self.notebook_1_pane_6, wx.ID_ANY, "")
        self.label_10 = wx.StaticText(self.notebook_1_pane_6, wx.ID_ANY, _("z"))
        self.text_ctrl_7 = wx.TextCtrl(self.notebook_1_pane_6, wx.ID_ANY, "")
        self.button_10 = wx.Button(self.notebook_1_pane_6, wx.ID_ANY, _(u"設定"))
        self.sizer_19_staticbox = wx.StaticBox(self.notebook_1_pane_6, wx.ID_ANY, _(u"集中荷重"))
        self.notebook_1_pane_7 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_11 = wx.StaticText(self.notebook_1_pane_7, wx.ID_ANY, _(u"surface group名:"))
        self.label_12 = wx.StaticText(self.notebook_1_pane_7, wx.ID_ANY, _(u"圧力"))
        self.text_ctrl_8 = wx.TextCtrl(self.notebook_1_pane_7, wx.ID_ANY, "")
        self.button_11 = wx.Button(self.notebook_1_pane_7, wx.ID_ANY, _(u"設定"))
        self.sizer_24_staticbox = wx.StaticBox(self.notebook_1_pane_7, wx.ID_ANY, _(u"圧力（面荷重）"))
        self.notebook_1_pane_8 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_13 = wx.StaticText(self.notebook_1_pane_8, wx.ID_ANY, _(u"名前:"))
        self.label_14 = wx.StaticText(self.notebook_1_pane_8, wx.ID_ANY, _("BX"))
        self.text_ctrl_9 = wx.TextCtrl(self.notebook_1_pane_8, wx.ID_ANY, "")
        self.label_15 = wx.StaticText(self.notebook_1_pane_8, wx.ID_ANY, _("BY"))
        self.text_ctrl_10 = wx.TextCtrl(self.notebook_1_pane_8, wx.ID_ANY, "")
        self.label_16 = wx.StaticText(self.notebook_1_pane_8, wx.ID_ANY, _("BZ"))
        self.text_ctrl_11 = wx.TextCtrl(self.notebook_1_pane_8, wx.ID_ANY, "")
        self.button_12 = wx.Button(self.notebook_1_pane_8, wx.ID_ANY, _(u"設定"))
        self.sizer_27_staticbox = wx.StaticBox(self.notebook_1_pane_8, wx.ID_ANY, _(u"体積力"))
        self.notebook_1_pane_9 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_17 = wx.StaticText(self.notebook_1_pane_9, wx.ID_ANY, _("x"))
        self.text_ctrl_12 = wx.TextCtrl(self.notebook_1_pane_9, wx.ID_ANY, "")
        self.label_18 = wx.StaticText(self.notebook_1_pane_9, wx.ID_ANY, _("y"))
        self.text_ctrl_13 = wx.TextCtrl(self.notebook_1_pane_9, wx.ID_ANY, "")
        self.label_19 = wx.StaticText(self.notebook_1_pane_9, wx.ID_ANY, _("z"))
        self.text_ctrl_14 = wx.TextCtrl(self.notebook_1_pane_9, wx.ID_ANY, "")
        self.button_13 = wx.Button(self.notebook_1_pane_9, wx.ID_ANY, _(u"設定"))
        self.sizer_33_staticbox = wx.StaticBox(self.notebook_1_pane_9, wx.ID_ANY, _(u"重力"))
        self.notebook_1_pane_10 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_20 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _(u"名前:"))
        self.label_21 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _(u"角速度"))
        self.text_ctrl_15 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.label_22 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _(u"位置ベクトル（中心座標）"))
        self.label_23 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _("x"))
        self.text_ctrl_16 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.label_24 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _("y"))
        self.text_ctrl_17 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.label_25 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _("z"))
        self.text_ctrl_18 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.label_26 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _(u"方向ベクトル（回転軸）"))
        self.label_27 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _("x"))
        self.text_ctrl_19 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.label_28 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _("y"))
        self.text_ctrl_20 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.label_29 = wx.StaticText(self.notebook_1_pane_10, wx.ID_ANY, _("z"))
        self.text_ctrl_21 = wx.TextCtrl(self.notebook_1_pane_10, wx.ID_ANY, "")
        self.button_14 = wx.Button(self.notebook_1_pane_10, wx.ID_ANY, _(u"設定"))
        self.sizer_38_staticbox = wx.StaticBox(self.notebook_1_pane_10, wx.ID_ANY, _(u"遠心力"))
        self.notebook_1_pane_11 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_39 = wx.StaticText(self.notebook_1_pane_11, wx.ID_ANY, _(u"参照温度"))
        self.text_ctrl_26 = wx.TextCtrl(self.notebook_1_pane_11, wx.ID_ANY, "")
        self.label_40 = wx.StaticText(self.notebook_1_pane_11, wx.ID_ANY, _(u"設定温度"))
        self.text_ctrl_27 = wx.TextCtrl(self.notebook_1_pane_11, wx.ID_ANY, "")
        self.button_41 = wx.Button(self.notebook_1_pane_11, wx.ID_ANY, _(u"設定"))
        self.sizer_90_staticbox = wx.StaticBox(self.notebook_1_pane_11, wx.ID_ANY, _(u"温度の設定"))
        self.notebook_1_pane_12 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.button_36 = wx.Button(self.notebook_1_pane_12, wx.ID_ANY, _(u"結果の再構築"))
        self.label_87 = wx.StaticText(self.notebook_1_pane_12, wx.ID_ANY, _(u"rmergeを実行（分散している結果dataを結合）"))
        self.sizer_189_staticbox = wx.StaticBox(self.notebook_1_pane_12, wx.ID_ANY, _(u"並列処理の場合"))
        self.button_37 = wx.Button(self.notebook_1_pane_12, wx.ID_ANY, _(u"REVOCAP起動"))
        self.sizer_148_staticbox = wx.StaticBox(self.notebook_1_pane_12, wx.ID_ANY, _(u"REVOCAPによる可視化"))
        self.button_38 = wx.Button(self.notebook_1_pane_12, wx.ID_ANY, _(u"データ変換"))
        self.label_64 = wx.StaticText(self.notebook_1_pane_12, wx.ID_ANY, _(u"paraViewが読めるdataに変換"))
        self.button_40 = wx.Button(self.notebook_1_pane_12, wx.ID_ANY, _(u"主応力追加"))
        self.label_66 = wx.StaticText(self.notebook_1_pane_12, wx.ID_ANY, _(u"変換したデータに主応力を追加"))
        self.button_39 = wx.Button(self.notebook_1_pane_12, wx.ID_ANY, _(u"paraView起動"))
        self.label_65 = wx.StaticText(self.notebook_1_pane_12, wx.ID_ANY, _(u"結果（inpファイル）を可視化"))
        self.sizer_149_staticbox = wx.StaticBox(self.notebook_1_pane_12, wx.ID_ANY, _(u"paraViewによる可視化"))
        self.notebook_1_pane_13 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_36 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _("METHOD"))
        self.combo_box_11 = wx.ComboBox(self.notebook_1_pane_13, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_78 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"解法"))
        self.label_79 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _("PRECOND"))
        self.text_ctrl_39 = wx.TextCtrl(self.notebook_1_pane_13, wx.ID_ANY, "")
        self.label_80 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"前処理手法"))
        self.label_81 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _("NIER"))
        self.text_ctrl_40 = wx.TextCtrl(self.notebook_1_pane_13, wx.ID_ANY, "")
        self.label_82 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"反復回数"))
        self.label_83 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _("RESID"))
        self.text_ctrl_41 = wx.TextCtrl(self.notebook_1_pane_13, wx.ID_ANY, "")
        self.label_84 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"打ち切り誤差"))
        self.sizer_177_staticbox = wx.StaticBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"計算方法"))
        self.button_16 = wx.Button(self.notebook_1_pane_13, wx.ID_ANY, _(u"設定"))
        self.sizer_138_staticbox = wx.StaticBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"線形solverの設定"))
        self.checkbox_6 = wx.CheckBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"並列計算する"))
        self.label_86 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"cpu数:"))
        self.text_ctrl_42 = wx.TextCtrl(self.notebook_1_pane_13, wx.ID_ANY, "")
        self.button_35 = wx.Button(self.notebook_1_pane_13, wx.ID_ANY, _(u"メッシュ分割"))
        self.sizer_182_staticbox = wx.StaticBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"並列処理の設定"))
        self.checkbox_4 = wx.CheckBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"iterationLog出力"))
        self.checkbox_5 = wx.CheckBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"timeLog出力"))
        self.label_101 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"結果出力頻度:"))
        self.text_ctrl_43 = wx.TextCtrl(self.notebook_1_pane_13, wx.ID_ANY, "")
        self.label_57 = wx.StaticText(self.notebook_1_pane_13, wx.ID_ANY, _(u"実行ファイル名:"))
        self.text_ctrl_25 = wx.TextCtrl(self.notebook_1_pane_13, wx.ID_ANY, "")
        self.button_26 = wx.Button(self.notebook_1_pane_13, wx.ID_ANY, _(u"FrontISTR実行"))
        self.sizer_127_staticbox = wx.StaticBox(self.notebook_1_pane_13, wx.ID_ANY, _(u"実行（シングル、並列処理共）"))
        self.notebook_1_pane_14 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_51 = wx.StaticText(self.notebook_1_pane_14, wx.ID_ANY, _("EasyISTR"))
        self.label_56 = wx.StaticText(self.notebook_1_pane_14, wx.ID_ANY, _("ver 1.0"))
        self.label_52 = wx.StaticText(self.notebook_1_pane_14, wx.ID_ANY, _("for FrontISTR"))
        self.text_ctrl_23 = wx.TextCtrl(self.notebook_1_pane_14, wx.ID_ANY, "")
        self.button_24 = wx.Button(self.notebook_1_pane_14, wx.ID_ANY, _(u"参照..."))
        self.sizer_119_staticbox = wx.StaticBox(self.notebook_1_pane_14, wx.ID_ANY, _(u"作業用folder"))
        self.label_53 = wx.StaticText(self.notebook_1_pane_14, wx.ID_ANY, _(u"全cntファイル"))
        self.list_box_9 = wx.ListBox(self.notebook_1_pane_14, wx.ID_ANY, choices=[])
        self.button_25 = wx.Button(self.notebook_1_pane_14, wx.ID_ANY, _(u"選択>>"))
        self.label_54 = wx.StaticText(self.notebook_1_pane_14, wx.ID_ANY, _(u"使用するcntファイル"))
        self.text_ctrl_24 = wx.TextCtrl(self.notebook_1_pane_14, wx.ID_ANY, "")
        self.button_27 = wx.Button(self.notebook_1_pane_14, wx.ID_ANY, _(u"cntファイル置き換え"))
        self.label_55 = wx.StaticText(self.notebook_1_pane_14, wx.ID_ANY, _(u"現在のFistrmodel.cntを\n置き換えて使用する"))
        self.sizer_121_staticbox = wx.StaticBox(self.notebook_1_pane_14, wx.ID_ANY, _(u"制御ファイル選択"))
        self.notebook_1_pane_15 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_44 = wx.StaticText(self.notebook_1_pane_15, wx.ID_ANY, _(u"elGroup名:"))
        self.label_45 = wx.StaticText(self.notebook_1_pane_15, wx.ID_ANY, _(u"材料名:"))
        self.combo_box_5 = wx.ComboBox(self.notebook_1_pane_15, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_46 = wx.StaticText(self.notebook_1_pane_15, wx.ID_ANY, _(u"材料モデル"))
        self.combo_box_6 = wx.ComboBox(self.notebook_1_pane_15, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_47 = wx.StaticText(self.notebook_1_pane_15, wx.ID_ANY, _(u"降伏条件/タイプ"))
        self.combo_box_7 = wx.ComboBox(self.notebook_1_pane_15, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_48 = wx.StaticText(self.notebook_1_pane_15, wx.ID_ANY, _(u"硬化則"))
        self.combo_box_8 = wx.ComboBox(self.notebook_1_pane_15, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_85 = wx.StaticText(self.notebook_1_pane_15, wx.ID_ANY, _(u"塑性(Plastic)data"))
        self.button_34 = wx.Button(self.notebook_1_pane_15, wx.ID_ANY, _(u"SS_data\n作成・編集"))
        self.sizer_102_staticbox = wx.StaticBox(self.notebook_1_pane_15, wx.ID_ANY, _(u"材料物性値"))
        self.button_19 = wx.Button(self.notebook_1_pane_15, wx.ID_ANY, _(u"設定"))
        self.sizer_100_staticbox = wx.StaticBox(self.notebook_1_pane_15, wx.ID_ANY, _(u"材料物性値の設定"))
        self.notebook_1_pane_16 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_31 = wx.StaticText(self.notebook_1_pane_16, wx.ID_ANY, _(u"contactのAlgorithmと設定"))
        self.label_34 = wx.StaticText(self.notebook_1_pane_16, wx.ID_ANY, _("Algorithm"))
        self.combo_box_2 = wx.ComboBox(self.notebook_1_pane_16, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.sizer_77_staticbox = wx.StaticBox(self.notebook_1_pane_16, wx.ID_ANY, _("Algorithm"))
        self.label_33 = wx.StaticText(self.notebook_1_pane_16, wx.ID_ANY, _(u"contact定義を\n「追加>>」ボタンで追加。\n「<<削除」ボタンで削除。"), style=wx.ST_NO_AUTORESIZE)
        self.button_5 = wx.Button(self.notebook_1_pane_16, wx.ID_ANY, _(u"追加>>"))
        self.button_7 = wx.Button(self.notebook_1_pane_16, wx.ID_ANY, _(u"<<削除"))
        self.label_32 = wx.StaticText(self.notebook_1_pane_16, wx.ID_ANY, _(u"設定するcontactPair名"))
        self.list_box_1 = wx.ListBox(self.notebook_1_pane_16, wx.ID_ANY, choices=[])
        self.sizer_61_staticbox = wx.StaticBox(self.notebook_1_pane_16, wx.ID_ANY, _(u"設定"))
        self.button_8 = wx.Button(self.notebook_1_pane_16, wx.ID_ANY, _(u"設定"))
        self.sizer_60_staticbox = wx.StaticBox(self.notebook_1_pane_16, wx.ID_ANY, _(u"contactの定義"))
        self.notebook_1_pane_17 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_35 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _(u"contactPair名"))
        self.text_ctrl_28 = wx.TextCtrl(self.notebook_1_pane_17, wx.ID_ANY, "")
        self.label_38 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _("slave"))
        self.combo_box_4 = wx.ComboBox(self.notebook_1_pane_17, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_49 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _("master"))
        self.combo_box_9 = wx.ComboBox(self.notebook_1_pane_17, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.sizer_91_staticbox = wx.StaticBox(self.notebook_1_pane_17, wx.ID_ANY, _("contact_pair"))
        self.label_50 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _("INTERACTION"))
        self.combo_box_10 = wx.ComboBox(self.notebook_1_pane_17, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_60 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _("NTOL"))
        self.text_ctrl_31 = wx.TextCtrl(self.notebook_1_pane_17, wx.ID_ANY, "")
        self.label_61 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _("TTOL"))
        self.text_ctrl_32 = wx.TextCtrl(self.notebook_1_pane_17, wx.ID_ANY, "")
        self.label_58 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _(u"摩擦係数"))
        self.text_ctrl_29 = wx.TextCtrl(self.notebook_1_pane_17, wx.ID_ANY, "")
        self.label_59 = wx.StaticText(self.notebook_1_pane_17, wx.ID_ANY, _("factor"))
        self.text_ctrl_30 = wx.TextCtrl(self.notebook_1_pane_17, wx.ID_ANY, "")
        self.sizer_114_staticbox = wx.StaticBox(self.notebook_1_pane_17, wx.ID_ANY, _("contact condition"))
        self.button_15 = wx.Button(self.notebook_1_pane_17, wx.ID_ANY, _(u"設定"))
        self.notebook_1_pane_18 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_68 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _(u"ステップ解析の条件設定"))
        self.label_69 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _("TYPE"))
        self.combo_box_3 = wx.ComboBox(self.notebook_1_pane_18, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_70 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _("CONVERG"))
        self.text_ctrl_34 = wx.TextCtrl(self.notebook_1_pane_18, wx.ID_ANY, "")
        self.label_71 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _("SUBSTEPS"))
        self.text_ctrl_35 = wx.TextCtrl(self.notebook_1_pane_18, wx.ID_ANY, "")
        self.label_72 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _("MAXITER"))
        self.text_ctrl_36 = wx.TextCtrl(self.notebook_1_pane_18, wx.ID_ANY, "")
        self.label_73 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _("DTIME, ETIME"))
        self.text_ctrl_37 = wx.TextCtrl(self.notebook_1_pane_18, wx.ID_ANY, "")
        self.text_ctrl_38 = wx.TextCtrl(self.notebook_1_pane_18, wx.ID_ANY, "")
        self.sizer_146_staticbox = wx.StaticBox(self.notebook_1_pane_18, wx.ID_ANY, _(u"収束条件"))
        self.label_75 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _(u"現在の境界条件"))
        self.list_box_3 = wx.ListBox(self.notebook_1_pane_18, wx.ID_ANY, choices=[])
        self.button_23 = wx.Button(self.notebook_1_pane_18, wx.ID_ANY, _(u"選択>>"))
        self.button_31 = wx.Button(self.notebook_1_pane_18, wx.ID_ANY, _(u"<<戻す"))
        self.label_76 = wx.StaticText(self.notebook_1_pane_18, wx.ID_ANY, _(u"設定する境界条件"))
        self.list_box_4 = wx.ListBox(self.notebook_1_pane_18, wx.ID_ANY, choices=[])
        self.sizer_166_staticbox = wx.StaticBox(self.notebook_1_pane_18, wx.ID_ANY, _(u"step解析する境界条件"))
        self.button_32 = wx.Button(self.notebook_1_pane_18, wx.ID_ANY, _(u"設定"))
        self.notebook_1_pane_19 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.label_74 = wx.StaticText(self.notebook_1_pane_19, wx.ID_ANY, _(u"項目"))
        self.list_box_2 = wx.ListBox(self.notebook_1_pane_19, wx.ID_ANY, choices=[])
        self.button_21 = wx.Button(self.notebook_1_pane_19, wx.ID_ANY, _(u"選択>>"))
        self.button_22 = wx.Button(self.notebook_1_pane_19, wx.ID_ANY, _(u"<<戻す"))
        self.label_77 = wx.StaticText(self.notebook_1_pane_19, wx.ID_ANY, _(u"出力項目"))
        self.list_box_7 = wx.ListBox(self.notebook_1_pane_19, wx.ID_ANY, choices=[])
        self.button_33 = wx.Button(self.notebook_1_pane_19, wx.ID_ANY, _(u"設定"))
        self.sizer_136_staticbox = wx.StaticBox(self.notebook_1_pane_19, wx.ID_ANY, _(u"出力の設定"))
        self.notebook_1_pane_20 = wx.Panel(self.notebook_1, wx.ID_ANY)
        self.panel_solution = wx.Panel(self.notebook_1_pane_20, wx.ID_ANY)
        self.label_88 = wx.StaticText(self.panel_solution, wx.ID_ANY, _(u"解析の種類を設定"))
        self.combo_box_12 = wx.ComboBox(self.panel_solution, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.button_46 = wx.Button(self.panel_solution, wx.ID_ANY, _(u"設定"))
        self.sizer_193_staticbox = wx.StaticBox(self.panel_solution, wx.ID_ANY, _(u"解析の種類"))
        self.panel_static = wx.Panel(self.notebook_1_pane_20, wx.ID_ANY)
        self.label_99 = wx.StaticText(self.panel_static, wx.ID_ANY, _("label_99"))
        self.panel_dynamic = wx.Panel(self.notebook_1_pane_20, wx.ID_ANY)
        self.label_89 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"動解析の設定"))
        self.label_90 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _("type"))
        self.combo_box_16 = wx.ComboBox(self.panel_dynamic, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_91 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"運動方程式解法"))
        self.combo_box_13 = wx.ComboBox(self.panel_dynamic, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_92 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"解析の種類"))
        self.combo_box_14 = wx.ComboBox(self.panel_dynamic, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_93 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"開始・終了時間"))
        self.text_ctrl_45 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.text_ctrl_46 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_94 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"全step数"))
        self.text_ctrl_47 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_95 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"時間増分"))
        self.text_ctrl_48 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_96 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"file出力間隔"))
        self.text_ctrl_49 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_97 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _("restart file"))
        self.sizer_197_staticbox = wx.StaticBox(self.panel_dynamic, wx.ID_ANY, _(u"設定内容"))
        self.label_98 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _("ganma, beta"))
        self.text_ctrl_50 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.text_ctrl_51 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_103 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"Newmark-β法"))
        self.label_100 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _("Rm, Rk"))
        self.text_ctrl_44 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.text_ctrl_52 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_104 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"減衰"))
        self.sizer_204_staticbox = wx.StaticBox(self.panel_dynamic, wx.ID_ANY, _(u"パラメータ"))
        self.label_102 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"group名"))
        self.combo_box_15 = wx.ComboBox(self.panel_dynamic, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN)
        self.label_105 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"出力間隔"))
        self.text_ctrl_53 = wx.TextCtrl(self.panel_dynamic, wx.ID_ANY, "")
        self.label_106 = wx.StaticText(self.panel_dynamic, wx.ID_ANY, _(u"モニタ内容"))
        self.checkbox_7 = wx.CheckBox(self.panel_dynamic, wx.ID_ANY, _(u"変位　"))
        self.checkbox_8 = wx.CheckBox(self.panel_dynamic, wx.ID_ANY, _(u"速度　"))
        self.checkbox_9 = wx.CheckBox(self.panel_dynamic, wx.ID_ANY, _(u"加速度"))
        self.checkbox_10 = wx.CheckBox(self.panel_dynamic, wx.ID_ANY, _(u"反力　"))
        self.checkbox_11 = wx.CheckBox(self.panel_dynamic, wx.ID_ANY, _(u"ひずみ"))
        self.checkbox_12 = wx.CheckBox(self.panel_dynamic, wx.ID_ANY, _(u"応力　"))
        self.sizer_207_staticbox = wx.StaticBox(self.panel_dynamic, wx.ID_ANY, _(u"モニタ出力指定"))
        self.button_45 = wx.Button(self.panel_dynamic, wx.ID_ANY, _(u"設定"))
        self.button_28 = wx.Button(self, wx.ID_ANY, _(u"folder開く"))
        self.button_29 = wx.Button(self, wx.ID_ANY, _(u"制御file編集"))
        self.button_30 = wx.Button(self, wx.ID_ANY, _(u"メッシュfile編集"))
        self.button_6 = wx.Button(self, wx.ID_ANY, _(u"閉じる"))

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.onSelectItem, self.tree_ctrl_1)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.onSelectItem, self.tree_ctrl_1)
        self.Bind(wx.EVT_BUTTON, self.onSetMeshFile, self.button_1)
        self.Bind(wx.EVT_BUTTON, self.onMeshConvert, self.button_2)
        self.Bind(wx.EVT_BUTTON, self.onReadMeshData, self.button_4)
        self.Bind(wx.EVT_BUTTON, self.onConfirmModelAtParaview, self.button_47)
        self.Bind(wx.EVT_BUTTON, self.onSetMaterialDB, self.button_3)
        self.Bind(wx.EVT_BUTTON, self.onSetMaterialGroup, self.button_17)
        self.Bind(wx.EVT_BUTTON, self.onReturnMaterialGroup, self.button_18)
        self.Bind(wx.EVT_BUTTON, self.onApplyMaterial, self.button_20)
        self.Bind(wx.EVT_BUTTON, self.onSetBCGroup, self.button_42)
        self.Bind(wx.EVT_BUTTON, self.onReturnBCGroup, self.button_43)
        self.Bind(wx.EVT_BUTTON, self.onApplyBCGoup, self.button_44)
        self.Bind(wx.EVT_BUTTON, self.onApplyBoundaryCont, self.button_9)
        self.Bind(wx.EVT_BUTTON, self.onApplyCloadCont, self.button_10)
        self.Bind(wx.EVT_BUTTON, self.onApplyDloadCont, self.button_11)
        self.Bind(wx.EVT_BUTTON, self.onApplyVloadCont, self.button_12)
        self.Bind(wx.EVT_BUTTON, self.onApplyGrav, self.button_13)
        self.Bind(wx.EVT_BUTTON, self.onApplyCentCont, self.button_14)
        self.Bind(wx.EVT_BUTTON, self.onApplyTemperature, self.button_41)
        self.Bind(wx.EVT_BUTTON, self.onResultMerge, self.button_36)
        self.Bind(wx.EVT_BUTTON, self.onRunRevocap, self.button_37)
        self.Bind(wx.EVT_BUTTON, self.onConvertResultData, self.button_38)
        self.Bind(wx.EVT_BUTTON, self.onAddPrincipalStress, self.button_40)
        self.Bind(wx.EVT_BUTTON, self.onRunParaView, self.button_39)
        self.Bind(wx.EVT_BUTTON, self.onApplySolverCont, self.button_16)
        self.Bind(wx.EVT_CHECKBOX, self.onSwitchParallel, self.checkbox_6)
        self.Bind(wx.EVT_BUTTON, self.onCreateMultiJob, self.button_35)
        self.Bind(wx.EVT_BUTTON, self.onRunFrontIstr, self.button_26)
        self.Bind(wx.EVT_TEXT_ENTER, self.onSetWorkFolder, self.text_ctrl_23)
        self.Bind(wx.EVT_BUTTON, self.onSelectWorkFolder, self.button_24)
        self.Bind(wx.EVT_BUTTON, self.onSetUseCntFile, self.button_25)
        self.Bind(wx.EVT_BUTTON, self.onCopyUseCntFile, self.button_27)
        self.Bind(wx.EVT_COMBOBOX, self.onSetEachMaterialModel, self.combo_box_6)
        self.Bind(wx.EVT_BUTTON, self.onEditSsData, self.button_34)
        self.Bind(wx.EVT_BUTTON, self.onApplyEachMaterial, self.button_19)
        self.Bind(wx.EVT_BUTTON, self.onAddContactCondition, self.button_5)
        self.Bind(wx.EVT_BUTTON, self.onDeleteContactCondition, self.button_7)
        self.Bind(wx.EVT_BUTTON, self.onApplyContact, self.button_8)
        self.Bind(wx.EVT_BUTTON, self.onApplyContactCont, self.button_15)
        self.Bind(wx.EVT_BUTTON, self.onSetBCGroupForStep, self.button_23)
        self.Bind(wx.EVT_BUTTON, self.onReturnBCGroupForStep, self.button_31)
        self.Bind(wx.EVT_BUTTON, self.onApplyStepCont, self.button_32)
        self.Bind(wx.EVT_BUTTON, self.onSetOutputItems, self.button_21)
        self.Bind(wx.EVT_BUTTON, self.onReturnOutputItems, self.button_22)
        self.Bind(wx.EVT_BUTTON, self.onApplyOutputCont, self.button_33)
        self.Bind(wx.EVT_BUTTON, self.onApplySolutionType, self.button_46)
        self.Bind(wx.EVT_BUTTON, self.onApplyDynamicSolution, self.button_45)
        self.Bind(wx.EVT_BUTTON, self.onOpenWorkFolder, self.button_28)
        self.Bind(wx.EVT_BUTTON, self.onEditCntFile, self.button_29)
        self.Bind(wx.EVT_BUTTON, self.onEditMshFile, self.button_30)
        self.Bind(wx.EVT_BUTTON, self.onClose, self.button_6)
        # end wxGlade

        initialize()
        self.subInitialize()


    def __set_properties(self):
        # begin wxGlade: MyFrame.__set_properties
        self.SetTitle(_("frame_1"))
        self.tree_ctrl_1.SetMinSize((200, 400))
        self.text_ctrl_1.SetMinSize((160, 28))
        self.button_1.SetMinSize((60, 30))
        self.text_ctrl_22.SetMinSize((180, 28))
        self.button_3.SetMinSize((60, 30))
        self.button_17.SetMinSize((60, 30))
        self.button_18.SetMinSize((60, 30))
        self.button_42.SetMinSize((60, 30))
        self.button_43.SetMinSize((60, 30))
        self.text_ctrl_2.SetMinSize((100, 28))
        self.text_ctrl_3.SetMinSize((100, 28))
        self.text_ctrl_4.SetMinSize((100, 28))
        self.text_ctrl_5.SetMinSize((100, 28))
        self.text_ctrl_6.SetMinSize((100, 28))
        self.text_ctrl_7.SetMinSize((100, 28))
        self.text_ctrl_8.SetMinSize((100, 28))
        self.text_ctrl_9.SetMinSize((100, 28))
        self.text_ctrl_10.SetMinSize((100, 28))
        self.text_ctrl_11.SetMinSize((100, 28))
        self.text_ctrl_12.SetMinSize((100, 28))
        self.text_ctrl_13.SetMinSize((100, 28))
        self.text_ctrl_14.SetMinSize((100, 28))
        self.text_ctrl_15.SetMinSize((100, 28))
        self.text_ctrl_16.SetMinSize((100, 28))
        self.text_ctrl_17.SetMinSize((100, 28))
        self.text_ctrl_18.SetMinSize((100, 28))
        self.text_ctrl_19.SetMinSize((100, 28))
        self.text_ctrl_20.SetMinSize((100, 28))
        self.text_ctrl_21.SetMinSize((100, 28))
        self.label_36.SetMinSize((100, 18))
        self.combo_box_11.SetMinSize((150, 30))
        self.label_79.SetMinSize((100, 18))
        self.text_ctrl_39.SetMinSize((150, 28))
        self.label_81.SetMinSize((100, 18))
        self.text_ctrl_40.SetMinSize((150, 28))
        self.label_83.SetMinSize((100, 18))
        self.text_ctrl_41.SetMinSize((150, 28))
        self.text_ctrl_43.SetMinSize((60, 28))
        self.text_ctrl_25.SetMinSize((150, 28))
        self.button_26.SetMinSize((110, 45))
        self.notebook_1_pane_13.SetMinSize((718, 512))
        self.label_51.SetFont(wx.Font(15, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0, "VL ゴシック"))
        self.button_24.SetMinSize((60, 30))
        self.list_box_9.SetMinSize((154, 130))
        self.button_25.SetMinSize((60, 30))
        self.label_46.SetMinSize((100, 18))
        self.combo_box_6.SetMinSize((150, 30))
        self.label_47.SetMinSize((100, 18))
        self.combo_box_7.SetMinSize((150, 30))
        self.label_48.SetMinSize((100, 18))
        self.combo_box_8.SetMinSize((150, 30))
        self.combo_box_2.SetMinSize((250, 30))
        self.button_5.SetMinSize((60, 30))
        self.button_7.SetMinSize((60, 30))
        self.text_ctrl_28.SetMinSize((150, 28))
        self.label_38.SetMinSize((100, 18))
        self.label_49.SetMinSize((100, 18))
        self.label_50.SetMinSize((100, 18))
        self.combo_box_10.SetMinSize((150, 30))
        self.label_60.SetMinSize((100, 18))
        self.text_ctrl_31.SetMinSize((150, 28))
        self.label_61.SetMinSize((100, 18))
        self.text_ctrl_32.SetMinSize((150, 28))
        self.label_58.SetMinSize((100, 18))
        self.text_ctrl_29.SetMinSize((150, 28))
        self.label_59.SetMinSize((100, 18))
        self.text_ctrl_30.SetMinSize((150, 28))
        self.label_69.SetMinSize((100, 18))
        self.combo_box_3.SetMinSize((150, 30))
        self.label_70.SetMinSize((100, 18))
        self.text_ctrl_34.SetMinSize((150, 28))
        self.label_71.SetMinSize((100, 18))
        self.text_ctrl_35.SetMinSize((150, 28))
        self.label_72.SetMinSize((100, 18))
        self.text_ctrl_36.SetMinSize((150, 28))
        self.label_73.SetMinSize((100, 18))
        self.text_ctrl_37.SetMinSize((150, 28))
        self.text_ctrl_38.SetMinSize((150, 28))
        self.button_23.SetMinSize((60, 30))
        self.button_31.SetMinSize((60, 30))
        self.button_21.SetMinSize((60, 30))
        self.button_22.SetMinSize((60, 30))
        self.panel_solution.Hide()
        self.panel_static.Hide()
        self.label_90.SetMinSize((100, 18))
        self.combo_box_16.SetMinSize((190, 30))
        self.label_91.SetMinSize((100, 18))
        self.combo_box_13.SetMinSize((190, 30))
        self.label_92.SetMinSize((100, 18))
        self.combo_box_14.SetMinSize((190, 30))
        self.label_93.SetMinSize((100, 18))
        self.text_ctrl_45.SetMinSize((100, 28))
        self.text_ctrl_46.SetMinSize((100, 28))
        self.label_94.SetMinSize((100, 18))
        self.text_ctrl_47.SetMinSize((100, 28))
        self.label_95.SetMinSize((100, 18))
        self.text_ctrl_48.SetMinSize((100, 28))
        self.label_96.SetMinSize((100, 18))
        self.text_ctrl_49.SetMinSize((100, 28))
        self.label_98.SetMinSize((100, 18))
        self.text_ctrl_50.SetMinSize((100, 28))
        self.text_ctrl_51.SetMinSize((100, 28))
        self.label_100.SetMinSize((100, 18))
        self.text_ctrl_44.SetMinSize((100, 28))
        self.text_ctrl_52.SetMinSize((100, 28))
        self.combo_box_15.SetMinSize((128, 30))
        self.text_ctrl_53.SetMinSize((100, 28))
        self.label_106.SetMinSize((80, 18))
        self.checkbox_10.SetMinSize((65, 24))
        # end wxGlade

        self.SetTitle(_(u"easyIstr"))


    def __do_layout(self):
        global sizer_194, sizer_1, sizer_3, sizer_191, sizer_192, sizer_209, sizer_195
        # begin wxGlade: MyFrame.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_56 = wx.BoxSizer(wx.VERTICAL)
        sizer_57 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_128 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_129 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_130 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_5 = wx.BoxSizer(wx.VERTICAL)
        sizer_191 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_194 = wx.BoxSizer(wx.VERTICAL)
        sizer_195 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_207_staticbox.Lower()
        sizer_207 = wx.StaticBoxSizer(self.sizer_207_staticbox, wx.VERTICAL)
        sizer_211 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_210 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_208 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_204_staticbox.Lower()
        sizer_204 = wx.StaticBoxSizer(self.sizer_204_staticbox, wx.VERTICAL)
        sizer_206 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_205 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_197_staticbox.Lower()
        sizer_197 = wx.StaticBoxSizer(self.sizer_197_staticbox, wx.VERTICAL)
        sizer_203 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_202 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_201 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_200 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_199 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_198 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_196 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_209 = wx.BoxSizer(wx.VERTICAL)
        sizer_192 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_193_staticbox.Lower()
        sizer_193 = wx.StaticBoxSizer(self.sizer_193_staticbox, wx.VERTICAL)
        sizer_137 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_136_staticbox.Lower()
        sizer_136 = wx.StaticBoxSizer(self.sizer_136_staticbox, wx.VERTICAL)
        sizer_165 = wx.BoxSizer(wx.VERTICAL)
        sizer_172 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_140 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_143 = wx.BoxSizer(wx.VERTICAL)
        sizer_142 = wx.BoxSizer(wx.VERTICAL)
        sizer_141 = wx.BoxSizer(wx.VERTICAL)
        sizer_144 = wx.BoxSizer(wx.VERTICAL)
        sizer_170 = wx.BoxSizer(wx.VERTICAL)
        sizer_171 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_166_staticbox.Lower()
        sizer_166 = wx.StaticBoxSizer(self.sizer_166_staticbox, wx.HORIZONTAL)
        sizer_169 = wx.BoxSizer(wx.VERTICAL)
        sizer_168 = wx.BoxSizer(wx.VERTICAL)
        sizer_167 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_146_staticbox.Lower()
        sizer_146 = wx.StaticBoxSizer(self.sizer_146_staticbox, wx.VERTICAL)
        sizer_164 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_163 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_162 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_154 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_147 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_145 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_80 = wx.BoxSizer(wx.VERTICAL)
        sizer_134 = wx.BoxSizer(wx.VERTICAL)
        sizer_135 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_114_staticbox.Lower()
        sizer_114 = wx.StaticBoxSizer(self.sizer_114_staticbox, wx.HORIZONTAL)
        sizer_115 = wx.BoxSizer(wx.VERTICAL)
        sizer_126 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_117 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_133 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_132 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_116 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_91_staticbox.Lower()
        sizer_91 = wx.StaticBoxSizer(self.sizer_91_staticbox, wx.HORIZONTAL)
        sizer_111 = wx.BoxSizer(wx.VERTICAL)
        sizer_113 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_112 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_89 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_59 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_60_staticbox.Lower()
        sizer_60 = wx.StaticBoxSizer(self.sizer_60_staticbox, wx.VERTICAL)
        sizer_78 = wx.BoxSizer(wx.VERTICAL)
        sizer_79 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_61_staticbox.Lower()
        sizer_61 = wx.StaticBoxSizer(self.sizer_61_staticbox, wx.HORIZONTAL)
        sizer_71 = wx.BoxSizer(wx.VERTICAL)
        sizer_76 = wx.BoxSizer(wx.VERTICAL)
        sizer_62 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_77_staticbox.Lower()
        sizer_77 = wx.StaticBoxSizer(self.sizer_77_staticbox, wx.HORIZONTAL)
        sizer_99 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_100_staticbox.Lower()
        sizer_100 = wx.StaticBoxSizer(self.sizer_100_staticbox, wx.VERTICAL)
        sizer_106 = wx.BoxSizer(wx.VERTICAL)
        sizer_107 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_102_staticbox.Lower()
        sizer_102 = wx.StaticBoxSizer(self.sizer_102_staticbox, wx.VERTICAL)
        sizer_179 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_181 = wx.BoxSizer(wx.VERTICAL)
        sizer_180 = wx.BoxSizer(wx.VERTICAL)
        sizer_105 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_104 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_103 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_101 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_118 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_121_staticbox.Lower()
        sizer_121 = wx.StaticBoxSizer(self.sizer_121_staticbox, wx.HORIZONTAL)
        sizer_124 = wx.BoxSizer(wx.VERTICAL)
        sizer_123 = wx.BoxSizer(wx.VERTICAL)
        sizer_122 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_119_staticbox.Lower()
        sizer_119 = wx.StaticBoxSizer(self.sizer_119_staticbox, wx.HORIZONTAL)
        sizer_120 = wx.BoxSizer(wx.VERTICAL)
        sizer_125 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_131 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_127_staticbox.Lower()
        sizer_127 = wx.StaticBoxSizer(self.sizer_127_staticbox, wx.VERTICAL)
        sizer_184 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_185 = wx.BoxSizer(wx.VERTICAL)
        sizer_187 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_186 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_182_staticbox.Lower()
        sizer_182 = wx.StaticBoxSizer(self.sizer_182_staticbox, wx.VERTICAL)
        sizer_183 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_138_staticbox.Lower()
        sizer_138 = wx.StaticBoxSizer(self.sizer_138_staticbox, wx.VERTICAL)
        sizer_178 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_177_staticbox.Lower()
        sizer_177 = wx.StaticBoxSizer(self.sizer_177_staticbox, wx.VERTICAL)
        sizer_176 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_175 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_174 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_173 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_110 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_149_staticbox.Lower()
        sizer_149 = wx.StaticBoxSizer(self.sizer_149_staticbox, wx.VERTICAL)
        sizer_151 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_152 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_150 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_148_staticbox.Lower()
        sizer_148 = wx.StaticBoxSizer(self.sizer_148_staticbox, wx.VERTICAL)
        self.sizer_189_staticbox.Lower()
        sizer_189 = wx.StaticBoxSizer(self.sizer_189_staticbox, wx.VERTICAL)
        sizer_190 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_58 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_90_staticbox.Lower()
        sizer_90 = wx.StaticBoxSizer(self.sizer_90_staticbox, wx.VERTICAL)
        sizer_94 = wx.BoxSizer(wx.VERTICAL)
        sizer_153 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_93 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_92 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_83 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_38_staticbox.Lower()
        sizer_38 = wx.StaticBoxSizer(self.sizer_38_staticbox, wx.VERTICAL)
        sizer_74 = wx.BoxSizer(wx.VERTICAL)
        sizer_75 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_41 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_46 = wx.BoxSizer(wx.VERTICAL)
        sizer_49 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_48 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_47 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_42 = wx.BoxSizer(wx.VERTICAL)
        sizer_45 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_44 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_43 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_40 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_39 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_37 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_33_staticbox.Lower()
        sizer_33 = wx.StaticBoxSizer(self.sizer_33_staticbox, wx.VERTICAL)
        sizer_72 = wx.BoxSizer(wx.VERTICAL)
        sizer_73 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_36 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_35 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_34 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_32 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_27_staticbox.Lower()
        sizer_27 = wx.StaticBoxSizer(self.sizer_27_staticbox, wx.VERTICAL)
        sizer_69 = wx.BoxSizer(wx.VERTICAL)
        sizer_70 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_28 = wx.BoxSizer(wx.VERTICAL)
        sizer_31 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_30 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_29 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_26 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_24_staticbox.Lower()
        sizer_24 = wx.StaticBoxSizer(self.sizer_24_staticbox, wx.VERTICAL)
        sizer_67 = wx.BoxSizer(wx.VERTICAL)
        sizer_68 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_25 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_23 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_19_staticbox.Lower()
        sizer_19 = wx.StaticBoxSizer(self.sizer_19_staticbox, wx.VERTICAL)
        sizer_65 = wx.BoxSizer(wx.VERTICAL)
        sizer_66 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_22 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_21 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_20 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_18 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_14_staticbox.Lower()
        sizer_14 = wx.StaticBoxSizer(self.sizer_14_staticbox, wx.VERTICAL)
        sizer_63 = wx.BoxSizer(wx.VERTICAL)
        sizer_64 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_17 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_16 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_15 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_50 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_155_staticbox.Lower()
        sizer_155 = wx.StaticBoxSizer(self.sizer_155_staticbox, wx.VERTICAL)
        sizer_160 = wx.BoxSizer(wx.VERTICAL)
        sizer_161 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_156 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_159 = wx.BoxSizer(wx.VERTICAL)
        sizer_158 = wx.BoxSizer(wx.VERTICAL)
        sizer_157 = wx.BoxSizer(wx.VERTICAL)
        sizer_11 = wx.BoxSizer(wx.VERTICAL)
        sizer_12 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_88 = wx.BoxSizer(wx.VERTICAL)
        sizer_108 = wx.BoxSizer(wx.VERTICAL)
        sizer_109 = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_82_staticbox.Lower()
        sizer_82 = wx.StaticBoxSizer(self.sizer_82_staticbox, wx.VERTICAL)
        sizer_95 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_98 = wx.BoxSizer(wx.VERTICAL)
        sizer_97 = wx.BoxSizer(wx.VERTICAL)
        sizer_96 = wx.BoxSizer(wx.VERTICAL)
        sizer_81 = wx.BoxSizer(wx.VERTICAL)
        sizer_13 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_10_staticbox.Lower()
        sizer_10 = wx.StaticBoxSizer(self.sizer_10_staticbox, wx.VERTICAL)
        sizer_51 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_52 = wx.BoxSizer(wx.VERTICAL)
        sizer_55 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_53 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_54 = wx.BoxSizer(wx.VERTICAL)
        self.sizer_7_staticbox.Lower()
        sizer_7 = wx.StaticBoxSizer(self.sizer_7_staticbox, wx.VERTICAL)
        sizer_9 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_8 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_84 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_86 = wx.BoxSizer(wx.VERTICAL)
        sizer_87 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_85 = wx.BoxSizer(wx.VERTICAL)
        sizer_4 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.label_1, 0, wx.TOP | wx.BOTTOM | wx.ALIGN_CENTER_HORIZONTAL, 10)
        sizer_4.Add(self.label_2, 0, wx.LEFT | wx.TOP, 10)
        sizer_4.Add(self.tree_ctrl_1, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM, 5)
        sizer_3.Add(sizer_4, 0, wx.EXPAND, 0)
        sizer_85.Add(self.radio_btn_1, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_85.Add(self.radio_btn_2, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_84.Add(sizer_85, 0, wx.EXPAND, 0)
        sizer_86.Add(self.label_3, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_87.Add(self.text_ctrl_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_87.Add(self.button_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_86.Add(sizer_87, 1, wx.EXPAND, 0)
        sizer_84.Add(sizer_86, 1, wx.EXPAND, 0)
        sizer_7.Add(sizer_84, 1, wx.EXPAND, 0)
        sizer_8.Add(self.button_2, 0, wx.LEFT | wx.TOP, 10)
        sizer_7.Add(sizer_8, 0, wx.EXPAND, 0)
        sizer_7.Add(sizer_9, 0, wx.EXPAND, 0)
        sizer_6.Add(sizer_7, 0, wx.ALL | wx.EXPAND, 5)
        sizer_54.Add(self.label_4, 0, wx.ALL, 10)
        sizer_51.Add(sizer_54, 1, wx.EXPAND, 0)
        sizer_53.Add(self.button_4, 0, wx.ALIGN_RIGHT, 0)
        sizer_52.Add(sizer_53, 0, wx.ALIGN_RIGHT, 0)
        sizer_55.Add(self.button_47, 0, wx.TOP, 5)
        sizer_52.Add(sizer_55, 1, wx.EXPAND, 0)
        sizer_51.Add(sizer_52, 0, 0, 0)
        sizer_10.Add(sizer_51, 1, wx.EXPAND, 0)
        sizer_6.Add(sizer_10, 1, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_1.SetSizer(sizer_6)
        sizer_13.Add(self.label_30, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_13.Add(self.text_ctrl_22, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_13.Add(self.button_3, 0, wx.RIGHT, 5)
        sizer_81.Add(sizer_13, 0, wx.TOP | wx.EXPAND, 5)
        sizer_88.Add(sizer_81, 0, wx.EXPAND, 0)
        sizer_96.Add(self.label_37, 0, wx.LEFT, 5)
        sizer_96.Add(self.list_box_5, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_95.Add(sizer_96, 1, wx.EXPAND, 0)
        sizer_97.Add(self.button_17, 0, 0, 0)
        sizer_97.Add(self.button_18, 0, 0, 0)
        sizer_95.Add(sizer_97, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_98.Add(self.label_43, 0, 0, 0)
        sizer_98.Add(self.list_box_6, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_95.Add(sizer_98, 1, wx.EXPAND, 0)
        sizer_82.Add(sizer_95, 1, wx.EXPAND, 0)
        sizer_88.Add(sizer_82, 1, wx.ALL | wx.EXPAND, 5)
        sizer_109.Add(self.button_20, 0, wx.ALL, 5)
        sizer_108.Add(sizer_109, 0, wx.TOP | wx.EXPAND, 5)
        sizer_88.Add(sizer_108, 0, wx.EXPAND, 0)
        self.notebook_1_pane_2.SetSizer(sizer_88)
        sizer_12.Add(self.label_5, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_11.Add(sizer_12, 1, wx.ALIGN_CENTER_HORIZONTAL, 0)
        self.notebook_1_pane_3.SetSizer(sizer_11)
        sizer_155.Add(self.label_41, 0, wx.ALL, 10)
        sizer_157.Add(self.label_42, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_157.Add(self.list_box_14, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_156.Add(sizer_157, 1, wx.EXPAND, 0)
        sizer_158.Add(self.button_42, 0, 0, 0)
        sizer_158.Add(self.button_43, 0, 0, 0)
        sizer_156.Add(sizer_158, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_159.Add(self.label_67, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_159.Add(self.list_box_15, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_156.Add(sizer_159, 1, wx.EXPAND, 0)
        sizer_155.Add(sizer_156, 1, wx.EXPAND, 0)
        sizer_161.Add(self.button_44, 0, wx.ALL, 5)
        sizer_160.Add(sizer_161, 0, wx.TOP | wx.EXPAND, 5)
        sizer_155.Add(sizer_160, 0, wx.EXPAND, 0)
        sizer_50.Add(sizer_155, 1, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_4.SetSizer(sizer_50)
        sizer_14.Add(self.label_6, 0, wx.ALL, 5)
        sizer_15.Add(self.checkbox_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_15.Add(self.text_ctrl_2, 0, wx.LEFT, 10)
        sizer_14.Add(sizer_15, 0, wx.EXPAND, 0)
        sizer_16.Add(self.checkbox_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_16.Add(self.text_ctrl_3, 0, wx.LEFT, 10)
        sizer_14.Add(sizer_16, 0, wx.EXPAND, 0)
        sizer_17.Add(self.checkbox_3, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_17.Add(self.text_ctrl_4, 0, wx.LEFT, 10)
        sizer_14.Add(sizer_17, 0, wx.EXPAND, 0)
        sizer_64.Add(self.button_9, 0, wx.ALL, 5)
        sizer_63.Add(sizer_64, 0, wx.TOP | wx.EXPAND, 5)
        sizer_14.Add(sizer_63, 0, wx.EXPAND, 0)
        sizer_18.Add(sizer_14, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_5.SetSizer(sizer_18)
        sizer_19.Add(self.label_8, 0, wx.ALL, 5)
        sizer_20.Add(self.label_7, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_20.Add(self.text_ctrl_5, 0, wx.LEFT, 10)
        sizer_19.Add(sizer_20, 0, wx.EXPAND, 0)
        sizer_21.Add(self.label_9, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_21.Add(self.text_ctrl_6, 0, wx.LEFT, 10)
        sizer_19.Add(sizer_21, 0, wx.EXPAND, 0)
        sizer_22.Add(self.label_10, 0, wx.LEFT, 10)
        sizer_22.Add(self.text_ctrl_7, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_19.Add(sizer_22, 0, wx.EXPAND, 0)
        sizer_66.Add(self.button_10, 0, wx.ALL, 5)
        sizer_65.Add(sizer_66, 0, wx.TOP | wx.EXPAND, 5)
        sizer_19.Add(sizer_65, 0, wx.EXPAND, 0)
        sizer_23.Add(sizer_19, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_6.SetSizer(sizer_23)
        sizer_24.Add(self.label_11, 0, wx.ALL, 5)
        sizer_25.Add(self.label_12, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_25.Add(self.text_ctrl_8, 0, wx.LEFT, 10)
        sizer_24.Add(sizer_25, 0, wx.EXPAND, 0)
        sizer_68.Add(self.button_11, 0, wx.ALL, 5)
        sizer_67.Add(sizer_68, 0, wx.TOP | wx.EXPAND, 5)
        sizer_24.Add(sizer_67, 0, wx.EXPAND, 0)
        sizer_26.Add(sizer_24, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_7.SetSizer(sizer_26)
        sizer_27.Add(self.label_13, 0, wx.ALL, 5)
        sizer_29.Add(self.label_14, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_29.Add(self.text_ctrl_9, 0, wx.LEFT, 10)
        sizer_28.Add(sizer_29, 0, wx.EXPAND, 0)
        sizer_30.Add(self.label_15, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_30.Add(self.text_ctrl_10, 0, wx.LEFT, 10)
        sizer_28.Add(sizer_30, 0, wx.EXPAND, 0)
        sizer_31.Add(self.label_16, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_31.Add(self.text_ctrl_11, 0, wx.LEFT, 10)
        sizer_28.Add(sizer_31, 0, wx.EXPAND, 0)
        sizer_27.Add(sizer_28, 0, wx.EXPAND, 0)
        sizer_70.Add(self.button_12, 0, wx.ALL, 5)
        sizer_69.Add(sizer_70, 0, wx.TOP | wx.EXPAND, 5)
        sizer_27.Add(sizer_69, 0, wx.EXPAND, 0)
        sizer_32.Add(sizer_27, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_8.SetSizer(sizer_32)
        sizer_34.Add(self.label_17, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_34.Add(self.text_ctrl_12, 0, wx.LEFT, 10)
        sizer_33.Add(sizer_34, 0, wx.EXPAND, 0)
        sizer_35.Add(self.label_18, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_35.Add(self.text_ctrl_13, 0, wx.LEFT, 10)
        sizer_33.Add(sizer_35, 0, wx.LEFT | wx.RIGHT | wx.EXPAND, 0)
        sizer_36.Add(self.label_19, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_36.Add(self.text_ctrl_14, 0, wx.LEFT, 10)
        sizer_33.Add(sizer_36, 0, wx.EXPAND, 0)
        sizer_73.Add(self.button_13, 0, wx.TOP, 5)
        sizer_72.Add(sizer_73, 0, wx.TOP | wx.EXPAND, 5)
        sizer_33.Add(sizer_72, 0, wx.EXPAND, 0)
        sizer_37.Add(sizer_33, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_9.SetSizer(sizer_37)
        sizer_39.Add(self.label_20, 0, wx.ALL, 5)
        sizer_38.Add(sizer_39, 0, wx.EXPAND, 0)
        sizer_40.Add(self.label_21, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_40.Add(self.text_ctrl_15, 0, wx.LEFT, 10)
        sizer_38.Add(sizer_40, 0, wx.EXPAND, 0)
        sizer_38.Add((20, 20), 0, 0, 0)
        sizer_42.Add(self.label_22, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_43.Add(self.label_23, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_43.Add(self.text_ctrl_16, 0, wx.LEFT, 10)
        sizer_42.Add(sizer_43, 0, wx.EXPAND, 0)
        sizer_44.Add(self.label_24, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_44.Add(self.text_ctrl_17, 0, wx.LEFT, 10)
        sizer_42.Add(sizer_44, 0, wx.EXPAND, 0)
        sizer_45.Add(self.label_25, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_45.Add(self.text_ctrl_18, 0, wx.LEFT, 10)
        sizer_42.Add(sizer_45, 0, wx.EXPAND, 0)
        sizer_41.Add(sizer_42, 1, wx.EXPAND, 0)
        sizer_46.Add(self.label_26, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_47.Add(self.label_27, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_47.Add(self.text_ctrl_19, 0, wx.LEFT, 10)
        sizer_46.Add(sizer_47, 0, wx.EXPAND, 0)
        sizer_48.Add(self.label_28, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_48.Add(self.text_ctrl_20, 0, wx.LEFT, 10)
        sizer_46.Add(sizer_48, 0, wx.EXPAND, 0)
        sizer_49.Add(self.label_29, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_49.Add(self.text_ctrl_21, 0, wx.LEFT, 10)
        sizer_46.Add(sizer_49, 0, wx.EXPAND, 0)
        sizer_41.Add(sizer_46, 1, wx.EXPAND, 0)
        sizer_38.Add(sizer_41, 0, wx.EXPAND, 0)
        sizer_75.Add(self.button_14, 0, wx.ALL, 5)
        sizer_74.Add(sizer_75, 0, wx.TOP | wx.EXPAND, 5)
        sizer_38.Add(sizer_74, 0, wx.EXPAND, 0)
        sizer_83.Add(sizer_38, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_10.SetSizer(sizer_83)
        sizer_92.Add(self.label_39, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_92.Add(self.text_ctrl_26, 0, wx.LEFT, 10)
        sizer_90.Add(sizer_92, 0, wx.EXPAND, 0)
        sizer_93.Add(self.label_40, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_93.Add(self.text_ctrl_27, 0, wx.LEFT, 10)
        sizer_90.Add(sizer_93, 0, wx.EXPAND, 0)
        sizer_153.Add(self.button_41, 0, wx.ALL, 5)
        sizer_94.Add(sizer_153, 1, wx.TOP | wx.EXPAND, 5)
        sizer_90.Add(sizer_94, 1, wx.EXPAND, 0)
        sizer_58.Add(sizer_90, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_11.SetSizer(sizer_58)
        sizer_190.Add(self.button_36, 0, wx.ALL, 10)
        sizer_190.Add(self.label_87, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_189.Add(sizer_190, 1, wx.EXPAND, 0)
        sizer_110.Add(sizer_189, 0, wx.ALL | wx.EXPAND, 5)
        sizer_148.Add(self.button_37, 0, wx.ALL, 10)
        sizer_110.Add(sizer_148, 0, wx.ALL | wx.EXPAND, 5)
        sizer_150.Add(self.button_38, 0, wx.LEFT | wx.RIGHT | wx.TOP, 10)
        sizer_150.Add(self.label_64, 0, wx.TOP | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_149.Add(sizer_150, 0, wx.EXPAND, 0)
        sizer_152.Add(self.button_40, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_152.Add(self.label_66, 0, wx.TOP, 5)
        sizer_149.Add(sizer_152, 1, wx.EXPAND, 0)
        sizer_151.Add(self.button_39, 0, wx.ALL, 10)
        sizer_151.Add(self.label_65, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_149.Add(sizer_151, 0, wx.EXPAND, 0)
        sizer_110.Add(sizer_149, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_12.SetSizer(sizer_110)
        sizer_173.Add(self.label_36, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_173.Add(self.combo_box_11, 0, 0, 0)
        sizer_173.Add(self.label_78, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_177.Add(sizer_173, 0, wx.EXPAND, 0)
        sizer_174.Add(self.label_79, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_174.Add(self.text_ctrl_39, 0, 0, 0)
        sizer_174.Add(self.label_80, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_177.Add(sizer_174, 0, wx.EXPAND, 0)
        sizer_175.Add(self.label_81, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_175.Add(self.text_ctrl_40, 0, 0, 0)
        sizer_175.Add(self.label_82, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_177.Add(sizer_175, 0, wx.EXPAND, 0)
        sizer_176.Add(self.label_83, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_176.Add(self.text_ctrl_41, 0, 0, 0)
        sizer_176.Add(self.label_84, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_177.Add(sizer_176, 0, wx.EXPAND, 0)
        sizer_138.Add(sizer_177, 1, wx.ALL | wx.EXPAND, 5)
        sizer_178.Add(self.button_16, 0, wx.LEFT, 5)
        sizer_138.Add(sizer_178, 0, wx.EXPAND, 0)
        sizer_131.Add(sizer_138, 0, wx.ALL | wx.EXPAND, 5)
        sizer_183.Add(self.checkbox_6, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_183.Add(self.label_86, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 20)
        sizer_183.Add(self.text_ctrl_42, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_183.Add(self.button_35, 0, wx.LEFT, 40)
        sizer_182.Add(sizer_183, 0, wx.EXPAND, 0)
        sizer_131.Add(sizer_182, 0, wx.ALL | wx.EXPAND, 5)
        sizer_186.Add(self.checkbox_4, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_186.Add(self.checkbox_5, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_186.Add(self.label_101, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_186.Add(self.text_ctrl_43, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_185.Add(sizer_186, 0, wx.EXPAND, 0)
        sizer_187.Add(self.label_57, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_187.Add(self.text_ctrl_25, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_187.Add(self.button_26, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 20)
        sizer_185.Add(sizer_187, 0, wx.EXPAND, 0)
        sizer_184.Add(sizer_185, 1, wx.EXPAND, 0)
        sizer_127.Add(sizer_184, 1, wx.EXPAND, 0)
        sizer_131.Add(sizer_127, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_13.SetSizer(sizer_131)
        sizer_125.Add(self.label_51, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.ALIGN_CENTER_HORIZONTAL, 20)
        sizer_125.Add(self.label_56, 0, wx.BOTTOM | wx.ALIGN_BOTTOM, 3)
        sizer_118.Add(sizer_125, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_118.Add(self.label_52, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 10)
        sizer_118.Add((20, 60), 0, 0, 0)
        sizer_120.Add(self.text_ctrl_23, 0, wx.LEFT | wx.EXPAND, 5)
        sizer_119.Add(sizer_120, 1, wx.EXPAND, 0)
        sizer_119.Add(self.button_24, 0, wx.RIGHT, 5)
        sizer_118.Add(sizer_119, 0, wx.ALL | wx.EXPAND, 5)
        sizer_122.Add(self.label_53, 0, wx.LEFT | wx.TOP, 5)
        sizer_122.Add(self.list_box_9, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_121.Add(sizer_122, 1, wx.EXPAND, 0)
        sizer_123.Add(self.button_25, 0, wx.TOP, 23)
        sizer_121.Add(sizer_123, 0, 0, 0)
        sizer_124.Add(self.label_54, 0, wx.LEFT | wx.RIGHT | wx.TOP, 5)
        sizer_124.Add(self.text_ctrl_24, 0, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_124.Add(self.button_27, 0, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_124.Add(self.label_55, 0, 0, 0)
        sizer_121.Add(sizer_124, 1, wx.EXPAND, 0)
        sizer_118.Add(sizer_121, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_14.SetSizer(sizer_118)
        sizer_100.Add(self.label_44, 0, wx.ALL, 5)
        sizer_101.Add(self.label_45, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_101.Add(self.combo_box_5, 0, 0, 0)
        sizer_100.Add(sizer_101, 0, wx.EXPAND, 0)
        sizer_103.Add(self.label_46, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_103.Add(self.combo_box_6, 0, 0, 0)
        sizer_180.Add(sizer_103, 0, wx.EXPAND, 0)
        sizer_104.Add(self.label_47, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_104.Add(self.combo_box_7, 0, 0, 0)
        sizer_180.Add(sizer_104, 0, wx.EXPAND, 0)
        sizer_105.Add(self.label_48, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_105.Add(self.combo_box_8, 0, 0, 0)
        sizer_180.Add(sizer_105, 0, wx.EXPAND, 0)
        sizer_179.Add(sizer_180, 0, wx.EXPAND, 0)
        sizer_181.Add(self.label_85, 0, 0, 0)
        sizer_181.Add(self.button_34, 0, wx.EXPAND, 0)
        sizer_179.Add(sizer_181, 1, wx.ALL, 5)
        sizer_102.Add(sizer_179, 1, wx.EXPAND, 0)
        sizer_100.Add(sizer_102, 0, wx.ALL | wx.EXPAND, 5)
        sizer_107.Add(self.button_19, 0, wx.ALL, 5)
        sizer_106.Add(sizer_107, 0, wx.TOP | wx.EXPAND, 5)
        sizer_100.Add(sizer_106, 0, wx.EXPAND, 0)
        sizer_99.Add(sizer_100, 0, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_15.SetSizer(sizer_99)
        sizer_60.Add(self.label_31, 0, wx.ALL, 10)
        sizer_77.Add(self.label_34, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_77.Add(self.combo_box_2, 0, wx.ALL, 5)
        sizer_60.Add(sizer_77, 0, wx.ALL | wx.EXPAND, 5)
        sizer_62.Add(self.label_33, 0, wx.FIXED_MINSIZE, 0)
        sizer_61.Add(sizer_62, 1, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_76.Add(self.button_5, 0, 0, 0)
        sizer_76.Add(self.button_7, 0, 0, 0)
        sizer_61.Add(sizer_76, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_71.Add(self.label_32, 0, wx.LEFT, 5)
        sizer_71.Add(self.list_box_1, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_61.Add(sizer_71, 1, wx.EXPAND, 0)
        sizer_60.Add(sizer_61, 1, wx.ALL | wx.EXPAND, 5)
        sizer_79.Add(self.button_8, 0, wx.ALL, 5)
        sizer_78.Add(sizer_79, 0, wx.TOP | wx.EXPAND, 5)
        sizer_60.Add(sizer_78, 0, wx.EXPAND, 0)
        sizer_59.Add(sizer_60, 1, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_16.SetSizer(sizer_59)
        sizer_89.Add(self.label_35, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_89.Add(self.text_ctrl_28, 0, wx.TOP | wx.BOTTOM, 10)
        sizer_80.Add(sizer_89, 0, wx.EXPAND, 0)
        sizer_112.Add(self.label_38, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_112.Add(self.combo_box_4, 0, 0, 0)
        sizer_111.Add(sizer_112, 0, wx.EXPAND, 0)
        sizer_113.Add(self.label_49, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_113.Add(self.combo_box_9, 0, 0, 0)
        sizer_111.Add(sizer_113, 0, wx.EXPAND, 0)
        sizer_91.Add(sizer_111, 1, wx.EXPAND, 0)
        sizer_80.Add(sizer_91, 0, wx.ALL | wx.EXPAND, 5)
        sizer_116.Add(self.label_50, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_116.Add(self.combo_box_10, 0, 0, 0)
        sizer_115.Add(sizer_116, 0, wx.EXPAND, 0)
        sizer_132.Add(self.label_60, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_132.Add(self.text_ctrl_31, 0, 0, 0)
        sizer_115.Add(sizer_132, 0, wx.EXPAND, 0)
        sizer_133.Add(self.label_61, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_133.Add(self.text_ctrl_32, 0, 0, 0)
        sizer_115.Add(sizer_133, 0, wx.EXPAND, 0)
        sizer_117.Add(self.label_58, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_117.Add(self.text_ctrl_29, 0, 0, 0)
        sizer_115.Add(sizer_117, 0, wx.EXPAND, 0)
        sizer_126.Add(self.label_59, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_126.Add(self.text_ctrl_30, 0, 0, 0)
        sizer_115.Add(sizer_126, 0, wx.EXPAND, 0)
        sizer_114.Add(sizer_115, 1, wx.EXPAND, 0)
        sizer_80.Add(sizer_114, 0, wx.ALL | wx.EXPAND, 5)
        sizer_135.Add(self.button_15, 0, wx.ALL, 5)
        sizer_134.Add(sizer_135, 0, wx.TOP | wx.EXPAND, 5)
        sizer_80.Add(sizer_134, 0, wx.EXPAND, 0)
        self.notebook_1_pane_17.SetSizer(sizer_80)
        sizer_145.Add(self.label_68, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_144.Add(sizer_145, 0, wx.EXPAND, 0)
        sizer_147.Add(self.label_69, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_147.Add(self.combo_box_3, 0, 0, 0)
        sizer_146.Add(sizer_147, 0, wx.EXPAND, 0)
        sizer_154.Add(self.label_70, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_154.Add(self.text_ctrl_34, 0, 0, 0)
        sizer_146.Add(sizer_154, 0, wx.EXPAND, 0)
        sizer_162.Add(self.label_71, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_162.Add(self.text_ctrl_35, 0, 0, 0)
        sizer_146.Add(sizer_162, 0, wx.EXPAND, 0)
        sizer_163.Add(self.label_72, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_163.Add(self.text_ctrl_36, 0, 0, 0)
        sizer_146.Add(sizer_163, 0, wx.ALIGN_CENTER_VERTICAL, 38)
        sizer_164.Add(self.label_73, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_164.Add(self.text_ctrl_37, 0, 0, 0)
        sizer_164.Add(self.text_ctrl_38, 0, wx.RIGHT, 5)
        sizer_146.Add(sizer_164, 0, wx.EXPAND, 0)
        sizer_144.Add(sizer_146, 0, wx.ALL | wx.EXPAND, 5)
        sizer_167.Add(self.label_75, 0, wx.LEFT, 5)
        sizer_167.Add(self.list_box_3, 1, wx.ALL | wx.EXPAND, 5)
        sizer_166.Add(sizer_167, 1, wx.EXPAND, 0)
        sizer_168.Add(self.button_23, 0, 0, 0)
        sizer_168.Add(self.button_31, 0, 0, 0)
        sizer_166.Add(sizer_168, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_169.Add(self.label_76, 0, 0, 0)
        sizer_169.Add(self.list_box_4, 1, wx.ALL | wx.EXPAND, 5)
        sizer_166.Add(sizer_169, 1, wx.EXPAND, 0)
        sizer_144.Add(sizer_166, 1, wx.ALL | wx.EXPAND, 5)
        sizer_171.Add(self.button_32, 0, wx.ALL, 5)
        sizer_170.Add(sizer_171, 0, wx.TOP | wx.EXPAND, 5)
        sizer_144.Add(sizer_170, 0, wx.EXPAND, 0)
        self.notebook_1_pane_18.SetSizer(sizer_144)
        sizer_141.Add(self.label_74, 0, wx.LEFT | wx.TOP, 5)
        sizer_141.Add(self.list_box_2, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_140.Add(sizer_141, 1, wx.EXPAND, 0)
        sizer_142.Add(self.button_21, 0, 0, 0)
        sizer_142.Add(self.button_22, 0, 0, 0)
        sizer_140.Add(sizer_142, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_143.Add(self.label_77, 0, wx.LEFT | wx.TOP, 5)
        sizer_143.Add(self.list_box_7, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_140.Add(sizer_143, 1, wx.EXPAND, 0)
        sizer_136.Add(sizer_140, 1, wx.EXPAND, 0)
        sizer_172.Add(self.button_33, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM, 5)
        sizer_165.Add(sizer_172, 0, wx.EXPAND, 0)
        sizer_136.Add(sizer_165, 0, wx.EXPAND, 0)
        sizer_137.Add(sizer_136, 1, wx.ALL | wx.EXPAND, 5)
        self.notebook_1_pane_19.SetSizer(sizer_137)
        sizer_192.Add(self.label_88, 0, wx.ALL, 10)
        sizer_193.Add(self.combo_box_12, 0, wx.ALL, 5)
        sizer_193.Add(self.button_46, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM, 5)
        sizer_192.Add(sizer_193, 0, wx.ALL | wx.EXPAND, 5)
        self.panel_solution.SetSizer(sizer_192)
        sizer_191.Add(self.panel_solution, 0, wx.EXPAND, 0)
        sizer_209.Add(self.label_99, 0, wx.ALL, 10)
        self.panel_static.SetSizer(sizer_209)
        sizer_191.Add(self.panel_static, 0, wx.EXPAND, 0)
        sizer_195.Add(self.label_89, 0, wx.TOP | wx.ALIGN_CENTER_HORIZONTAL, 10)
        sizer_196.Add(self.label_90, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_196.Add(self.combo_box_16, 0, 0, 0)
        sizer_197.Add(sizer_196, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 0)
        sizer_198.Add(self.label_91, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_198.Add(self.combo_box_13, 0, 0, 0)
        sizer_197.Add(sizer_198, 0, wx.EXPAND, 0)
        sizer_199.Add(self.label_92, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_199.Add(self.combo_box_14, 0, 0, 0)
        sizer_197.Add(sizer_199, 0, wx.EXPAND, 0)
        sizer_200.Add(self.label_93, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_200.Add(self.text_ctrl_45, 0, 0, 0)
        sizer_200.Add(self.text_ctrl_46, 0, 0, 0)
        sizer_197.Add(sizer_200, 0, wx.EXPAND, 0)
        sizer_201.Add(self.label_94, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_201.Add(self.text_ctrl_47, 0, 0, 0)
        sizer_197.Add(sizer_201, 0, wx.EXPAND, 0)
        sizer_202.Add(self.label_95, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_202.Add(self.text_ctrl_48, 0, 0, 0)
        sizer_197.Add(sizer_202, 0, wx.EXPAND, 0)
        sizer_203.Add(self.label_96, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_203.Add(self.text_ctrl_49, 0, 0, 0)
        sizer_203.Add(self.label_97, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_197.Add(sizer_203, 0, wx.EXPAND, 0)
        sizer_195.Add(sizer_197, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_205.Add(self.label_98, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_205.Add(self.text_ctrl_50, 0, 0, 0)
        sizer_205.Add(self.text_ctrl_51, 0, 0, 0)
        sizer_205.Add(self.label_103, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_204.Add(sizer_205, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_206.Add(self.label_100, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_206.Add(self.text_ctrl_44, 0, 0, 0)
        sizer_206.Add(self.text_ctrl_52, 0, 0, 0)
        sizer_206.Add(self.label_104, 0, wx.LEFT, 5)
        sizer_204.Add(sizer_206, 0, wx.EXPAND, 0)
        sizer_195.Add(sizer_204, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_208.Add(self.label_102, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_208.Add(self.combo_box_15, 0, 0, 0)
        sizer_208.Add(self.label_105, 0, wx.LEFT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)
        sizer_208.Add(self.text_ctrl_53, 0, 0, 0)
        sizer_207.Add(sizer_208, 0, wx.EXPAND, 0)
        sizer_210.Add(self.label_106, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_210.Add(self.checkbox_7, 0, 0, 0)
        sizer_210.Add(self.checkbox_8, 0, 0, 0)
        sizer_210.Add(self.checkbox_9, 0, 0, 0)
        sizer_207.Add(sizer_210, 0, wx.EXPAND, 0)
        sizer_211.Add(self.checkbox_10, 0, wx.LEFT, 85)
        sizer_211.Add(self.checkbox_11, 0, 0, 0)
        sizer_211.Add(self.checkbox_12, 0, 0, 0)
        sizer_207.Add(sizer_211, 1, wx.EXPAND, 0)
        sizer_195.Add(sizer_207, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_195.Add(self.button_45, 0, wx.LEFT | wx.RIGHT | wx.BOTTOM, 5)
        sizer_194.Add(sizer_195, 0, wx.EXPAND, 0)
        self.panel_dynamic.SetSizer(sizer_194)
        sizer_191.Add(self.panel_dynamic, 1, wx.EXPAND, 0)
        self.notebook_1_pane_20.SetSizer(sizer_191)
        self.notebook_1.AddPage(self.notebook_1_pane_1, _("FistrModel"))
        self.notebook_1.AddPage(self.notebook_1_pane_2, _(u"材料物性値"))
        self.notebook_1.AddPage(self.notebook_1_pane_3, _(u"境界条件"))
        self.notebook_1.AddPage(self.notebook_1_pane_4, _("setGroup"))
        self.notebook_1.AddPage(self.notebook_1_pane_5, _("BOUNDARY_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_6, _("CLOAD_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_7, _("DLOAD_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_8, _("VLOAD_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_9, _("GRAV_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_10, _("CENT_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_11, _("TEMPERATURE_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_12, _("post"))
        self.notebook_1.AddPage(self.notebook_1_pane_13, _("solver"))
        self.notebook_1.AddPage(self.notebook_1_pane_14, _("FrontISTR"))
        self.notebook_1.AddPage(self.notebook_1_pane_15, _(u"材料物性値_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_16, _("contact"))
        self.notebook_1.AddPage(self.notebook_1_pane_17, _("contact_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_18, _("step_sub"))
        self.notebook_1.AddPage(self.notebook_1_pane_19, _("output"))
        self.notebook_1.AddPage(self.notebook_1_pane_20, _("solutionType"))
        sizer_5.Add(self.notebook_1, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 5)
        sizer_3.Add(sizer_5, 1, wx.EXPAND, 0)
        sizer_2.Add(sizer_3, 1, wx.EXPAND, 0)
        sizer_130.Add(self.button_28, 0, wx.ALL, 10)
        sizer_130.Add(self.button_29, 0, wx.TOP, 10)
        sizer_130.Add(self.button_30, 0, wx.LEFT | wx.TOP, 10)
        sizer_128.Add(sizer_130, 1, wx.EXPAND, 0)
        sizer_129.Add(self.button_6, 0, wx.ALL | wx.ALIGN_RIGHT, 10)
        sizer_128.Add(sizer_129, 0, wx.ALIGN_RIGHT, 0)
        sizer_56.Add(sizer_128, 1, wx.EXPAND, 0)
        sizer_56.Add(sizer_57, 0, wx.ALIGN_RIGHT, 0)
        sizer_2.Add(sizer_56, 0, wx.EXPAND, 0)
        sizer_1.Add(sizer_2, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()
        # end wxGlade



    #----- TreeItem選択 -----------------
    #  項目選択時
    def onSelectItem(self, event):  # wxGlade: MyFrame.<event_handler>
        self.selectItem(event)

    #----- FistrModel page -----------------
    #  meshFileをセット
    def onSetMeshFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setMeshFile(event)
    #  meshFile変換
    def onMeshConvert(self, event):  # wxGlade: MyFrame.<event_handler>
        print _(u"メッシュfileを変換します")
        self.meshConvert(event)
    #  形状確認
    def onConfirmModelAtParaview(self, event):  # wxGlade: MyFrame.<event_handler>
        self.confirmModelAtParaview(event)

    #----- solutionType page -----------------
    def onApplySolutionType(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applySolutionType(event)
    #----- solutionType Dynamicpage -----------------
    def onApplyDynamicSolution(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyDynamicSolution(event)

    #----- 材料物性値 page -----------------
    #  setMaterialDB(参照...)
    def onSetMaterialDB(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setMaterialDB(event)
    #  changeSameMaterialチェックボックス
    def onChangeSameMaterial(self, event):  # wxGlade: MyFrame.<event_handler>
        self.changeSameMaterial(event)
    #  設定>> ボタン
    def onSetMaterialGroup(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setMaterialGroup(event)
    #  <<戻す ボタン
    def onReturnMaterialGroup(self, event):  # wxGlade: MyFrame.<event_handler>
        self.returnMaterialGroup(event)
    #  設定
    def onApplyMaterial(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyMaterial(event)

    #----- 材料物性値_sub page -----------------
    #  材料モデルセット
    def onSetEachMaterialModel(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setEachMaterialTypeHard()
    #  塑性ボタン
    def onEditSsData(self, event):  # wxGlade: MyFrame.<event_handler>
        self.editPlasticSsData(event)
    #  設定
    def onApplyEachMaterial(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyEachMaterial()

    #----- BOUNDARY_sub page -----------------
    #  設定
    def onApplyBoundaryCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyBoundaryCont(event)

    #----- CLOAD_sub page -----------------
    #  設定
    def onApplyCloadCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyCloadCont(event)

    #----- DLOAD_sub page -----------------
    def onApplyDloadCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyDloadCont(event)

    #----- VLOAD_sub page -----------------
    #  設定
    def onApplyVloadCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyVloadCont(event)

    #----- GRAV_sub page -----------------
    #  設定
    def onApplyGrav(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyGravCont(event)

    #----- CENT_sub page -----------------
    #  設定
    def onApplyCentCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyCentCont(event)

    #----- TEMPERATURE page -----------------
    #  設定
    def onApplyTemperature(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyTemperatureCont(event)

    #----- setGroup page -----------------
    #  選択>>
    def onSetBCGroup(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setBCGroup(event)
    #  <<戻す
    def onReturnBCGroup(self, event):  # wxGlade: MyFrame.<event_handler>
        self.returnBCGroup(event)
    #  設定
    def onApplyBCGoup(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyBCGroup(event)

    #----- contact page -----------------
    def onAddContactCondition(self, event):  # wxGlade: MyFrame.<event_handler>
        self.addContactCondition(event)
    def onDeleteContactCondition(self, event):  # wxGlade: MyFrame.<event_handler>
        self.deleteContactConditions(event)
    def onApplyContact(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyContact(event)

    #----- contact_sub page -----------------
    def onApplyContactCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyContactCont(event)

    #----- step_sub page -----------------
    def onSetBCGroupForStep(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setBCGroupForStep(event)
    def onReturnBCGroupForStep(self, event):  # wxGlade: MyFrame.<event_handler>
        self.returnBCGroupForStep(event)
    def onApplyStepCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyStepCont(event)

    #----- FrontISTR page -----------------
    def onSetWorkFolder(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setWorkFolder(event)
    #  参照...
    def onSelectWorkFolder(self, event):  # wxGlade: MyFrame.<event_handler>
        self.selectWorkFolder(event)
    #  選択>>
    def onSetUseCntFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setUseCntFile(event)
    #  cntファイル置き換え
    def onCopyUseCntFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.copyUseCntFile(event)
    #  解析の種類
    def onSelectAnaType(self, event):  # wxGlade: MyFrame.<event_handler>
        self.selectAnaType(event)

    #----- output page -----------------
    def onSetOutputItems(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setOutputItems(event)
    def onReturnOutputItems(self, event):  # wxGlade: MyFrame.<event_handler>
        self.returnOutputItems(event)
    def onApplyOutputCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applyOutputCont(event)
    #----- solver page -----------------
    def onSwitchParallel(self, event):  # wxGlade: MyFrame.<event_handler>
        self.switchParallel(event)
    def onCreateMultiJob(self, event):  # wxGlade: MyFrame.<event_handler>
        self.createMultiJob(event)
    def onApplySolverCont(self, event):  # wxGlade: MyFrame.<event_handler>
        self.applySolverCont(event)
    #  実行
    def onRunFrontIstr(self, event):  # wxGlade: MyFrame.<event_handler>
        self.runFrontIstr(event)

    #----- post page -----------------
    def onResultMerge(self, event):  # wxGlade: MyFrame.<event_handler>
        self.resultMerge(event)
    def onRunRevocap(self, event):  # wxGlade: MyFrame.<event_handler>
        self.runRevocap(event)
    def onConvertResultData(self, event):  # wxGlade: MyFrame.<event_handler>
        self.convertResultData(event)
    def onRunParaView(self, event):  # wxGlade: MyFrame.<event_handler>
        self.runParaView(event)
    def onAddPrincipalStress(self, event):  # wxGlade: MyFrame.<event_handler>
        self.addPrincipalStress(event)

    #----- 欄外 -----------------
    #  close
    def onClose(self, event):  # wxGlade: MyFrame.<event_handler>
        self.Destroy()
    #  workFolder開く
    def onOpenWorkFolder(self, event):  # wxGlade: MyFrame.<event_handler>
        self.openWorkFolder(event)
    #  cntファイル編集
    def onEditCntFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.editCntFile(event)
    def onEditMshFile(self, event):  # wxGlade: MyFrame.<event_handler>
        self.editMshFile(event)

    #  meshデータの読込
    def onReadMeshData(self, event):  # wxGlade: MyFrame.<event_handler>
        self.setMeshCont()
        #FistrModel.cntデータ取得
        self.setControlCont()       #cntFileの内容を取得
        matchControlCont()          #meshとcntFileの整合をとる
        #Tree作成
        makeIniTreeData()
        self.setIniTreeData()
        #材料物性値のDialog表示
        self.readMaterialDB()
        self.setMaterialNameToListBox()
        self.setMaterialModelToComboBox()
        treeList = [_(u"FistrModel.msh")]
        self.showTreeData(treeList)

# end of class MyFrame

    #  setOutputItems
    def setOutputItems(self, event):
        self.moveItemToListBox(self.list_box_2, self.list_box_7)

    #  returnOutputItems
    def returnOutputItems(self, event):
        self.moveItemToListBox(self.list_box_7, self.list_box_2)

    #  setBCGroupForStep
    def setBCGroupForStep(self, event):
        self.moveItemToListBox(self.list_box_3, self.list_box_4)

    #  returnBCGroupForStep
    def returnBCGroupForStep(self, event):
        self.moveItemToListBox(self.list_box_4, self.list_box_3)

    #  setBCGroup
    def setBCGroup(self, event):
        self.moveItemToListBox(self.list_box_14, self.list_box_15)

    #  returnBCGroup
    def returnBCGroup(self, event):
        self.moveItemToListBox(self.list_box_15, self.list_box_14)

    #
    #  editPlasticSsData
    #    塑性data作成
    def editPlasticSsData(self, event):
        matName = self.combo_box_5.GetStringSelection()
        fileName = currDir + "/" + matName + "_PlasticSSdata.csv"
        if len(glob.glob(fileName)) == 0:
            createPlasticSsDataFile(fileName)
        pathes = os.getenv("PATH").split(":")
        office = ""
        for path in pathes:
            if len(glob.glob(path+"/loffice")) > 0:
                office = "loffice"
                break
            if len(glob.glob(path+"/oofice")) > 0:
                office = "ooficce"
                break
        if office != "":
            comm = office + " " + fileName + " &"
            os.system(comm)
        else:
            comm = "nautilus " + currDir
            os.system(comm)

    #
    #  confirmModelAtParaview
    #    メッシュ形状確認
    def confirmModelAtParaview(self, event):
        os.chdir(currDir)
        meshFile = "FistrModel.msh"
        f=open(meshFile); lines=f.readlines(); f.close()
        newLines = []
        #nodeを取得
        nodesNo = []
        i = 0
        while i < len(lines):
            if lines[i][:len("!NODE")] == "!NODE":
                i += 1
                flag = 0
                n = 0
                while i < len(lines):
                    if lines[i][0] == "!":
                        flag = 1
                        break
                    else:
                        if lines[i][0] != "#":
                            words = lines[i].split(",")
                            newLines += [" ".join(words)]
                            nodesNo.append(int(words[0]))
                    n += 1
                    i += 1
                if flag == 1:
                    i -= 1
                    nNodes = n
            i += 1
        #elementを取得
        elementsNo = []
        matNo = 0
        i = 0
        n = 0
        while i < len(lines):
            if lines[i][:len("!ELEMENT")] == "!ELEMENT":
                elemType = abaqus2fistr.getValName(lines[i], "TYPE")
                egrp = abaqus2fistr.getValName(lines[i], "EGRP")
                inpType = pyFistr.getElementTypeFistr2inp(elemType)
                i += 1
                flag = 0
                matNo += 1
                while i < len(lines):
                    if lines[i][0] == "!":
                        flag = 1
                        break
                    else:
                        if lines[i][0] != "#":
                            words = lines[i].split(",")
                            newLines += [str(words[0]) + " " + str(matNo) + " " + inpType + " " + " ".join(words[1:])]
                            elementsNo.append(int(words[0]))
                    n += 1
                    i += 1
                if flag == 1:
                    i -= 1
                    nElements = n
            i += 1
        #node, elementNoの配列を準備
        nodeGrp = []
        maxNodeNo = max(nodesNo)
        i = 0
        while i < maxNodeNo + 1:
            nodeGrp += [[0, 0]]
            i += 1
        elementNo = []
        maxElementNo = max(elementsNo)
        i = 0
        while i < maxElementNo + 1:
            elementNo += [[]]
            i += 1
        elLines = newLines[nNodes:]
        for line in elLines:
            words = line.split()
            nodes = []
            for word in words[3:]:
                nodes.append(int(word))
            elementNo[int(words[0])] = [words[2]] + nodes
        #nodeGroupを取得
        nodeGroups = pyFistr.getNodeGroupsInLines(lines)
        i = 0
        for nodes in nodeGroups:
            for node in nodes:
                nodeGrp[int(node)][0] = i + 1
            i += 1
        #surfaceGroupを取得
        surfaceGroups = pyFistr.getSurfaceGroupsInLines(lines)
        j = 0
        while j < len(surfaceGroups):
            k = 0
            while k < len(surfaceGroups[j]):
                element = int(surfaceGroups[j][k])
                faceNo = int(surfaceGroups[j][k+1])
                nodes = pyFistr.getNodesFromElementSurface(elementNo, element, faceNo)
                for node in nodes:
                    nodeGrp[node][1] = j + 1
                k += 2
            j += 1
        #nodeNo, elementNoをrenumberする
        newLines = pyFistr.renumberMeshData(newLines, nNodes, nElements)
        #file作成
        header = [str(nNodes) + " " + str(nElements) + " 3 1 4\n"]
        newLines = header + newLines
        #nodeNo追加
        newLines += ["3 1 1 1\n"]
        newLines += ["nodeNo, unit_unknown\n"]
        newLines += ["nodeGroup, unit_unknown\n"]
        newLines += ["faceGroup, unit_unknown\n"]
        i = 0
        while i < nNodes:
            newLines += [(str(i+1) + " " + str(nodesNo[i]) + " " +
                    str(nodeGrp[nodesNo[i]][0]) + " " +
                    str(nodeGrp[nodesNo[i]][1]) + "\n")]
            i += 1
        #要素No追加
        newLines += ["1 1\n"]
        newLines += ["elementNo, unit_unknown\n"]
        i = 0
        while i < nElements:
            newLines += [str(i+1) + " " + str(elementsNo[i]) + "\n"]
            i += 1
        #file書き込み
        fileName = "convFistrModel.mesh.inp"
        f=open(fileName, "w")
        for line in newLines:
            f.write(line)
        f.close()
        #paraview起動
        comm = "paraview convFistrModel.mesh.inp &"
        os.system(comm)

    #
    #  runParaView
    #    paraViewの起動
    def runParaView(self, event):
        comm = "cd " + currDir + ";paraview &"
        os.system(comm)

    #
    #  addPrincipalStress
    #    変換した結果データに主応力を追加
    def addPrincipalStress(self, event):
        error = addPrincipalStressData()
        if error == "":
            title = _(u"データ変換")
            mess = _(u"結果データに主応力を追加しました。")
            self.okDialog(title, mess)
        elif error == "noFile":
            title = _(u"エラー")
            mess = _(u"結果データがparaView用に変換されていません。\n")
            mess += _(u"データ変換後、主応力を追加してください。")
            self.errDialog(title, mess)
        elif error == "added":
            title = _(u"エラー")
            mess = _(u"既に主応力が追加されています。\n")
            self.errDialog(title, mess)

    #
    #  resultMerge
    #    分散している結果を結合する
    def resultMerge(self, event):
        if len(glob.glob(currDir + "/FistrModel.res.1*")) == 0:
            title = _(u"エラー")
            mess = _(u"並列処理した計算結果ではありません")
            self.errDialog(title, mess)
            return
        os.chdir(currDir)
        names = glob.glob(currDir + "/FistrModel.res.0*")
        steps = []
        for name in names:
            step = int(name.split(".")[-1])
            steps.append(step)
        steps.sort()
        startStp = str(steps[0])
        endStp = str(steps[-1])
        if len(steps) > 1:
            nextStp = str(steps[1])
            stpInt = str(int(nextStp) - int(startStp))
            comm = "rmerge -s "+ startStp + " -e " + endStp + " -i " + stpInt + " FistrModel.res.merge"
        else:
            comm = "rmerge -s "+ startStp + " -e " + endStp + " FistrModel.res.merge"
        #nStep = len(glob.glob(currDir + "/FistrModel.res.0*"))
        #comm = "rmerge -s 1 -e " + str(nStep) + " FistrModel.res.merge"
        print comm
        os.system(comm)
        title = _(u"結果の再構築")
        mess = _(u"分散結果をマージしました。")
        self.okDialog(title, mess)

    #
    #  convertResultData
    #    結果fileをparaView用にデータ変換
    def convertResultData(self, event):
        error = convertDataForParaview()
        if error == "":
            title = _(u"データ変換")
            mess = _(u"paraview用にデータ変換しました。")
            self.okDialog(title, mess)
        else:
            title = _(u"エラー")
            self.errDialog(title, error)

    #
    #  runRevocap
    #    REVOCAOの起動
    def runRevocap(self, event):
        comm = "revocap &"
        os.system(comm)

    #
    #  openWorkFolder
    #    currDirを開く
    def openWorkFolder(self, event):
        comm = "nautilus " + currDir
        os.system(comm)

    #
    #  editCntFile
    #    FistrModel.cntファイル編集
    def editCntFile(self, event):
        fileNames = currDir + "/FistrModel.cnt " + currDir+ "/hecmw_ctrl.dat"
        partFile = currDir + "/hecmw_part_ctrl.dat"
        if len(glob.glob(partFile)) != 0:
            fileNames += " " + partFile        
        comm = "gedit --new-window " + fileNames + "&"
        os.system(comm)

    #
    #  editMshFile
    #    FistrModel.mshファイル編集
    def editMshFile(self, event):
        fileName = currDir + "/FistrModel.msh"
        comm = "gedit --new-window " + fileName + "&"
        os.system(comm)

    #
    #  copyUseCntFile
    #    cntファイルをコピー
    def copyUseCntFile(self, event):
        name = self.text_ctrl_24.GetValue()
        copyFile = os.getenv("easyIstrPath") + "/data/" + name
        if len(glob.glob(copyFile)) == 0:
            title = _(u"エラー")
            mess = _(u"cntファイルが正しく設定されていません")
            self.errDialog(title, mess)
            return
        pasteFile = currDir + "/FistrModel.cnt"
        comm = "cp " + copyFile + " " + pasteFile
        os.system(comm)
        pasteFile = currDir + "/hecmw_ctrl.dat"
        if len(glob.glob(pasteFile)) == 0:
            copyFile = os.getenv("easyIstrPath") + "/data/hecmw_ctrl.dat"
            comm = "cp " + copyFile + " " + pasteFile
            os.system(comm)
        #cntファイルを初期化
        resetCntMshFile()
        self.setIniTreeData()
        title = _(u"cntファイルの置き換え")
        mess = _(u"cntファイルを置き換えて、制御ファイルを初期化しました")
        self.okDialog(title, mess)

    #
    #  createMultiJob
    #    meshを分割する
    def createMultiJob(self, event):
        [para, nCpu] = getHecmwContFromDat()
        if para == False:
            title = _(u"エラー")
            mess = _(u"hecmw_part_ctrl.datファイルがありません\n")
            mess += _(u"並列処理をチェックしてください")
            self.errDialog(title, mess)
            return
        #分割メッシュが残っていれば削除
        delNames = "FistrModel_p*"
        if len(glob.glob(delNames)) > 0:
            comm = "rm FistrModel_p*"
            os.system(comm)
        #nCpuをセット
        nCpu = self.text_ctrl_42.GetValue()
        setHecmwContFromDat(para, nCpu)
        os.chdir(currDir)
        comm = "hecmw_part1"
        os.system(comm)

        title = _(u"メッシュ分割")
        mess = _(u"メッシュを分割しました。\n")
        mess += _(u"  ") + nCpu + _(u"並列")
        self.okDialog(title, mess)

    #
    #  switchParallel
    #    single, parallelのswitch
    def switchParallel(self, event):
        parallelSwitch = self.checkbox_6.GetValue()
        if parallelSwitch == True:
            #parallelの設定
            loadHcmwPartCtrlDat()
            [para, nCpu] = getHecmwContFromDat()
            self.text_ctrl_42.SetValue(nCpu)
            setParallelHcmwCtrlDat(nCpu)
        else:
            #singleの設定
            hideHcmwPartCtrlDat()
            setSingleHcmwCtrlDat()

    #
    #  runFrontIstr
    #    実行
    def runFrontIstr(self, event):
        #logの出力設定
        if self.checkbox_4.GetValue() == True:
            iterLog = "YES"
        else:
            iterLog = "NO"
        if self.checkbox_5.GetValue() == True:
            timeLog = "YES"
        else:
            timeLog = "NO"
        freq = self.text_ctrl_43.GetValue()
        setOutputLogInCnt(iterLog, timeLog, freq)
        #結果ファイルを削除
        os.chdir(currDir)
        delNames = "FistrModel*.inp"
        if len(glob.glob(delNames)) > 0:
            comm = "rm " + delNames
            os.system(comm)
        delNames = "convFistrModel*.inp"
        if len(glob.glob(delNames)) > 0:
            comm = "rm convFistrModel*.inp"
            os.system(comm)
        delNames = "FistrModel.res.*"
        if len(glob.glob(delNames)) > 0:
            comm = "rm " + delNames
            os.system(comm)
        delNames = "dyna_*.txt"
        if len(glob.glob(delNames)) > 0:
            comm = "rm " + delNames
            os.system(comm)
        #実行
        [para, nCpu] = getHecmwContFromDat()
        if para == True:
            runComm = "'mpirun -np " + nCpu + " fistr1'"
            comm = "echo " + runComm + " > run; chmod a+x run; gnome-terminal -x bash --rcfile run"
        else:
            comm = "echo fistr1 > run; chmod a+x run; gnome-terminal -x bash --rcfile run"
        os.system(comm)


    #
    #  setUseCntFile
    #    手本のcntファイルをセット
    def setUseCntFile(self, event):
        useFile = self.list_box_9.GetStringSelection()
        if useFile == "":
            title = _(u"エラー")
            mess = _(u"cntファイルを選択してください。")
            self.errDialog(title, mess)
            return
        self.text_ctrl_24.SetValue(useFile)

    #
    #  setWorkFolder
    #    作業用folderをセット
    def setWorkFolder(self, event):
        global currDir
        currDir = self.text_ctrl_23.GetValue()
        os.chdir(currDir)

    #
    #  selectWorkFolder
    #    作業用folderを設定
    def selectWorkFolder(self, event):
        global currDir
        title = _(u"作業用folderの選択")
        result = self.dirDialog(title, currDir)
        if result != "":
            currDir = result
            self.text_ctrl_23.SetValue(currDir)
            os.chdir(currDir)
            saveIstrData(["currDir "+currDir])
            stat = self.checkCntMshFile()
            if stat == "OK":
                self.setCntFilesToListBox()
                #Fistrmodel.mshデータを取得
                self.setMeshCont()
                #FistrModel.cntデータ取得
                self.setControlCont()       #cntFileの内容を取得
                matchControlCont()          #meshとcntFileの整合をとる
                #Tree作成
                makeIniTreeData()
                self.setIniTreeData()
                #材料物性値のDialog表示
                self.readMaterialDB()
                self.setMaterialNameToListBox()
                self.setMaterialModelToComboBox()
                #titleページを表示
                #self.setAnalysisTypeToComboBox()
                showPage = _(u"FrontISTR")
                self.showNoteBook(showPage)
            else:
                title = _(u"workFolderの初期化")
                mess = _(u"必要なファイルが存在しなかったので、コピーしました")
                self.okDialog(title, mess)
                self.setControlCont()
                #self.setAnalysisTypeToComboBox()
                resetCntMshFile()

    #
    #  deleteContactConditions
    #    contactをlistboxから削除
    def deleteContactConditions(self, event):
        names = self.list_box_1.GetStrings()
        delName = self.list_box_1.GetStringSelection()
        if delName == "":
            title = _(u"エラー")
            mess = _(u"削除するpairNameを選択してください")
            self.errDialog(title, mess)
            return
        newNames = []
        for name in names:
            if name != delName:
                newNames.append(name)
        self.list_box_1.Clear()
        self.list_box_1.AppendItems(newNames)

    #
    #  addContactCondition
    #    contactをlistboxに追加
    def addContactCondition(self, event):
        names = self.list_box_1.GetStrings()
        i = 0
        flag = ""
        while flag != "end":
            same = "no"
            for name in names:
                if name == "CP" + str(i):
                    same = "yes"
            if same == "yes":
                i += 1
            else:
                flag = "end"
        newName = "CP" + str(i)
        names.append(newName)
        names.sort()
        self.list_box_1.Clear()
        self.list_box_1.AppendItems(names)
        self.list_box_1.SetStringSelection(newName)

    #
    #  applyDynamicSolution
    #    dynamicSolutionの内容を設定
    def applyDynamicSolution(self, event):
        values = getDynamicContFromCnt()
        if values[0] == "":
            values[0] = "LINEAR"
        if len(values[1]) != 21:
            values[1] = defaultDynamicSolutionCont()
        if values[2] == "":
            values[2] = "1"
        #線形・非線形
        n = self.combo_box_16.GetSelection()
        if n == 0:
            values[0] = "LINEAR"
        else:
            values[0] = "NONLINEAR"
        #運動方程式
        n = self.combo_box_13.GetSelection()
        if n == 0:
            values[1][0] = "1"
        else:
            values[1][0] = "11"
        #解析の種類
        n = self.combo_box_14.GetSelection()
        if n == 0:
            values[1][1] = "1"
        else:
            values[1][1] = "2"
        #開始時刻
        values[1][2] = self.text_ctrl_45.GetValue()
        values[1][3] = self.text_ctrl_46.GetValue()
        values[1][4] = self.text_ctrl_47.GetValue()
        values[1][5] = self.text_ctrl_48.GetValue()
        values[2] = self.text_ctrl_49.GetValue()
        #パラメータ
        values[1][6] = self.text_ctrl_50.GetValue()
        values[1][7] = self.text_ctrl_51.GetValue()
        values[1][10] = self.text_ctrl_44.GetValue()
        values[1][11] = self.text_ctrl_52.GetValue()
        #出力
        values[1][13] = self.combo_box_15.GetValue()
        values[1][14] = self.text_ctrl_53.GetValue()
        if self.checkbox_7.GetValue() == False:
            values[1][15] = "0"
        else:
            values[1][15] = "1"
        if self.checkbox_8.GetValue() == False:
            values[1][16] = "0"
        else:
            values[1][16] = "1"
        if self.checkbox_9.GetValue() == False:
            values[1][17] = "0"
        else:
            values[1][17] = "1"
        if self.checkbox_10.GetValue() == False:
            values[1][18] = "0"
        else:
            values[1][18] = "1"
        if self.checkbox_11.GetValue() == False:
            values[1][19] = "0"
        else:
            values[1][19] = "1"
        if self.checkbox_12.GetValue() == False:
            values[1][20] = "0"
        else:
            values[1][20] = "1"
        #valuesの内容をcntに書き込み
        setDynamicContInCnt(values)


    #
    #  applySolutionType
    #    solutionTypeを設定
    def applySolutionType(self, event):
        global anaType
        orgAnaType = getSolutitionTypeFromCnt()
        anaTypeJ = self.combo_box_12.GetStringSelection()
        i = 0
        for name in analysisTypes():
            if name == anaTypeJ:
                break        
            i += 1
        if i == 0:
            anaType = "STATIC"
        elif i == 1:
            anaType = "NLSTATIC"
        elif i == 2:
            anaType = "DYNAMIC"
        else:
            anaType = "STATIC" 
        if anaType == orgAnaType:
            #設定が変わっていない場合は、何もしない
            return

        else:
            #cntの内容を修正する
            lines = readFistrModelCnt()
            i = pyFistr.getNextHeaderName(lines, 0, "!SOLUTION", "")
            lines[i] = abaqus2fistr.setValName(lines[i], "TYPE="+anaType)
            lines[i+1] = "!" + anaType + "\n"
            writeFistrModelCnt(lines)

    #
    #  applyTemperatureCont
    #    TEMPERATURE内容を設定
    def applyTemperatureCont(self, event):
        #TEMPERATUREを設定する
        refTemp = self.text_ctrl_26.GetValue()
        temp = self.text_ctrl_27.GetValue()
        addLines =  ["!TEMPERATURE\n"]
        addLines += ["ALL, " + temp + "\n"]
        addLines += ["!REFTEMP\n"]
        addLines += [refTemp + "\n"]
        writeTemperatureContInCnt(addLines)

    #
    #  applyGravCont
    #    GRAV内容を設定
    def applyGravCont(self, event):
        #GRAVを設定する
        ax = self.text_ctrl_12.GetValue()
        ay = self.text_ctrl_13.GetValue()
        az = self.text_ctrl_14.GetValue()
        values = [ax, ay, az]
        i = 0
        while i < 3:
            if deleteSp(values[i]) == "":
                values[i] == "0.0"
            i += 1
        x=float(values[0]); y=float(values[1]); z=float(values[2])
        g = pow((x*x + y*y + z*z), 0.5)
        ag = str(g)
        if g != 0:
            ax=str(x/g); ay=str(y/g); az=str(z/g)
        else:
            ax="0.0"; ay="0.0"; az="0.0"
        addLines =  ["!DLOAD, GRPID=1\n"]
        addLines += ["ALL, GRAV, " + ", ".join([ag, ax, ay, az]) + "\n"]
        writeGravContInCnt("ALL", addLines)

    #
    #  applyContact
    #    contactのpairNameを決定
    def applyContact(self, event):
        names = self.list_box_1.GetStrings()
        lines = readFistrModelCnt()
        currPairNames = getContactGroupInCnt(lines)
        #不要なpairNameを削除
        delNames = []
        for currName in currPairNames:
            flag = 0
            for name in names:
                if name == currName:
                    flag = 1
                    break
            if flag != 1:
                delNames.append(currName)
        deleteContactPairNameInCntMsh(delNames)
        #新しいpairNamesを取得
        lines = readFistrModelCnt()
        currPairNames = getContactGroupInCnt(lines)
        newPairNames = []
        for name in names:
            flag = 0
            for currName in currPairNames:
                if name == currName:
                    flag = 1
                    break
            if flag == 0:
                newPairNames.append(name)
        #contact_Algoをチェック、無ければ追加
        i = 0
        flag = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!CONTACT_ALGO")] == "!CONTACT_ALGO":
                flag = 1
                break
            i += 1
        if flag == 0:
            #contact_Algoを追加
            ip = pyFistr.getInsertPointInCnt(lines, "!CONTACT_ALGO", ["Contact"])
            addLines =  ["!CONTACT_ALGO, TYPE=ALAGRANGE\n"]
            lines = lines[:ip] + addLines + lines[ip:]
        #defaultの新しいpairNamesを追加
        ip = pyFistr.getInsertPointInCnt(lines, "!CONTACT", ["Contact"])
        addLines = []
        for pairName in newPairNames:
            addLines += ["!CONTACT, GRPID=1, INTERACTION=FSLID, NTOL=1.0e-5, TTOL=1.0e-3\n"]
            addLines += [pairName + ", 0.0, 0.0\n"]
        lines = lines[:ip] + addLines + lines[ip:]
        #cntを保存
        writeFistrModelCnt(lines)
        #Treeを再作成
        global contactDefine
        contactDefine = getContactGroupInCnt(lines)
        addContactGroupInTreeData(contactDefine)
        treeList = [_(u"境界条件"), _(u"CONTACT（接触）")]
        self.showTreeData(treeList)

    #
    #  applySolverCont
    #    solver内容を決定
    def applySolverCont(self, event):
        method = self.combo_box_11.GetStringSelection()
        precond = self.text_ctrl_39.GetValue()
        nIter = self.text_ctrl_40.GetValue()
        resid = self.text_ctrl_41.GetValue()
        lines = readFistrModelCnt()
        i = pyFistr.getNextHeaderName(lines, 0, "!SOLVER", "")
        lines[i] = abaqus2fistr.setValName(lines[i], "METHOD="+method)
        lines[i] = abaqus2fistr.setValName(lines[i], "PRECOND="+precond)
        i += 1
        words = deleteSp(lines[i]).split(",")
        words[0] = nIter
        lines[i] = ", ".join(words) + "\n"
        i += 1
        words = deleteSp(lines[i]).split(",")
        words[0] = resid
        lines[i] = ", ".join(words) + "\n"
        writeFistrModelCnt(lines)

    #
    #  applyOutputCont
    #    output内容を決定
    def applyOutputCont(self, event):

        #  remakeOutputItems
        #    取得したoutputItemsを元のコマンドに戻す
        def remakeOutputItems(items):
            ans = []
            for item in items:
                #ans.append(item.split(":")[0])
                ans.append(item.split(":")[1])
            return ans

        #freq = self.text_ctrl_33.GetValue()
        selItems = self.list_box_7.GetStrings()
        outputItems = remakeOutputItems(selItems)
        lines = readFistrModelCnt()
        #outputの書き換え
        i = pyFistr.getNextHeaderName(lines, 0, "!OUTPUT_RES", "")
        if i < len(lines):
            lines = pyFistr.delete1header(lines, i)
        else:
            i = pyFistr.getInsertPointInCnt(lines, "!OUTPUT_RES", ["Output"])

        addLines = ["!OUTPUT_RES\n"]
        for item in outputItems:
            addLines += [item + ", ON\n"]
        lines = lines[:i] + addLines + lines[i:]
        writeFistrModelCnt(lines)

    #
    #  applyStepCont
    #    STEP内容を決定
    def applyStepCont(self, event):
        bndList = getBndListFromDefineData()
        stepType = self.combo_box_3.GetStringSelection()
        converg = self.text_ctrl_34.GetValue()
        subSteps = self.text_ctrl_35.GetValue()
        maxIter = self.text_ctrl_36.GetValue()
        dTime = self.text_ctrl_37.GetValue()
        eTime = self.text_ctrl_38.GetValue()
        bnds = self.list_box_4.GetStrings()
        rBnds = reconstructStepBCCont(bnds, bndList)
        values = self.readStepValueFromCnt()
        addLines =  ["!STEP, TYPE=" + stepType + ", CONVERG=" + converg + ", SUBSTEPS=" + subSteps + ", MAXITER=" + maxIter + "\n"]
        if values[7] == "DYNAMIC":
            addLines += [dTime + ", " + eTime + "\n"]
        for bnd in rBnds:
            addLines += bnd
        writeStepContInCnt(addLines)        

    #
    #  applyContactCont
    #    CONTACT内容を決定
    def applyContactCont(self, event):
        newPairName = self.text_ctrl_28.GetValue()
        slave = self.combo_box_4.GetStringSelection()
        master = self.combo_box_9.GetStringSelection()
        if slave == "" or master == "":
            title = _(u"エラー")
            mess = _(u"slave名、または、master名が入力されていません。")
            self.errDialog(title, mess)
            return
        interaction = self.combo_box_10.GetStringSelection()
        nTol = self.text_ctrl_31.GetValue()
        tTol = self.text_ctrl_32.GetValue()
        coeff = self.text_ctrl_29.GetValue()
        factor = self.text_ctrl_30.GetValue()
        deleteContactPairNameInCntMsh([selPairName])
        addContactPairNameInCntMsh(newPairName, slave, master, interaction,
            nTol, tTol, coeff, factor)
        lines = readFistrModelCnt()
        lines = pyFistr.remakeGRPID(lines, "!CONTACT")
        writeFistrModelCnt(lines)
        if newPairName != selPairName:
            #Treeを再作成
            global contactDefine
            newDefine = []
            for name in contactDefine:
                if name == selPairName:
                    newDefine.append(newPairName)
                else:
                    newDefine.append(name)
            contactDefine = newDefine
            addContactGroupInTreeData(contactDefine)
            treeList = [_(u"境界条件"), _(u"CONTACT（接触）"), newPairName]
            self.showTreeData(treeList)
            

    #
    #  applyCentCont
    #    CENT内容を決定
    def applyCentCont(self, event):
        egrpName = self.getElGroupFromLabel(self.label_20)
        values = []
        values.append(self.text_ctrl_15.GetValue())
        values.append(self.text_ctrl_16.GetValue())
        values.append(self.text_ctrl_17.GetValue())
        values.append(self.text_ctrl_18.GetValue())
        values.append(self.text_ctrl_19.GetValue())
        values.append(self.text_ctrl_20.GetValue())
        values.append(self.text_ctrl_21.GetValue())
        i = 0
        while i < 7:
            if pyFistr.deleteSp(values[i]) == "":
                values[i] == "0.0"
            i += 1
        addLines =  ["!DLOAD, GRPID=1\n"]
        addLines += [egrpName + ", CENT, " + ", ".join(values) + "\n"]
        writeCentContInCnt(egrpName, addLines)
        

    #
    #  applyVloadCont
    #    VLOAD内容を決定
    def applyVloadCont(self, event):
        egrpName = self.getElGroupFromLabel(self.label_13)
        values = []
        values.append(self.text_ctrl_9.GetValue())
        values.append(self.text_ctrl_10.GetValue())
        values.append(self.text_ctrl_11.GetValue())
        i = 0
        while i < 3:
            if deleteSp(values[i]) == "":
                values[i] == "0.0"
            i += 1
        addLines =  ["!DLOAD, GRPID=1\n"]
        addLines += [egrpName + ", BX, " + values[0] + "\n"]
        addLines += [egrpName + ", BY, " + values[1] + "\n"]
        addLines += [egrpName + ", BZ, " + values[2] + "\n"]
        writeVloadContInCnt(egrpName, addLines)        

    #
    #  applyDloadCont
    #    DLOAD内容を決定
    def applyDloadCont(self, event):
        sgrpName = self.getElGroupFromLabel(self.label_11)
        dload = self.text_ctrl_8.GetValue()
        if dload == "":
            dload = "0.0"
        addLines =  ["!DLOAD, GRPID=1\n"]
        addLines += [sgrpName + ", S, " + dload + "\n"] 
        writeDloadContInCnt(sgrpName, addLines)

    #
    #  applyCloadCont
    #    CLOAD内容を決定
    def applyCloadCont(self, event):
        ngrpName = self.getElGroupFromLabel(self.label_8)
        cloadCont = self.getCloadContFromTextBox()
        writeCloadContInCnt(ngrpName, cloadCont)

    #
    #  applyBoundaryCont
    #    BOUNDARY内容を決定
    def applyBoundaryCont(self, event):
        ngrpName = self.getElGroupFromLabel(self.label_6)
        bndCont = self.getBoundaryContFromTextBox()
        writeBoundaryContInCnt(ngrpName, bndCont)

    #  getCloadContFromTextBox
    def getCloadContFromTextBox(self):
        ans = []
        val = self.text_ctrl_5.GetValue()
        if val == "":
            val = "0.0"
        ans += ["1, " + val + "\n"]
        val = self.text_ctrl_6.GetValue()
        if val == "":
            val = "0.0"
        ans += ["2, " + val + "\n"]
        val = self.text_ctrl_7.GetValue()
        if val == "":
            val = "0.0"
        ans += ["3, " + val + "\n"]
        return ans

    #  getBoundaryContFromTextBox
    def getBoundaryContFromTextBox(self):
        ans = []
        if self.checkbox_1.GetValue() == True:
            val = self.text_ctrl_2.GetValue()
            if val == "":
                val = "0.0"
            ans += ["1, 1, " + val + "\n"]
        if self.checkbox_2.GetValue() == True:
            val = self.text_ctrl_3.GetValue()
            if val == "":
                val = "0.0"
            ans += ["2, 2, " + val + "\n"]
        if self.checkbox_3.GetValue() == True:
            val = self.text_ctrl_4.GetValue()
            if val == "":
                val = "0.0"
            ans += ["3, 3, " + val + "\n"]
        return ans

    #
    #  applyBCGroup
    #    BCGroupを決定
    def applyBCGroup(self, event):
        global tempDefine, gravDefine, stepDefine
        itemName = self.sizer_155_staticbox.GetLabel()
        names = self.list_box_15.GetStrings()
        if itemName == u"TEMPERATURE":
            if len(names) == 0:
                tempDefine = []
                deleteTemperatureContInCnt()
            else:
                tempDefine = ["ALL"]
                values = self.readTemperatureValueFromCnt()
                if len(values) == 0:
                    temp = "0.0"
                    addLines =  ["!TEMPERATURE\n"]
                    addLines += ["ALL, 0.0\n"]
                    addLines += ["!REFTEMP\n"]
                    addLines += ["0.0\n"]
                    writeTemperatureContInCnt(addLines)
            addTempGroupInTreeData(names)
            treeList = [_(u"境界条件"), _(u"TEMPERATURE（温度）")]
            self.showTreeData(treeList)
        elif itemName == u"GRAV":
            if len(names) == 0:
                gravDefine = []
                deleteGravContInCnt()
            else:
                gravDefine = ["ALL"]
                values = self.readGravValueFromCnt()
                if len(values) == 0:
                    addLines =  ["!DLOAD, GRPID=1\n"]
                    addLines += ["ALL, GRAV, 0.0, 1.0, 0.0, 0.0\n"]
                    writeGravContInCnt("ALL", addLines)
            addGravGroupInTreeData(names)
            treeList = [_(u"境界条件"), _(u"GRAV（重力）")]
            self.showTreeData(treeList)
        elif itemName == u"BOUNDARY":
            addBndGroupInTreeData(names)
            writeDefaultBoundaryInCnt(names)
            treeList = [_(u"境界条件"), _(u"BOUNDARY（変位）")]
            self.showTreeData(treeList)
        elif itemName == u"CLOAD":
            addCloadGroupInTreeData(names)
            writeDefaultCloadInCnt(names)
            treeList = [_(u"境界条件"), _(u"CLOAD（荷重）")]
            self.showTreeData(treeList)
        elif itemName == u"DLOAD":
            addDloadGroupInTreeData(names)
            writeDefaultDloadInCnt(names)
            treeList = [_(u"境界条件"), _(u"DLOAD（圧力）")]
            self.showTreeData(treeList)
        elif itemName == u"VLOAD":
            addVloadGroupInTreeData(names)
            writeDefaultVloadInCnt(names)
            treeList = [_(u"境界条件"), _(u"VLOAD（体積力）")]
            self.showTreeData(treeList)
        elif itemName == u"CENT":
            addCentGroupInTreeData(names)
            writeDefaultCentInCnt(names)
            treeList = [_(u"境界条件"), _(u"CENT（遠心力）")]
            self.showTreeData(treeList)
        elif itemName == u"step":
            if len(names) == 0:
                stepDefine = []
                deleteStepContInCnt()
            else:
                stepDefine = ["STEP"]
                values = self.readStepValueFromCnt()
                if len(values) == 0:
                    addLines =  ["!STEP, TYPE=STATIC, CONVERG=1e-6, SUBSTEP=1, MAXITER=50\n"]
                    writeStepContInCnt(addLines)
            addStepGroupInTreeData(names)
            treeList = [_(u"ステップ解析")]
            self.showTreeData(treeList)

    #
    #  applyEachMaterial
    #    各groupの材料を設定
    def applyEachMaterial(self):
        line = self.label_44.GetLabel()
        elGroup = line.split(":")[1]
        #matName = self.combo_box_5.GetStringSelection()
        matName = self.combo_box_5.GetValue()
        elastic = getMaterialElasticValue(matName)
        if len(elastic) == 0:
            title = _(u"エラー")
            mess = _(u"設定されている材料名は、材料データベース内に存在しません")
            self.errDialog(title, mess)
            return
        matModel = self.combo_box_6.GetStringSelection()
        matType = ""; matHard = ""
        if matModel != "ELASTIC":
            matType = self.combo_box_7.GetStringSelection()
            matHard = self.combo_box_8.GetStringSelection()
        if matModel == "PLASTIC":
            ssData = getPlasticSsData(matName)
            if ssData == "":
                title = _(u"エラー")
                mess = _(u"塑性data（SS_curve）が入力されていません。")
                self.errDialog(title, mess)
                return
            writeFistrModelFile(elGroup, matName, elastic, matModel, matType,
                matHard, ssData)
        else:
            writeFistrModelFile(elGroup, matName, elastic, matModel, matType,
                matHard, "")
        self.remakeDefineMaterial()

    #
    #  remakeDefineMaterial
    #    材料設定を再修正
    #    余分な材料設定を削除、cntの材料定義順をmshファイルに合わせる
    def remakeDefineMaterial(self):

        #  1ヶ分の材料定義内容を取得
        def get1MaterialDataInCnt(lines, i):
            newLines = []
            matGrp = matGrpHeader   #material定義内のheader名のlist
            while i < len(lines):
                if lines[i][0] == "!":
                    flag = 0
                    for header in matGrp:
                        if lines[i][:len(header)] == header:
                            newLines += [lines[i]]
                            flag = 1
                            break
                    if flag == 0:
                        break
                else:
                    if lines[i][0] != "#":
                        newLines += [lines[i]]
                i += 1
            return [newLines, i]

        #  全ての材料定義内容を取得
        def getAllMaterialDataInCnt(lines):
            matLines = []
            i = pyFistr.getNextHeaderName(lines, 0, "!MATERIAL", "")
            while i < len(lines):
                name = abaqus2fistr.getValName(lines[i], "NAME")
                conts = [lines[i]]
                i += 1
                [matLine, i] = get1MaterialDataInCnt(lines, i)
                matLines += [[name, conts+matLine]]
                i = pyFistr.getNextHeaderName(lines, i, "!MATERIAL", "")
            return matLines

        #  mshファイル内の材料定義順で、cntファイル内に材料定義を追加する
        def addSettingMaterial(lines, matNames, matLines):
            addLines = []
            for matName in matNames:
                for matLine in matLines:
                    if matName == matLine[0]:
                        addLines += matLine[1]
                        break
            i = pyFistr.getInsertPointInCnt(lines, "!MATERIAL", ["Material"])
            lines = lines[:i] + addLines + lines[i:]
            return lines

        global matDefine
        #matDefineを取得
        lines = readFistrModelMsh()
        matDefine = createMaterialDefine(lines)
        matNames = []
        for name in matDefine:
            matNames.append(name[1])
        #FistrModel.mshの修正
        lines = deleteMaterialWithoutNames(lines, matNames)
        writeFistrModelMsh(lines)
        #FistrModel.cntの修正
        lines = readFistrModelCnt()
        matLines = getAllMaterialDataInCnt(lines)
        lines = deleteMaterialWithoutNames(lines, [])
        lines = addSettingMaterial(lines, matNames, matLines)
        writeFistrModelCnt(lines)

    #
    #  setEachMaterialTypeHard
    #    材料タイプ、硬化則をcomboBoxに再セット
    def setEachMaterialTypeHard(self):
        matTypes = []
        matHards = []
        matModel = self.combo_box_6.GetStringSelection()
        for matMTH in materialType:
            if matModel == matMTH[0]:
                matTypes = matMTH[1]
                matHards = matMTH[2]
                break
        if len(matTypes) == 0:
            self.combo_box_7.Clear()
            self.combo_box_7.Enable(False)
        else:
            self.combo_box_7.Enable(True)
            self.combo_box_7.Clear()
            self.combo_box_7.AppendItems(matTypes)
            self.combo_box_7.Select(0)
        if len(matHards) == 0:
            self.combo_box_8.Clear()
            self.combo_box_8.Enable(False)
        else:
            self.combo_box_8.Enable(True)
            self.combo_box_8.Clear()
            self.combo_box_8.AppendItems(matHards)
            self.combo_box_8.Select(0)
        #塑性ボタンのActive
        if matModel == "PLASTIC":
            self.label_85.Enable(True)
            self.button_34.Enable(True)
        else:
            self.label_85.Enable(False)
            self.button_34.Enable(False)

    #
    #  applyMaterial
    #    setした内容をFistrModel.cntに設定する
    def applyMaterial(self, event):
        global matDefine
        #elGroupを取得
        names = self.list_box_6.GetStrings()
        matDefine = []
        for name in names:
            matDefine.append([name, ""])
        #treeを表示
        addMatGroupInTreeData(names)
        treeList = [_(u"材料物性値")]
        self.showTreeData(treeList)
        #材料定義を修正
        lines = readFistrModelCnt()
        lines = deleteMaterialWithoutNames(lines, names)
        matNames = getMatNamesInCnt(lines)
        writeFistrModelCnt(lines)
        lines = readFistrModelMsh()
        lines = deleteMaterialWithoutNames(lines, names)
        lines = deleteMaterialInSectionWithoutNames(lines, matNames)
        writeFistrModelMsh(lines)

    #
    #  setMaterialGroup
    #    elGroupを選択
    def setMaterialGroup(self, event):
        self.moveItemToListBox(self.list_box_5, self.list_box_6)

    #
    #  returnMaterialGroup
    #    elGroupを戻す
    def returnMaterialGroup(self, event):
        self.moveItemToListBox(self.list_box_6, self.list_box_5)

    #
    #  moveItemToListBox
    #    選択したitemをlistBox1からlistBox2に移動する
    def moveItemToListBox(self, listBox1, listBox2):
        selName = listBox1.GetStringSelection()
        if selName == "":
            title = _(u"groupの選択")
            mess = _(u"group名を選択してください")
            self.errDialog(title, mess)
            return
        #元listBoxからselNameを削除
        n = listBox1.GetSelection()
        listBox1.Delete(n)
        #先listBoxにselNameを追加
        items = listBox2.GetStrings()
        items += [selName]
        items.sort()
        listBox2.Clear()
        listBox2.AppendItems(items)
        listBox2.SetStringSelection(selName)


    #
    #  setMaterialDB
    #    材料DBの場所を設定
    def setMaterialDB(self, event):
        global materialDBName
        if len(glob.glob(materialDBName)) == 0:
            dbDir = os.getcwd()
        else:
            dbDir = "/".join(materialDBName.split("/")[:-1])
        fileType = "*.csv"
        dlg = wx.FileDialog(self, _(u"ファイルを選択してください"), dbDir, "", fileType, wx.OPEN)
        stat = dlg.ShowModal()
        if stat == wx.ID_OK:
            name = dlg.GetFilename()
            dbDir = dlg.GetDirectory()
            materialDBName = dbDir + "/" + name
            self.text_ctrl_22.SetValue(materialDBName)
            saveIstrData(["materialDB "+materialDBName])
            self.readMaterialDB()
            self.setMaterialNameToListBox()

    #
    #  selectItem
    #    Treeの項目選択
    def selectItem(self, event):
        treeCtrl = self.tree_ctrl_1
        item = treeCtrl.GetSelection()
        name = treeCtrl.GetItemText(item)
        parent = treeCtrl.GetItemParent(item)
        parentName = treeCtrl.GetItemText(parent)
        showPage = self.getShowPageName(parentName, name)
        if showPage == _(u"材料物性値"):
            self.readSetMaterialType()
            self.setElGroupToListBoxInMaterial()
        elif showPage == _(u"材料物性値_sub"):
            self.readSetMaterialType()
            self.setElGroupToLabel(self.label_44, name)
            self.readSetEachMaterialName(name)
        elif showPage == _(u"BOUNDARY"):
            self.setBCGroupToListBox(showPage)
            #self.setBoundaryGroupToListBox()
        elif showPage == _(u"BOUNDARY_sub"):
            self.setBoundary_sub(name)
        elif showPage == _(u"CLOAD"):
            self.setBCGroupToListBox(showPage)
            #self.setCloadGroupToListBox()
        elif showPage == _(u"CLOAD_sub"):
            self.setCload_sub(name)
        elif showPage == _(u"DLOAD"):
            self.setBCGroupToListBox(showPage)
            #self.setDloadGroupToListBox()
        elif showPage == _(u"DLOAD_sub"):
            self.setDload_sub(name)
        elif showPage == _(u"VLOAD"):
            self.setBCGroupToListBox(showPage)
            #self.setVloadGroupToListBox()
        elif showPage == _(u"VLOAD_sub"):
            self.setVload_sub(name)
        elif showPage == _(u"GRAV"):
            self.setBCGroupToListBox(showPage)
        elif showPage == _(u"GRAV_sub"):
            self.setGravToListBox(name)
        elif showPage == _(u"CENT"):
            #self.setCentGroupToListBox()
            self.setBCGroupToListBox(showPage)
        elif showPage == _(u"CENT_sub"):
            self.setCent_sub(name)
        elif showPage == _(u"TEMPERATURE"):
            self.setBCGroupToListBox(showPage)
        elif showPage == _(u"TEMPERATURE_sub"):
            self.setTemperatureToListBox()
        elif showPage == _(u"contact"):
            self.setContactGroupToListBox()
        elif showPage == _(u"contact_sub"):
            self.setContact_sub(name)
        elif showPage == _(u"step"):
            #self.setStepGroupToListBox()
            self.setBCGroupToListBox(showPage)
        elif showPage == _(u"step_sub"):
            self.setStep_sub()
        elif showPage == _(u"solver"):
            self.setSolverPage()
        elif showPage == _(u"output"):
            self.setOutputPage()
        elif showPage == _(u"solutionType"):
            self.setSolutionTypePage(name)
        self.showNoteBook(showPage)

    #  getShowPageName
    def getShowPageName(self, parentName, name):
        pageName = ""
        if parentName == _(u"材料物性値"):
            pageName = _(u"材料物性値_sub")
        elif parentName == _(u"BOUNDARY（変位）"):
            pageName = _(u"BOUNDARY_sub")
        elif parentName == _(u"CLOAD（荷重）"):
            pageName = _(u"CLOAD_sub")
        elif parentName == _(u"DLOAD（圧力）"):
            pageName = _(u"DLOAD_sub")
        elif parentName == _(u"VLOAD（体積力）"):
            pageName = _(u"VLOAD_sub")
        elif parentName == _(u"GRAV（重力）"):
            pageName = _(u"GRAV_sub")
        elif parentName == _(u"CENT（遠心力）"):
            pageName = _(u"CENT_sub")
        elif parentName == _(u"TEMPERATURE（温度）"):
            pageName = _(u"TEMPERATURE_sub")
        elif parentName == _(u"CONTACT（接触）"):
            pageName = _(u"contact_sub")
        elif parentName == _(u"ステップ解析"):
            pageName = _(u"step_sub")
        elif parentName == _(u"解析の種類"):
            pageName = _(u"solutionType")
        else:
            if name == _(u"solver"):
                pageName = _(u"solver") 
            elif name == _(u"FrontISTR anarysis"):
                pageName = _("FrontISTR")
            elif name == _(u"FistrModel.msh"):
                pageName = _(u"FistrModel")
            elif name == _(u"解析の種類"):
                pageName = _(u"solutionType")
            elif name == _(u"境界条件"):
                pageName = _(u"境界条件")
            elif name == _(u"材料物性値"):
                pageName = _(u"材料物性値")
            elif name == _(u"BOUNDARY（変位）"):
                pageName = _(u"BOUNDARY")
            elif name == _(u"CLOAD（荷重）"):
                pageName = _(u"CLOAD")
            elif name == _(u"DLOAD（圧力）"):
                pageName = _(u"DLOAD")
            elif name == _(u"VLOAD（体積力）"):
                pageName = _(u"VLOAD")
            elif name == _(u"GRAV（重力）"):
                pageName = _(u"GRAV")
            elif name == _(u"CENT（遠心力）"):
                pageName = _(u"CENT")
            elif name == _(u"post"):
                pageName = _(u"post")
            elif name == _(u"TEMPERATURE（温度）"):
                pageName = _(u"TEMPERATURE")
            elif name == _(u"CONTACT（接触）"):
                pageName = _(u"contact")
            elif name == _(u"ステップ解析"):
                pageName = _(u"step")
            elif name == _(u"出力"):
                pageName = _(u"output")
        return pageName

    #  showNoteBook
    def showNoteBook(self, name):
        if name == u"TEMPERATURE":
            name = u"setGroup"
        elif name == u"GRAV":
            name = u"setGroup"
        elif name == u"BOUNDARY":
            name = u"setGroup"
        elif name == u"CLOAD":
            name = u"setGroup"
        elif name == u"DLOAD":
            name = u"setGroup"
        elif name == u"VLOAD":
            name = u"setGroup"
        elif name == u"CENT":
            name = u"setGroup"
        elif name == u"step":
            name = u"setGroup"
        pageText = ""
        nPage = self.notebook_1.GetPageCount()
        i = 0
        while i < nPage:
            pageName = self.notebook_1.GetPageText(i)
            if pageName == name:
                self.notebook_1.SetSelection(i)
                break
            i += 1

    #
    #  setStepGroupToListBox
    #
    def setStepGroupToListBox(self):
        self.list_box_2.Clear()
        self.list_box_2.AppendItems(stepDefine)

    #
    #  setContactGroupToListBox
    #
    def setContactGroupToListBox(self):
        contactAlgos = [_(u"SLAGRANGE（Lagrange法）"), _(u"ALAGRANGE（拡張Lagrange法）")]
        self.combo_box_2.Clear()
        self.combo_box_2.AppendItems(contactAlgos)
        selAlgo = getContactAlgoInCnt()
        selNo = 1
        if selAlgo == "SLAGRANGE":
            selNo = 0
        elif selAlgo == "ALAGRANGE":
            selNo = 1
        self.combo_box_2.SetSelection(selNo)
        self.list_box_1.Clear()
        self.list_box_1.AppendItems(contactDefine)

    #
    #  setTemperatureToListBox
    #
    def setTemperatureToListBox(self):
        values = self.readTemperatureValueFromCnt()
        if len(values) == 0:
            temp = "0.0"
            refTemp = "0.0"
        else:
            temp = values[0]
            refTemp = values[1]
        self.text_ctrl_26.SetValue(refTemp)
        self.text_ctrl_27.SetValue(temp)

    #
    #  setGravToListBox
    #
    def setGravToListBox(self, name):
        values = self.readGravValueFromCnt()
        if len(values) == 0:
            self.checkbox_4.SetValue(False)
            val = "0.0"
            ax = "0.0"
            ay = "0.0"
            az = "0.0"
        else:
            #self.checkbox_4.SetValue(True)
            g = float(values[0])
            x = float(values[1])
            y = float(values[2])
            z = float(values[3])
            if g != 0:
                l = pow((x*x + y*y + z*z), 0.5)
                ax = str(x/l*g)
                ay = str(y/l*g)
                az = str(z/l*g)
            else:
                ax="0.0"; ay="0.0"; az="0.0"
        self.text_ctrl_12.SetValue(ax)
        self.text_ctrl_13.SetValue(ay)
        self.text_ctrl_14.SetValue(az)

    #
    #  setSolutionTypePage
    #    solutionTypePageをセット
    def setSolutionTypePage(self, name):
        global anaType
        anaType = getSolutitionTypeFromCnt()
        #panelを設定
        if name == _(u"設定"):
            if anaType == "STATIC":
                self.panel_static.Show()
                self.panel_solution.Hide()
                self.panel_dynamic.Hide()
                self.label_99.SetLabel(_(u"静解析の設定"))
                sizer_191.Layout()
            elif anaType == "DYNAMIC":
                self.panel_static.Hide()
                self.panel_dynamic.Show()
                self.panel_solution.Hide()
                self.label_89.SetLabel(_(u"動解析の設定"))
                sizer_191.Layout()
                sizer_195.Layout()
                self.setDynamicContToListBox()
            elif anaType == "NLSTATIC":
                self.panel_static.Show()
                self.panel_solution.Hide()
                self.panel_dynamic.Hide()
                self.label_99.SetLabel(_(u"非線形静解析の設定"))
                sizer_191.Layout()
        else:
            self.panel_solution.Show()
            self.panel_static.Hide()
            self.panel_dynamic.Hide()
            self.label_88.SetLabel(_(u"解析の種類を設定"))
            sizer_191.Layout()
            self.setAnalysisTypeToComboBox()

    #
    #  setDynamicCont
    #    動解析の内容をセット
    def setDynamicContToListBox(self):
        #comboBoxをセット
        self.combo_box_16.Clear()
        self.combo_box_16.AppendItems([_(u"線形解析"), _(u"非線形解析")])
        self.combo_box_13.Clear()
        self.combo_box_13.AppendItems([_(u"陰解法（Newmark-β法）"), _(u"陽解法（中央差分法）")])
        self.combo_box_14.Clear()
        self.combo_box_14.AppendItems([_(u"時刻歴応答解析"), _(u"周波数応答解析")])
        nodeGrp = meshGroup[2]
        self.combo_box_15.Clear()
        self.combo_box_15.AppendItems(nodeGrp)
        #開始・終了時間を隠す
        self.text_ctrl_45.Enable(False)
        self.text_ctrl_46.Enable(False)
        #dataを取得
        values = getDynamicContFromCnt()
        if values[0] == "LINEAR" or values[0] == "NONLINEAR":
            pass
        else:
            values[0] == "LINEAR"
        if len(values[1]) == 21:
            pass
        else:
            values[1] = defaultDynamicSolutionCont()
        #線形・非線形
        self.combo_box_16.SetSelection(0)
        if values[0] == "NONLINEAR":
            self.combo_box_16.SetSelection(1)
        data = values[1]
        #運動方程式
        self.combo_box_13.SetSelection(0)
        if data[0] == "11":
            self.combo_box_13.SetSelection(1)
        #解析の種類
        self.combo_box_14.SetSelection(0)
        if data[1] == "2":
            self.combo_box_14.SetSelection(1)
        #開始時間等
        self.text_ctrl_45.SetValue(data[2])
        self.text_ctrl_46.SetValue(data[3])
        self.text_ctrl_47.SetValue(data[4])
        self.text_ctrl_48.SetValue(data[5])
        self.text_ctrl_49.SetValue(values[2])
        #パラメータ
        self.text_ctrl_50.SetValue(data[6])
        self.text_ctrl_51.SetValue(data[7])
        self.text_ctrl_44.SetValue(data[10])
        self.text_ctrl_52.SetValue(data[11])
        #モニタ
        self.combo_box_15.SetValue(data[13])
        self.text_ctrl_53.SetValue(data[14])
        self.checkbox_7.SetValue(False)
        self.checkbox_8.SetValue(False)
        self.checkbox_9.SetValue(False)
        self.checkbox_10.SetValue(False)
        self.checkbox_11.SetValue(False)
        self.checkbox_12.SetValue(False)
        if data[15] == "1":
            self.checkbox_7.SetValue(True)
        if data[16] == "1":
            self.checkbox_8.SetValue(True)
        if data[17] == "1":
            self.checkbox_9.SetValue(True)
        if data[18] == "1":
            self.checkbox_10.SetValue(True)
        if data[19] == "1":
            self.checkbox_11.SetValue(True)
        if data[20] == "1":
            self.checkbox_12.SetValue(True)

    #
    #  setOutputPage
    #
    def setOutputPage(self):
        values = getOutputContFromCnt()
        #freq = values[0]
        noSelItems = values[0]
        selItems = values[1]
        #self.text_ctrl_33.SetValue(freq)
        self.list_box_2.Clear()
        self.list_box_2.AppendItems(noSelItems)
        self.list_box_7.Clear()
        self.list_box_7.AppendItems(selItems)

    #
    #  setSolverPage
    #
    def setSolverPage(self):
        values = getSolverContFromCnt()
        method = values[0]
        precond = values[1]
        nIter = values[2]
        resid = values[3]
        iterLog = values[4]
        timeLog = values[5]
        freq = values[6]
        solvers = solveMethod
        self.combo_box_11.Clear()
        self.combo_box_11.AppendItems(solvers)
        self.combo_box_11.SetStringSelection(method)
        self.text_ctrl_39.SetValue(precond)
        self.text_ctrl_40.SetValue(nIter)
        self.text_ctrl_41.SetValue(resid)
        if iterLog == "YES":
            self.checkbox_4.SetValue(True)
        else:
            self.checkbox_4.SetValue(False)
        if timeLog == "YES":
            self.checkbox_5.SetValue(True)
        else:
            self.checkbox_5.SetValue(False)
        values = getHecmwContFromDat()
        parallel = values[0]
        nCpu = values[1]
        self.checkbox_6.SetValue(parallel)
        self.text_ctrl_42.SetValue(nCpu)
        self.text_ctrl_43.SetValue(freq)

    #
    #  setStep_sub
    #
    def setStep_sub(self):

        def getNoBnds(allBnds, cLines):
            noLines = []
            for allBnd in allBnds:
                allLine = ",".join(allBnd[:2])
                flag = 0
                for cLine in cLines:
                    if allLine == cLine:
                        flag = 1
                        break
                if flag == 0:
                    noLines.append(allLine)
            return noLines

        bndList = getBndListFromDefineData()
        stepTypes = [u"STATIC", u"VISCO"]
        self.combo_box_3.Clear()
        self.combo_box_3.AppendItems(stepTypes)
        values = self.readStepValueFromCnt()
        stepType = values[0]
        converg = values[1]
        subSteps = values[2]
        maxIter = values[3]
        dTime = values[4]
        eTime = values[5]
        bnds = values[6]
        solutionType = values[7]
        cBnds = reformStepBCCont(bnds, bndList)
        noBnds = getNoBnds(bndList, cBnds)
        self.combo_box_3.SetStringSelection(stepType)
        self.text_ctrl_34.SetValue(converg)
        self.text_ctrl_35.SetValue(subSteps)
        self.text_ctrl_36.SetValue(maxIter)
        if solutionType == "DYNAMIC":
            self.text_ctrl_37.Enable(True)
            self.text_ctrl_38.Enable(True)
            self.text_ctrl_37.SetValue(dTime)
            self.text_ctrl_38.SetValue(eTime)
        else:
            self.text_ctrl_37.Enable(False)
            self.text_ctrl_38.Enable(False)
        self.list_box_3.Clear()
        self.list_box_3.AppendItems(noBnds)
        self.list_box_4.Clear()
        self.list_box_4.AppendItems(cBnds)



    #
    #  setContact_sub
    #
    def setContact_sub(self, name):
        global selPairName
        #pairName
        self.text_ctrl_28.SetValue(name)
        selPairName = name
        #conts
        conts = getContactContInCntMsh(name)
        slave = conts[0]
        master = conts[1]
        interaction = conts[2]
        nTol = conts[3]
        tTol = conts[4]
        coeff = conts[5]
        factor = conts[6]
        interactions = ["SSLID", "FSLID"]
        self.combo_box_10.Clear()
        self.combo_box_10.AppendItems(interactions)
        self.combo_box_10.SetStringSelection(interaction)
        self.text_ctrl_31.SetValue(nTol)
        self.text_ctrl_32.SetValue(tTol)
        self.text_ctrl_29.SetValue(coeff)
        self.text_ctrl_30.SetValue(factor) 
        #slave
        self.combo_box_4.Clear()
        self.combo_box_4.AppendItems(meshGroup[2]+[""])
        self.combo_box_4.SetStringSelection(slave)
        #master
        self.combo_box_9.Clear()
        self.combo_box_9.AppendItems(meshGroup[1]+[""])
        self.combo_box_9.SetStringSelection(master)

    #
    #  setCent_sub
    #
    def setCent_sub(self, name):
        self.setElGroupToLabel(self.label_20, name)
        values = self.readCentEgrpName(name)
        self.text_ctrl_15.SetValue(values[0])
        self.text_ctrl_16.SetValue(values[1])
        self.text_ctrl_17.SetValue(values[2])
        self.text_ctrl_18.SetValue(values[3])
        self.text_ctrl_19.SetValue(values[4])
        self.text_ctrl_20.SetValue(values[5])
        self.text_ctrl_21.SetValue(values[6])

    #
    #  setVload_sub
    #
    def setVload_sub(self, name):
        self.setElGroupToLabel(self.label_13, name)
        values = self.readVloadEgrpName(name)
        self.text_ctrl_9.SetValue(values[0])
        self.text_ctrl_10.SetValue(values[1])
        self.text_ctrl_11.SetValue(values[2])

    #
    #  setDload_sub
    #    pageの初期化
    def setDload_sub(self, name):
        self.setElGroupToLabel(self.label_11, name)
        values = self.readDloadSgrpName(name)
        self.text_ctrl_8.SetValue(values[0])

    #
    #  setCload_sub
    #    pageの初期化
    def setCload_sub(self, name):
        self.setElGroupToLabel(self.label_8, name)
        [x, y, z] = self.readCloadNgrpName(name)
        self.setCloadContToTextBox([x, y, z])

    #
    #  setBoundary_sub
    #    pageの初期化
    def setBoundary_sub(self, name):
        self.setElGroupToLabel(self.label_6, name)
        [x, y, z] = self.readBoundaryNgrpName(name)
        self.setBoundaryContToTextBox([x, y, z])

    #  setCloadContToTextBox
    def setCloadContToTextBox(self, bndCont):
        x = bndCont[0]
        y = bndCont[1]
        z = bndCont[2]
        if len(x) > 0:
            v = bndCont[0][1]
            self.text_ctrl_5.SetValue(v)
        else:
            self.text_ctrl_5.SetValue("0.0")
        if len(y) > 0:
            v = bndCont[1][1]
            self.text_ctrl_6.SetValue(v)
        else:
            self.text_ctrl_6.SetValue("0.0")
        if len(z) > 0:
            v = bndCont[2][1]
            self.text_ctrl_7.SetValue(v)
        else:
            self.text_ctrl_7.SetValue("0.0")

    #  setBoundaryContToTextBox
    def setBoundaryContToTextBox(self, bndCont):
        x = bndCont[0]
        y = bndCont[1]
        z = bndCont[2]
        if len(x) > 0:
            v = bndCont[0][1]
            self.checkbox_1.SetValue(True)
            self.text_ctrl_2.SetValue(v)
        else:
            self.checkbox_1.SetValue(False)
            self.text_ctrl_2.SetValue("")
        if len(y) > 0:
            v = bndCont[1][1]
            self.checkbox_2.SetValue(True)
            self.text_ctrl_3.SetValue(v)
        else:
            self.checkbox_2.SetValue(False)
            self.text_ctrl_3.SetValue("")
        if len(z) > 0:
            v = bndCont[2][1]
            self.checkbox_3.SetValue(True)
            self.text_ctrl_4.SetValue(v)
        else:
            self.checkbox_3.SetValue(False)
            self.text_ctrl_4.SetValue("")

    #
    #  readStepValueFromCnt
    #    STEPの値を取得
    def readStepValueFromCnt(self):
        values = []
        lines = readFistrModelCnt()
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!STEP")] == "!STEP":
                stepType = abaqus2fistr.getValName(line, "TYPE")
                if stepType == "":
                    stepType = "STATIC"
                converg = abaqus2fistr.getValName(line, "CONVERG")
                if converg == "":
                    converg = "1.0e-6"
                subSteps = abaqus2fistr.getValName(line, "SUBSTEPS")
                if subSteps == "":
                    subSteps = "1"
                maxIter = abaqus2fistr.getValName(line, "MAXITER")
                if maxIter == "":
                    maxIter = "50"
                i += 1
                words = deleteSp(lines[i]).split(",")
                flag = 0
                try:
                    dTime = str(float(words[0]))
                    eTime = str(float(words[1]))
                    flag = 1
                except:
                    dTime = "1.0"
                    eTime = "1.0"
                if flag == 1:
                    i += 1
                bnds = []
                while i < len(lines):
                    line = lines[i]
                    if line[0] != "!":
                        if line[0] != "#":
                            bnds.append(line)
                    else:
                        break
                    i += 1
                values = [stepType, converg, subSteps, maxIter, dTime, eTime, bnds]
                break
            i += 1
        if len(values) > 0:
            #SOLUTIONのTYPEを追加
            solutionType = "STATIC"
            for line in lines:
                if line[:len("!SOLUTION")] == "!SOLUTION":
                    solutionType = abaqus2fistr.getValName(line, "TYPE")
                    break
            values += [solutionType]
        return values

    #
    #  readTemperatureValueFromCnt
    #    TEMPERATUREの値を取得
    def readTemperatureValueFromCnt(self):
        values = []
        temp = ""
        refTemp = ""
        lines = readFistrModelCnt()
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!TEMPERATURE")] == "!TEMPERATURE":
                temp = deleteSp(lines[i+1]).split(",")[1]
                break
            i += 1
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!REFTEMP")] == "!REFTEMP":
                refTemp = deleteSp(lines[i+1].split(",")[0])
                break
            i += 1
        if temp != "" and refTemp != "":
            values = [temp, refTemp]
        return values

    #
    #  readGravValueFromCnt
    #    GRAVの値を取得
    def readGravValueFromCnt(self):
        lines = readFistrModelCnt()
        ansLine = ""
        values = []
        i = 0
        while i < len(lines):
            grpTypes = pyFistr.getNextDloadGrpType(lines, i)
            if len(grpTypes) > 0:
                for [grp, bndType, line, i] in grpTypes:
                    if grp == "ALL" and bndType == "GRAV":
                        ansLine = line
                i += 1
            else:
                break
        if ansLine != "":
            values = deleteSp(ansLine).split(",")[2:]
        return values

    #
    #  readCentEgrpName
    #    EGRP名のCENT値を取得
    def readCentEgrpName(self, name):
        lines = readFistrModelCnt()
        getLine = ""
        values = []
        i = 0
        while i < len(lines):
            grpTypes = pyFistr.getNextDloadGrpType(lines, i)
            if len(grpTypes) > 0:
                if len(grpTypes) == 1:
                    [grp, bndType, line, i] = grpTypes[0]
                    if grp == name and bndType == "CENT":
                        getLine = line
                        break
            else:
                break
            i += 1        
        if getLine != "":
            words = getLine.split(",")
            values = words[2:]
        return values

    #
    #  readVloadEgrpName
    #    EGRP名のVLOAD値を取得
    def readVloadEgrpName(self, name):
        lines = readFistrModelCnt()
        bxLine = ""
        byLine = ""
        bzLine = ""
        values = []
        i = 0
        while i < len(lines):
            grpTypes = pyFistr.getNextDloadGrpType(lines, i)
            if len(grpTypes) > 0:
                for [grp, bndType, line, i] in grpTypes:
                    if grp == name and bndType == "BX":
                        bxLine = line
                    elif grp == name and bndType == "BY":
                        byLine = line
                    elif grp == name and bndType == "BZ":
                        bzLine = line
                i += 1
            else:
                break
        bx = "0.0"; by = "0.0"; bz = "0.0"
        if bxLine != "":
            bx = deleteSp(bxLine).split(",")[2]
        if byLine != "":
            by = deleteSp(byLine).split(",")[2]
        if bzLine != "":
            bz = deleteSp(bzLine).split(",")[2]
        values = [bx, by, bz]
        return values

    #
    #  readDloadSgrpName
    #    SGRP名のDLOADを取得
    def readDloadSgrpName(self, name):
        lines = readFistrModelCnt()
        ansLine = ""
        values = []
        i = 0
        while i < len(lines):
            grpTypes = pyFistr.getNextDloadGrpType(lines, i)
            if len(grpTypes) > 0:
                for [grp, bndType, line, i] in grpTypes:
                    if grp == name and bndType == "S":
                        ansLine = line
                i += 1
            else:
                break
        if ansLine != "":
            values = deleteSp(ansLine).split(",")[2:]
        return values

    #
    #  readCloadNgrpName
    #    NGRP名のCLOADを取得
    def readCloadNgrpName(self, name):

        def getBndLines(lines, i, name):
            newLine = []
            while i < len(lines):
                line = deleteSp(lines[i])
                ngrp = line.split(",")[0]
                if ngrp == name:
                    newLine.append(line)
                else:
                    break
                i += 1 
            return newLine

        def remakeBndCont(lines):
            x = []; y = []; z = []
            for line in lines:
                words = line.split(",")
                if words[1] == "1":
                    x += [words[1]] + [words[2]]
                elif words[1] == "2":
                    y += [words[1]] + [words[2]]
                elif words[1] == "3":
                    z += [words[1]] + [words[2]]
            return [x, y, z]

        lines = readFistrModelCnt()
        i = 0
        bndLines = [[], [], []]
        while i < len(lines):
            line = lines[i]
            if line[:len("!CLOAD")] == "!CLOAD":
                i += 1
                line = deleteSp(lines[i])
                ngrp = line.split(",")[0]
                if ngrp == name:
                    bndLines = getBndLines(lines, i, name)
                    break
            i += 1
        ans = remakeBndCont(bndLines)
        return ans

    #
    #  readBoundaryNgrpName
    #    NGRP名のBOUNDARYを取得
    def readBoundaryNgrpName(self, name):

        def getBndLines(lines, i, name):
            newLine = []
            while i < len(lines):
                line = deleteSp(lines[i])
                ngrp = line.split(",")[0]
                if ngrp == name:
                    newLine.append(line)
                else:
                    break
                i += 1 
            return newLine

        def remakeBndCont(lines):
            x = []; y = []; z = []
            for line in lines:
                words = line.split(",")
                if words[1] == "1":
                    x += [words[1]] + [words[3]]
                elif words[1] == "2":
                    y += [words[1]] + [words[3]]
                elif words[1] == "3":
                    z += [words[1]] + [words[3]]
            return [x, y, z]
                

        lines = readFistrModelCnt()
        i = 0
        bndLines = [[], [], []]
        while i < len(lines):
            line = lines[i]
            if line[:len("!BOUNDARY")] == "!BOUNDARY":
                i += 1
                line = deleteSp(lines[i])
                ngrp = line.split(",")[0]
                if ngrp == name:
                    bndLines = getBndLines(lines, i, name)
                    break
            i += 1
        ans = remakeBndCont(bndLines)
        return ans


    #
    #  setBCGroupToListBox
    #    BCGroup選択時のpageの初期化
    def setBCGroupToListBox(self, showPage):
        global tempDefine
        self.sizer_155_staticbox.SetLabel(showPage)
        if showPage == _(u"TEMPERATURE"):
            values = self.readTemperatureValueFromCnt()
            if len(values) == 0:
                self.list_box_14.Clear()
                self.list_box_14.AppendItems(["ALL"])
                self.list_box_15.Clear()
            else:
                self.list_box_14.Clear()
                self.list_box_15.Clear()
                self.list_box_15.AppendItems(["ALL"])
        elif showPage == _(u"GRAV"):
            values = self.readGravValueFromCnt()
            if len(values) == 0:
                self.list_box_14.Clear()
                self.list_box_14.AppendItems(["ALL"])
                self.list_box_15.Clear()
            else:
                self.list_box_14.Clear()
                self.list_box_15.Clear()
                self.list_box_15.AppendItems(["ALL"])
        elif showPage == _(u"BOUNDARY"):
            global bndDefine
            lines = readFistrModelCnt()
            bndDefine = getBoundaryGroupInCnt(lines)
            ngrps = meshGroup[2]            #nodeGroup
            noBnd = self.getNoGroups(ngrps, bndDefine)      
            self.list_box_14.Clear()
            self.list_box_14.AppendItems(noBnd)
            self.list_box_15.Clear()
            self.list_box_15.AppendItems(bndDefine)
        elif showPage == _(u"CLOAD"):
            global cloadDefine
            lines = readFistrModelCnt()
            cloadDefine = getCloadGroupInCnt(lines)
            ngrps = meshGroup[2]
            cloads = cloadDefine
            noCload = self.getNoGroups(ngrps, cloadDefine)
            self.list_box_14.Clear()
            self.list_box_14.AppendItems(noCload)
            self.list_box_15.Clear()
            self.list_box_15.AppendItems(cloadDefine)
        elif showPage == _(u"DLOAD"):
            global dloadDefine
            lines = readFistrModelCnt()
            dloadDefine = getDloadGroupInCnt(lines)
            sgrps = meshGroup[1]
            noGrps = self.getNoGroups(sgrps, dloadDefine)
            self.list_box_14.Clear()
            self.list_box_14.AppendItems(noGrps)
            self.list_box_15.Clear()
            self.list_box_15.AppendItems(dloadDefine)
        elif showPage == _(u"VLOAD"):
            global vloadDefine
            lines = readFistrModelCnt()
            vloadDefine = getVloadGroupInCnt(lines)
            egrps = meshGroup[0]
            noGrps = self.getNoGroups(egrps, vloadDefine)
            self.list_box_14.Clear()
            self.list_box_14.AppendItems(noGrps)
            self.list_box_15.Clear()
            self.list_box_15.AppendItems(vloadDefine)
        elif showPage == _(u"CENT"):
            global centDefine
            lines = readFistrModelCnt()
            centDefine = getCentGroupInCnt(lines)
            egrps = meshGroup[0]
            noGrps = self.getNoGroups(egrps, centDefine)
            self.list_box_14.Clear()
            self.list_box_14.AppendItems(noGrps)
            self.list_box_15.Clear()
            self.list_box_15.AppendItems(centDefine)
        elif showPage == _(u"step"):
            values = self.readStepValueFromCnt()
            if len(values) == 0:
                self.list_box_14.Clear()
                self.list_box_14.AppendItems(["STEP"])
                self.list_box_15.Clear()
            else:
                self.list_box_14.Clear()
                self.list_box_15.Clear()
                self.list_box_15.AppendItems(["STEP"])

    #
    #  setElGroupToListBoxInMaterial
    #    elGrpと材料定義をlistboxに追加
    def setElGroupToListBoxInMaterial(self):
        egrps = meshGroup[0]    #EGRP名
        mats = []               #材料定義済のEGRP名
        for names in matDefine:
            mats.append(names[0])
        noMat = []              #未定義のEGRP名
        for egrp in egrps:
            flag = 0
            for mat in mats:
                if egrp == mat:
                    flag = 1
                    break
            if flag == 0:
                noMat.append(egrp)

        self.list_box_5.Clear()
        if len(noMat) > 0:
            self.list_box_5.AppendItems(noMat)
        self.list_box_6.Clear()
        if len(matDefine) > 0:
            self.list_box_6.AppendItems(mats)

    #
    #  setElGroupToLabel
    #    group名をlabelにセット
    def setElGroupToLabel(self, label, name):
        string = label.GetLabel()
        newString = string.split(":")[0] + ":" + name
        label.SetLabel(newString)

    #
    #  getElGroupFromLabel
    #    group名をlabelから取得
    def getElGroupFromLabel(self, label):
        line = label.GetLabel()
        name = line.split(":")[1]
        return name

    #
    #  readSetEachMaterialName
    #    各groupの材料名をセット
    def readSetEachMaterialName(self, elGroupName):
        matName = ""
        elGrpsMatNames = readMeshGetEachMaterialName()
        for grpName in elGrpsMatNames:
            if elGroupName == grpName[0]:
                matName = grpName[1]
                break
        if matName != "":
            self.combo_box_5.SetStringSelection(matName)

    #
    #  readSetMaterialType
    #    材料名、タイプを設定
    def readSetMaterialType(self):
        [matModel, matType, matHard] = readGetMaterialModel()
        self.setMaterialTypeToListBox([matModel, matType, matHard])

    #
    #  setMaterialTypeToListBox
    #    材料モデルなどをlistboxにセット
    def setMaterialTypeToListBox(self, modelType):
        [matModel, matType, matHard] = modelType
        self.combo_box_6.SetStringSelection(matModel)
        matTypes = []
        matHards = []
        for nameGroup in materialType:
            if matModel == nameGroup[0]:
                matTypes = nameGroup[1]
                matHards = nameGroup[2]
                break
        self.combo_box_7.Clear()
        if len(matTypes) == 0:
            self.combo_box_7.Enable(False)
        else:
            self.combo_box_7.AppendItems(matTypes)
            self.combo_box_7.SetStringSelection(matType)
        self.combo_box_8.Clear()
        if len(matHards) == 0:
            self.combo_box_8.Enable(False)
        else:
            self.combo_box_8.AppendItems(matHards)
            self.combo_box_8.SetStringSelection(matHard)
        if matModel == "PLASTIC":
            self.label_85.Enable(True)
            self.button_34.Enable(True)
        else:
            self.label_85.Enable(False)
            self.button_34.Enable(False)

    #
    #  subInitialize
    #
    def subInitialize(self):
        self.label_56.SetLabel("ver "+version)
        #istrDataをセット
        self.setIstrData()
        #cnt,mshファイルの有無チェック
        self.checkCntMshFile()
        #cntファイルをlistBoxにセット
        self.setCntFilesToListBox()
        #Fistrmodel.mshデータを取得
        self.setMeshCont()
        #FistrModel.cntデータ取得
        self.setControlCont()       #cntFileの内容を取得
        matchControlCont()          #meshとcntFileの整合をとる
        #Tree作成
        makeIniTreeData()
        self.setIniTreeData()
        #材料物性値のDialog表示
        self.readMaterialDB()
        self.setMaterialNameToListBox()
        self.setMaterialModelToComboBox()
        #titleページを表示
        #self.setAnalysisTypeToComboBox()
        showPage = _(u"FrontISTR")
        self.showNoteBook(showPage)

    #
    #  checkCntMshFile
    #    cnt, mshファイルをチェックし、存在しない場合defaultのファイルをcopyする
    def checkCntMshFile(self):
        flag = "OK"
        cntFile = currDir + "/FistrModel.cnt"
        if len(glob.glob(cntFile)) == 0:
            name = self.text_ctrl_24.GetValue()
            copyFile = os.getenv("easyIstrPath") + "/data/" + name
            pasteFile = cntFile
            comm = "cp " + copyFile + " " + pasteFile
            os.system(comm)
            flag = "copied"
        hecmwFile = currDir + "/hecmw_ctrl.dat"
        if len(glob.glob(hecmwFile)) == 0:
            copyFile = os.getenv("easyIstrPath") + "/data/hecmw_ctrl.dat"
            pasteFile = hecmwFile
            comm = "cp " + copyFile + " " + pasteFile
            os.system(comm)
            flag = "copied"
        mshFile = currDir + "/FistrModel.msh"
        if len(glob.glob(mshFile)) == 0:
            copyFile = os.getenv("easyIstrPath") + "/data/FistrModel_noMesh.msh"
            pasteFile = mshFile
            comm = "cp " + copyFile + " " + pasteFile
            os.system(comm)
            flag = "copied"
        return flag

    #
    #  setCntFilesToListBox
    #    cntファイルをlistboxに表示
    def setCntFilesToListBox(self):
        cntDir = os.getenv("easyIstrPath") + "/data"
        files = glob.glob(cntDir + "/*.cnt")
        names = []
        for fileName in files:
            name = fileName.split("/")[-1]
            names.append(name)
        self.list_box_9.Clear()
        self.list_box_9.AppendItems(names)

    #
    #  setControlCont
    #    境界条件を取得
    def setControlCont(self):
        global bndDefine, cloadDefine, dloadDefine, vloadDefine, centDefine, tempDefine
        global gravDefine, anaType, contactDefine, stepDefine
        lines = readFistrModelCnt()
        anaType = getAnalysisTypeInCnt(lines)
        bndDefine = getBoundaryGroupInCnt(lines)
        cloadDefine = getCloadGroupInCnt(lines)
        dloadDefine = getDloadGroupInCnt(lines)
        vloadDefine = getVloadGroupInCnt(lines)
        centDefine = getCentGroupInCnt(lines)
        contactDefine = getContactGroupInCnt(lines)
        values = self.readTemperatureValueFromCnt()
        if len(values) == 0:
            tempDefine = []
        else:
            tempDefine = ["ALL"]
        values = self.readGravValueFromCnt()
        if len(values) == 0:
            gravDefine = []
        else:
            gravDefine = ["ALL"]
        values = self.readStepValueFromCnt()
        if len(values) == 0:
            stepDefine = []
        else:
            stepDefine = ["STEP"]

    #  setAnalysisTypeToComboBox
    #
    def setAnalysisTypeToComboBox(self):
        anaTypes = analysisTypes()
        self.combo_box_12.Clear()
        self.combo_box_12.AppendItems(anaTypes)
        if anaType == "STATIC":
            self.combo_box_12.Select(0)
        elif anaType == "NLSTATIC":
            self.combo_box_12.Select(1)
        elif anaType == "DYNAMIC":
            self.combo_box_12.Select(2)
        else:
            self.combo_box_12.Select(0)

    #  getNoGroups
    #    allとselectのgroupからunselectのgroupを返す
    def getNoGroups(self, allGroups, selGroups):
        noSels = []
        for allGroup in allGroups:
            flag = 0
            for selGroup in selGroups:
                if allGroup == selGroup:
                    flag = 1
                    break
            if flag == 0:
                noSels.append(allGroup)
        noSels.sort()
        return noSels

    #
    #  setMaterialModelToComboBox
    #    材料モデルをcomboBoxにセットする
    def setMaterialModelToComboBox(self):
        names = []
        for nameType in materialType:
            names.append(nameType[0])
        self.combo_box_6.Clear()
        self.combo_box_6.AppendItems(names)

    #
    #  setMaterialNameToListBox
    #    材料名をlistBoxにセット
    def setMaterialNameToListBox(self):
        global materialDBCont
        self.combo_box_5.Clear()
        self.combo_box_5.AppendItems(materialDBCont)


    #
    #  readMaterialDB
    #    材料DBを読み込み表示
    def readMaterialDB(self):
        global materialDBCont
        fileName = materialDBName
        if len(glob.glob(fileName)) == 0:
            return

        data = pyFistr.readCsvData(fileName)
        materialDBCont = []
        i = 2
        while i < len(data):
            mat = data[i]
            materialDBCont.append(mat[0])
            i += 1
        materialDBCont.sort()

    #
    #  setIstrData
    #    istr_dataの内容を設定する
    def setIstrData(self):
        global currDir, sourceMeshFile, istrData, materialDBName
        for line in istrData:
            items = line.split()
            if items[0] == "convertOpt":
                if items[1] == "unv2fistr":
                    self.radio_btn_1.SetValue(True)
                else:
                    self.radio_btn_2.SetValue(True)
            elif items[0] == "currDir":
                currDir = items[1]
                self.text_ctrl_23.SetValue(currDir)
            elif items[0] == "materialDB":
                materialDBName = items[1]
                self.text_ctrl_22.SetValue(materialDBName)
            elif items[0] == "useCntFile":
                self.text_ctrl_24.SetValue(items[1])
            elif items[0] == "execFile":
                self.text_ctrl_25.SetValue(items[1])
        if len(glob.glob(currDir)) == 0:
            currDir = os.getenv("HOME")

    #
    #  showTreeData
    #    treeList構造を開いて表示
    def showTreeData(self, treeList):
        self.setIniTreeData()
        self.showTreeParentExpand(treeList)

    #
    #  showTreeParentExpand
    #    親itemを表示し、親をexpandする
    def showTreeParentExpand(self, treeList):
        parent = self.tree_ctrl_1.GetRootItem()
        for name in treeList:
            childrenItems = self.getChildrenItems(parent)
            flag = 0
            for item in childrenItems:
                child = self.tree_ctrl_1.GetItemText(item)
                if name == child:
                    flag = 1
                    break
            if flag == 1:
                parent = item
        self.tree_ctrl_1.SelectItem(parent, True)
        self.tree_ctrl_1.Expand(parent)
        self.tree_ctrl_1.ScrollTo(parent)
        return

    #  getChildrenItems
    #    親itemから子itemを取得
    def getChildrenItems(self, parent):
        items = []
        #firstChild
        item, cookie = self.tree_ctrl_1.GetFirstChild(parent)
        while item:
            items.append(item)
            #nextChild
            item, cookie = self.tree_ctrl_1.GetNextChild(parent, cookie)
        return items
        


    #
    #  setIniTreeData
    #    初期のTreeを作成
    def setIniTreeData(self):
        #dataSet
        treeCtrl = self.tree_ctrl_1
        rootName = treeData[0]
        items = treeData[1]
        treeCtrl.DeleteAllItems()
        root = treeCtrl.AddRoot(rootName)
        self.addTreeNodes(treeCtrl, root, items)
        rootItem = treeCtrl.GetRootItem()
        treeCtrl.Expand(rootItem)

    #  addTreeNodes
    def addTreeNodes(self, treeCtrl, parentItem, items):
        for item in items:
            if type(item) == str or type(item) == unicode:
                newItem = treeCtrl.AppendItem(parentItem, item)
            else:
                newItem = treeCtrl.AppendItem(parentItem, item[0])
                self.addTreeNodes(treeCtrl, newItem, item[1])

    #
    #  setMeshCont
    #    mesh内容を読み込み、表示
    def setMeshCont(self):
        #meshデータ読み込み
        meshCont = getFistrModelMeshData()
        #labelに表示
        self.label_4.SetLabel(meshCont)

    #
    #  setMeshFile
    #    meshFileをセット
    def setMeshFile(self, event):
        global currDir, sourceMeshFile
        if self.radio_btn_1.GetValue() == True:
            fileType = "*.unv"
        else:
            fileType = "*.inp"
        dlg = wx.FileDialog(self, _(u"ファイルを選択してください"), currDir, "", fileType, wx.OPEN)
        stat = dlg.ShowModal()
        if stat == wx.ID_OK:
            sourceMeshFile = dlg.GetFilename()
            currDir = dlg.GetDirectory()
            os.chdir(currDir)
            self.text_ctrl_1.SetValue(sourceMeshFile)
        dlg.Destroy()

    #
    #  meshConvert
    #    mesh変換
    def meshConvert(self, event):
        sourceMeshFile = self.text_ctrl_1.GetValue()
        if sourceMeshFile == "":
            title = _(u"エラー")
            mess = _(u"ファイルが選択されていません")
            self.errDialog(title, mess)
            return

        #cntファイルチェック
        self.checkCntMshFile()
        #mesh変換
        if self.radio_btn_1.GetValue() == True:
            comm = "unv2fistr.py " + currDir + "/" + sourceMeshFile
            option = "unv2fistr"
        else:
            comm = "abaqus2fistr.py " + currDir + "/" + sourceMeshFile
            option = "abaqus2fistr"
        os.system(comm)
        #fistrmodel.msh作成
        fileName = sourceMeshFile.split(".")[0] + ".msh"
        comm = "cp " + fileName + " FistrModel.msh"
        os.system(comm)
        #mshファイル内のheaderチェック修正
        checkFistrModelMeshFile()
        #mesh内容をlabelに書き込み
        self.setMeshCont()
        #cntファイルを初期化
        resetCntMshFile()
        self.setIniTreeData()
        treeList = [_(u"FistrModel.msh")]
        self.showTreeData(treeList)
        #set内容をistrDataに書き込み
        saveIstrData(["convertOpt "+option, "currDir "+currDir])
        #終了メッセージ
        title = _(u"メッシュ変換")
        mess = _(u"mesh変換しました。\nFistrModel.mshを作成しました。")
        self.okDialog(title, mess)


    #-----------------
    def okDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_INFORMATION) 
        dlg.ShowModal()
        dlg.Destroy()

    def okCancelDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                             wx.OK | wx.CANCEL | wx.ICON_INFORMATION) 
        ans = dlg.ShowModal()
        stat = ""
        if ans == wx.ID_OK:
            #dlg.Destroy()
            stat = "OK"
            #return "OK"
        else:
            #dlg.Destroy()
            stat = "CANCEL"
            #return "CANCEL"
        return stat

    def errDialog(self, title, message):
        dlg = wx.MessageDialog(self, message, title,
                               style = wx.OK | wx.ICON_HAND) 
        dlg.ShowModal()
        dlg.Destroy()

    def dirDialog(self, title, path):
        dlg = wx.DirDialog(self, title, 
                            defaultPath=path)
        result = dlg.ShowModal()
        if result == wx.ID_OK:
            folderDir = dlg.GetPath()
        else:
            folderDir = ""
        dlg.Destroy()
        return folderDir

#
#  setSingleHcmwCtrlDat
#    hcmw_ctrl.datをsingle設定にする
def setSingleHcmwCtrlDat():
    fileName = currDir + "/hecmw_ctrl.dat"
    f=open(fileName); lines=f.readlines(); f.close()
    #for partitionを削除
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=part_in")
    lines = lines[:i] + lines[i+2:]
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=part_out")
    lines = lines[:i] + lines[i+2:]
    #for solver
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=fstrMSH")
    lines[i] = abaqus2fistr.setValName(lines[i], "TYPE=HECMW-ENTIRE")
    lines[i+1] = "FistrModel.msh\n"
    #for visualizer
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=mesh")
    lines[i] = abaqus2fistr.setValName(lines[i], "TYPE=HECMW-ENTIRE")
    lines[i+1] = "FistrModel.msh\n"
    f=open(fileName, "w")
    for line in lines:
        f.write(line)

#
#  setParallelHcmwCtrlDat
#    hcmw_ctrl.datをparallel設定にする
def setParallelHcmwCtrlDat(nCpu):
    fileName = currDir + "/hecmw_ctrl.dat"
    f=open(fileName); lines=f.readlines(); f.close()
    #for partitionerを削除（無ければ何もしない）
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=part_in")
    if i < len(lines):
        lines = lines[:i] + lines[i+1:]
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=part_out")
    if i < len(lines):
        lines = lines[:i] + lines[i+1:]
    #for partitionerを追加
    addLines =  ["!MESH, NAME=part_in, TYPE=HECMW-ENTIRE\n"]
    addLines += ["FistrModel.msh\n"]
    addLines += ["!MESH, NAME=part_out, TYPE=HECMW-DIST\n"]
    addLines += ["FistrModel_p" + nCpu + "\n"]
    i = pyFistr.getInsertPointInCnt(lines, "!MESH", ["partition"])
    lines = lines[:i] + addLines + lines[i:]
    #for solver
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=fstrMSH")
    lines[i] = abaqus2fistr.setValName(lines[i], "TYPE=HECMW-DIST")
    lines[i+1] = "FistrModel_p" + nCpu + "\n"
    #for visualizer
    i = pyFistr.getNextHeaderName(lines, 0, "!MESH", "NAME=mesh")
    lines[i] = abaqus2fistr.setValName(lines[i], "TYPE=HECMW-DIST")
    lines[i+1] = "FistrModel_p" + nCpu + "\n"
    f=open(fileName, "w")
    for line in lines:
        f.write(line)

#
#  hideHcmwPartCtrlDat
#    hcmw_part_ctrl.datを隠す
def hideHcmwPartCtrlDat():
    fileName = currDir + "/hecmw_part_ctrl.dat"
    if len(glob.glob(fileName)) != 0:
        comm = "mv -f " + fileName + " " + fileName+".bak"
        os.system(comm)

#
#  loadHcmwPartCtrlDat
#    hcmw_part_ctrl.datをloadする
def loadHcmwPartCtrlDat():
    fileName = currDir + "/hecmw_part_ctrl.dat"
    if len(glob.glob(fileName+".bak")) != 0:
        #bakファイルをrenameする
        comm = "cp " + fileName+".bak " + fileName
        os.system(comm)
    else:
        #defaultのhcmw_part_ctrl.datをコピーする
        copyFile = os.getenv("easyIstrPath") + "/data/hecmw_part_ctrl.dat"
        comm = "cp " + copyFile + " " + currDir
        os.system(comm)

#
#  setHecmwContFromDat
#    hecmwの内容をセット
def setHecmwContFromDat(para, nCpu):
    fileName = currDir+"/hecmw_part_ctrl.dat"
    if len(glob.glob(fileName)) == 0:
        comm = "cp " + currDir + "/hecmw_part_ctrl.dat " + currDir
        os.system(comm)
    #hecmw_part_ctrl.datを書き換え
    f=open(fileName); lines=f.readlines(); f.close()
    i = pyFistr.getNextHeaderName(lines, 0, "!PARTITION", "")
    lines[i] = abaqus2fistr.setValName(lines[i], "DOMAIN="+nCpu)
    f=open(fileName, "w")
    for line in lines:
        f.write(line)

#
#  getHecmwContFromDat
#    hecmwの内容を取得
def getHecmwContFromDat():
    fileName = currDir+"/hecmw_part_ctrl.dat"
    if len(glob.glob(fileName)) == 0:
        #シングルコア
        parallel = False
        nCpu = "1"
    else:
        parallel = True
        f=open(fileName); lines=f.readlines(); f.close()
        i = pyFistr.getNextHeaderName(lines, 0, "!PARTITION", "")
        nCpu = abaqus2fistr.getValName(lines[i], "DOMAIN")
    return [parallel, nCpu]

#
#  setDynamicContInCnt
#    dynamicSolutionの内容を設定
def setDynamicContInCnt(values):
    lines = readFistrModelCnt()
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    lines[i] = abaqus2fistr.setValName(lines[i], "TYPE="+values[0])
    #data部を削除
    i += 1
    ip = i
    while i < len(lines):
        if lines[i][0] == "!" or lines[i][0] == "#":
            break
        i += 1
    lines = lines[:ip] + lines[i:]
    #data部を追加
    data = values[1]
    addLines =  [", ".join(data[:2]) + "\n"]
    addLines += [", ".join(data[2:6]) + "\n"]
    addLines += [", ".join(data[6:8]) + "\n"]
    addLines += [", ".join(data[8:12]) + "\n"]
    addLines += [", ".join(data[12:15]) + "\n"]
    addLines += [", ".join(data[15:21]) + "\n"]
    lines = lines[:ip] + addLines + lines[ip:]
    #restartを修正
    i = pyFistr.getNextHeaderName(lines, 0, "!RESTART", "")
    if i >= len(lines):
        i = pyFistr.getInsertPointInCnt(lines, "!RESTART", ["SOLVER", "Solver"])
        lines = lines[:i] + ["!RESTART\n"] + lines[i:]
    lines[i] = abaqus2fistr.setValName(lines[i], "FREQUENCY="+values[2])
    writeFistrModelCnt(lines)


#
#  setOutputLogInCnt
#    logの出力設定
def setOutputLogInCnt(iterLog, timeLog, freq):
    lines = readFistrModelCnt()
    i = pyFistr.getNextHeaderName(lines, 0, "!SOLVER", "")
    if iterLog == "YES":
        lines[i] = abaqus2fistr.setValName(lines[i], "ITERLOG=YES")
    else:
        lines[i] = abaqus2fistr.setValName(lines[i], "ITERLOG=NO")
    if timeLog == "YES":
        lines[i] = abaqus2fistr.setValName(lines[i], "TIMELOG=YES")
    else:
        lines[i] = abaqus2fistr.setValName(lines[i], "TIMELOG=NO")
    i = pyFistr.getNextHeaderName(lines, 0, "!WRITE", "")
    lines[i] = abaqus2fistr.setValName(lines[i], "FREQUENCY="+freq)
    i = pyFistr.getNextHeaderName(lines, i+1, "!WRITE", "")
    lines[i] = abaqus2fistr.setValName(lines[i], "FREQUENCY="+freq)
    writeFistrModelCnt(lines)

#
#  getPlasticSsData
#    csvデータを取得する
def getPlasticSsData(matName):
    fileName = currDir + "/" + matName + "_PlasticSSdata.csv"
    ssData = ""
    if len(glob.glob(fileName)) == 0:
        return ssData
    contList = pyFistr.readCsvData(fileName)
    for row in contList[2:]:
        data = row[1] + row[2]
        if data != "":
            #strain, stressを入れ替える
            ssData += ", ".join([row[2], row[1]]) + "\n"
    return ssData

#
#  createPlasticSsDataFile
#    csvファイルを作成
def createPlasticSsDataFile(fileName):
    rows = []
    rows += [["", "SS_curve", ""]]
    rows += [["", "strain", "stress"]]
    f=open(fileName, "w")
    for row in rows:
        line = ",".join(row) + "\n"
        f.write(line)
    f.close()

#
#  getSolutionTypeFromCnt
#    solutionTypeを取得
def getSolutitionTypeFromCnt():
    values = []
    lines = readFistrModelCnt()
    i = pyFistr.getNextHeaderName(lines, 0, "!SOLUTION", "")
    solType = abaqus2fistr.getValName(lines[i], "TYPE")
    return solType

#
#  getDynamicContFromCnt
#    動解析に関するdataをcntファイルから取得
def getDynamicContFromCnt():
    values = []
    lines = readFistrModelCnt()
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    dynType = abaqus2fistr.getValName(lines[i], "TYPE")
    values.append(dynType)
    i += 1
    data = []
    while i < len(lines):
        if lines[i][0] == "!":
            break
        else:
            if lines[i][0] != "#":
                line = deleteSp(lines[i])
                words = line.split(",")
                data += words
        i += 1
    values += [data]
    i = pyFistr.getNextHeaderName(lines, 0, "!RESTART", "")
    if i >= len(lines):
        resFreq = "1"
    else:
        resFreq = abaqus2fistr.getValName(lines[i], "FREQUENCY")
        if resFreq == "":
            resFreq = "1"
    values += [resFreq]
    return values

#
#  getSolverContFromCnt
#    solverに関するdataをcntファイルから取得
def getSolverContFromCnt():
    values = []
    lines = readFistrModelCnt()
    i = pyFistr.getNextHeaderName(lines, 0, "!SOLVER", "")
    method = abaqus2fistr.getValName(lines[i], "METHOD")
    precond = abaqus2fistr.getValName(lines[i], "PRECOND")
    iterLog = abaqus2fistr.getValName(lines[i], "ITERLOG")
    timeLog = abaqus2fistr.getValName(lines[i], "TIMELOG")
    i += 1
    nIter = deleteSp(lines[i]).split(",")[0]
    i += 1
    resid = deleteSp(lines[i]).split(",")[0]
    #出力間隔を取得
    freq = "1"
    i = pyFistr.getNextHeaderName(lines, 0, "!WRITE", "")
    val = abaqus2fistr.getValName(lines[i], "FREQUENCY")
    if val != "":
        freq = val
    return [method, precond, nIter, resid, iterLog, timeLog, freq]

#
#  getOutputContFromCnt
#    outputに関するdataをcntファイルから取得
def getOutputContFromCnt():
    values = []
    lines = readFistrModelCnt()
    #outputItemsを取得
    allItems = outputItems()
    selItems = []
    
    i = pyFistr.getNextHeaderName(lines, 0, "!OUTPUT_RES", "")
    i += 1
    while i < len(lines):
        line = lines[i]
        if line[0] == "!" or line[0] == "#":
            break
        words = deleteSp(line).split(",")
        if len(words) == 2:
            if words[1] == "ON":
                selItems.append(words[0])
        i += 1
    if len(selItems) == 0:
        selItems = iniOutputItems()
        selItems.sort()
    else:
        selItems = makeLongOutputName(selItems)
    noItems = pyFistr.getNoGroups(allItems, selItems)
    noItems.sort()
    return [noItems, selItems]

#
#  makeLongOutputName
#    outputNameに説明追加
def makeLongOutputName(names):
    newNames = []
    allNames = outputItems()
    for name in names:
        for longName in allNames:
            #if name == longName.split(":")[0]:
            if name == longName.split(":")[1]:
                newNames.append(longName)
                break
    return newNames

#
#  getBndListFromDefineData
#    DefineDataからStep解析の為のListを作成する
def getBndListFromDefineData():
    bndList = []
    lines = readFistrModelCnt()
    #BOUNDARY
    for name in bndDefine:
        grpId = getBoundaryId(lines, "!BOUNDARY", name)
        bndList.append(["BOUNDART", name, "BOUNDARY", grpId])
    #CLOAD
    for name in cloadDefine:
        grpId = getBoundaryId(lines, "!CLOAD", name)
        bndList.append(["CLOAD", name, "CLOAD", grpId])
    #DLOAD(S)
    for name in dloadDefine:
        grpId = getDloadBoundaryId(lines, "S", name)
        bndList.append(["DLOAD", name, "DLOAD", grpId])
    #DLOAD(BX)
    for name in vloadDefine:
        grpId = getDloadBoundaryId(lines, "BX", name)
        bndList.append(["VLOAD", name, "DLOAD", grpId])
    #DLOAD(GRAV)
    for name in gravDefine:
        grpId = getDloadBoundaryId(lines, "GRAV", name)
        bndList.append(["GRAV", name, "DLOAD", grpId])
    #DLOAD(CENT)
    for name in centDefine:
        grpId = getDloadBoundaryId(lines, "CENT", name)
        bndList.append(["CENT", name, "DLOAD", grpId])
    #TEMPERATURE
    for name in tempDefine:
        grpId = getBoundaryId(lines, "!TEMPERATURE", name)
        bndList.append(["TEMPERATURE", name, "TEMPERATURE", grpId])
    #CONTACT
    for name in contactDefine:
        grpId = getBoundaryId(lines, "!CONTACT", name)
        bndList.append(["CONTACT", name, "CONTACT", grpId])
    return bndList

#  getBoundaryId
def getBoundaryId(lines, bndName, name):
    grpId = ""
    i = 0
    while i < len(lines):
        if lines[i][:len(bndName)] == bndName:
            if deleteSp(lines[i+1]).split(",")[0] == name:
                grpId = abaqus2fistr.getValName(lines[i], "GRPID")
                break
        i += 1
    return grpId

#  getDloadBoundaryId
def getDloadBoundaryId(lines, bndName, name):
    grpId = ""
    i = 0
    while i < len(lines):
        if lines[i][:len("!DLOAD")] == "!DLOAD":
            i += 1
            words = deleteSp(lines[i]).split(",")
            if words[1] == bndName:
                grpId = abaqus2fistr.getValName(lines[i], "GRPID")
                break
        i += 1
    return grpId

#
#  reformStepBCCont
#    STEP解析に使用する境界条件を成形する
def reformStepBCCont(bnds, bndList):
    ans = []
    for bnd in bnds:
        bnd = deleteSp(bnd)
        for blist in bndList:
            if bnd == ",".join(blist[2:4]):
                ans.append(",".join(blist[:2]))
                break
    return ans

#
#  reconstructStepBCCont
#    STEP解析に使用する境界条件を元に戻す
def reconstructStepBCCont(bnds, bndList):
    ans = []
    for bnd in bnds:
        bnd = deleteSp(bnd)
        for blist in bndList:
            bLine = ",".join(blist[:2])
            if bnd == bLine:
                rLine = ", ".join(blist[2:4]) + "\n"
                ans.append(rLine)
                break
    return ans

#
#  addPrincipalStressData
#    結果ファイルに主応力を追加
def addPrincipalStressData():
    files = glob.glob(currDir + "/convFistrModel.res*.inp")
    if len(files) == 0:
        error = "noFile"
        return error
    #主応力を追加
    steps = []
    nameHeader = "/".join(files[0].split("/")[:-1])
    name = ".".join(files[0].split("/")[-1].split(".")[:-2])
    nameHeader += "/" + name
    for fileName in files:
        name = fileName.split("/")[-1]
        step = name.split(".")[-2]
        steps.append(int(step))
    steps.sort()
    names = []
    i = 0
    for fileName in files:
        name = nameHeader+ "." + str(steps[i]) + ".inp"
        names.append(name)
        i += 1

    error = ""
    #for fileName in files:
    for fileName in names:
        name = fileName.split("/")[-1]
        print name + _(u" 主応力データ追加中...")
        f=open(fileName); lines=f.readlines(); f.close()
        words = lines[0].split()
        #      nNodal        nElement
        ip = int(words[0]) + int(words[1])
        checkLines = lines[ip:ip+50]
        i = 0
        while i < len(checkLines):
            line = checkLines[i]
            if line[:len("maxSigma1")] == "maxSigma1":
                error = "added"
                break
            i += 1
        if error != "":
            break
        else:
            lines = pyFistr.addPrincipalStressData(lines)
            f=open(fileName, "w")
            for line in lines:
                f.write(line)
            f.close()
    if error == "":
        print _(u"主応力を追加しました。")
    return error


#
#  convertDataForParaview
#    データ変換
def convertDataForParaview():
    #meshデータ取得
    if len(glob.glob(currDir + "/FistrModel.res.1.*")) == 0:
        #シングル計算の場合 
        files = glob.glob(currDir + "/FistrModel*.inp")
        [meshData, nNode, nElement] = pyFistr.getMeshDataForParaview(files[0])
        meshData = pyFistr.renumberMeshData(meshData, nNode, nElement)
        files = glob.glob(currDir + "/FistrModel.res.0*")
        if len(files) == 0:
            mess = _(u"計算結果が存在しません。")
            return mess
    else:
        #並列計算の場合
        fileName = currDir + "/part.inp"
        f=open(fileName); lines=f.readlines(); f.close()
        words = lines[0].split()
        nNode = int(words[0])
        nElement = int(words[1])
        meshData = lines[1:1+nNode+nElement]
        files = glob.glob(currDir + "/FistrModel.res.merge*")
        if len(files) == 0:
            mess = _(u"計算結果が分散したままです。\n")
            mess += _(u"rmergeで計算結果を結合して、FistrModel.res.merge.*ファイルを作成してください。")
            return mess
    #メッシュ変換
    steps = []
    for name in files:
        step = int(name.split("/")[-1].split(".")[-1])
        steps.append(step)
    steps.sort()
    convFiles = []
    i = 0
    while i < len(files):
        newName = ".".join(files[i].split("/")[-1].split(".")[:-1]) + "." + str(steps[i])
        convFile = "/".join(files[i].split("/")[:-1]) + "/" + newName
        convFiles.append(convFile)
        i += 1
    for fileName in convFiles:
        name = fileName.split("/")[-1]
        print name + _(u" ファイル変換中...")
        #結果データ読み込み
        f=open(fileName); lines=f.readlines(); f.close()
        newLines = pyFistr.addMeshDataConvVisData(lines, meshData)
        #データ変換
        #newLines = pyFistr.convertResultDataForParaview(lines)
        newLines = pyFistr.renumberResultDataForParaview(newLines)
        #変換結果を保存
        words = fileName.split("/")
        words[-1] = "conv" + words[-1]
        #convName = "/".join(words)
        convName = "/".join(words) + ".inp"
        f=open(convName, "w")
        for line in newLines:
            f.write(line)
        f.close()
    print _(u"変換しました。")
    return ""
#
#  deleteStepContInCnt
#    STEPを削除
def deleteStepContInCnt():
    lines = readFistrModelCnt()
    lines = deleteStepCont(lines)
    writeFistrModelCnt(lines)

#
#  deleteTemperatureContInCnt
#    TEMPERATUREを削除
def deleteTemperatureContInCnt():
    lines = readFistrModelCnt()
    lines = deleteTemperatureCont(lines)
    writeFistrModelCnt(lines)
 
#
#  deleteGravContInCnt
#    GRAVを削除
def deleteGravContInCnt():
    lines = readFistrModelCnt()
    lines = deleteGravCont(lines)
    writeFistrModelCnt(lines)

#
#  deleteGravCont
#
def deleteGravCont(lines):
    newLines = []
    i = 0
    while i < len(lines):
        grpTypes = pyFistr.getNextDloadGrpType(lines, i)
        if len(grpTypes) > 0:
            if len(grpTypes) == 1:
                [grp, bndType, line, i] = grpTypes[0]
                if grp == "ALL" and bndType == "GRAV":
                    newLines = lines[:i-1] + lines[i+1:]
                    break
                else:
                    i += 1
            else:
                i += 1
        else:
            break
    if len(newLines) > 0:
        #GRPIDを修正
        newLines = pyFistr.remakeGRPID(newLines, "!DLOAD")
    else:
        newLines = lines
    return newLines

#
#  matchControlCont
#    cntFileの内容をmeshに合わせる
def matchControlCont():
    lines = readFistrModelCnt()
    lines = deleteDloadWithoutNames(lines, meshGroup[1])
    lines = deleteCloadWithoutNames(lines, meshGroup[2])
    lines = deleteBoundaryWithoutNames(lines, meshGroup[2])
    lines = deleteVloadWithoutNames(lines, meshGroup[0])
    lines = deleteCentWithoutNames(lines, meshGroup[0])
    writeFistrModelCnt(lines)
    matNames = getMatNamesInCnt(lines)
    mshLines = readFistrModelMsh()
    mshLines = deleteMaterialInSectionWithoutNames(mshLines, matNames)
    writeFistrModelMsh(mshLines)

#
#  deleteMaterialInSectionWithoutNames
#    mshファイルのSECTION行から指定されていないmatNamesを削除する
def deleteMaterialInSectionWithoutNames(lines, matNames):
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!SECTION")] == "!SECTION":
            name = abaqus2fistr.getValName(line, "MATERIAL")
            delName = "yes"
            for matName in matNames:
                if name == matName:
                    delName = "no"
                    break
            if delName == "yes":
                #matNameを削除
                words = deleteSp(line).split(",")
                newWords = []
                for word in words:
                    if word[:len("MATERIAL=")] != "MATERIAL=":
                        newWords.append(word)
                lines[i] = ", ".join(newWords) + "\n"
        i += 1
    return lines

#
#  getMatNamesInCnt
#    cntファイルに定義されているmatNamesを取得
def getMatNamesInCnt(lines):
    matNames = []
    for line in lines:
        if line[:len("!MATERIAL")] == "!MATERIAL":
            matName = abaqus2fistr.getValName(line, "NAME")
            matNames.append(matName)
    return matNames

#
#  resetCntMshFile
#    cntファイルの材料と境界条件を削除する
def resetCntMshFile():
    global matDefine, bndDefine, cloadDefine, dloadDefine, vloadDefine
    global tempDefine, gravDefine, contactDefine, stepDefine
    lines = readFistrModelCnt()
    lines = deleteMaterialWithoutNames(lines, [])
    lines = deleteDloadWithoutNames(lines, [])
    lines = deleteCloadWithoutNames(lines, [])
    lines = deleteBoundaryWithoutNames(lines, [])
    lines = deleteVloadWithoutNames(lines, [])
    lines = deleteTemperatureCont(lines)
    lines = deleteGravCont(lines)
    writeFistrModelCnt(lines)
    lines = readFistrModelMsh()
    lines = deleteMaterialWithoutNames(lines, [])
    writeFistrModelMsh(lines)
    matDefine = []
    bndDefine = []
    cloadDefine = []
    dloadDefine = []
    vloadDefine = []
    centDefine = []
    tempDefine = []
    gravDefine = []
    contactDefine = []
    stepDefine = []
    makeIniTreeData()

#
#  deleteMaterialWithoutNames
#    不要なMATERIALの定義を削除
def deleteMaterialWithoutNames(lines, matNames):

    def isOtherName(matNames, name):
        ans = False
        flag = 0
        for matName in matNames:
            if name == matName:
                flag = 1
                break
        if flag == 0:
            ans = True
        return ans

    def isInMaterial(line, matGrp):
        ans = False
        for mat in matGrp:
            if line[:len(mat)] == mat:
                ans = True
                break
        return ans    

    def skipNextLine(lines, i, matGrp):
        ans = i
        while i < len(lines):
            if lines[i][0] == "!" or lines[i][0] == "#":
                if isInMaterial(lines[i], matGrp) == False:
                    ans = i - 1
                    break
            i += 1
        return ans

    #matGrp = ["!ITEM", "!ELASTIC", "!DENSITY", "!EXPANSION_COEFF", "!PLASTIC", "!HYPERELASTIC", "!VISCOELASTIC", "!CREEP"]
    matGrp = matGrpHeader
    newLines = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!MATERIAL")] == "!MATERIAL":
            name = abaqus2fistr.getValName(line, "NAME")
            if isOtherName(matNames, name) == True:
                i = skipNextLine(lines, i+1, matGrp)
            else:
                newLines.append(lines[i])
        else:
            newLines.append(lines[i])
        i += 1
    return newLines

#
#  addContactPairNameInCntMsh
#    contact定義を追加
def addContactPairNameInCntMsh(pairName, slave, master, interaction,
            nTol, tTol, coeff, factor):
    #mshにpairName追加
    lines = readFistrModelMsh()
    addLines =  ["!CONTACT PAIR, NAME=" + pairName + "\n"]
    addLines += [slave + ", " + master + "\n"]
    newLines = lines[:-1] + addLines + [lines[-1]]
    writeFistrModelMsh(newLines)
    #cntに追加
    lines = readFistrModelCnt()
    addLines =  ["!CONTACT, GRPID=1, INTERACTION=" + interaction + ", NTOL=" + nTol
                    + ", TTOL=" + tTol + "\n"]
    addLines += [pairName + ", " + coeff + ", " + factor + "\n"]
    ip = pyFistr.getInsertPointInCnt(lines, "!CONTACT", ["Contact"])
    newLines = lines[:ip] + addLines + lines[ip:]
    writeFistrModelCnt(newLines)

#
#  deleteContactPairNameInCntMsh
#    contact定義を削除
def deleteContactPairNameInCntMsh(pairNames):
    #msh側のcontactPairを削除
    newLines = []
    lines = readFistrModelMsh()
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CONTACT PAIR")] == "!CONTACT PAIR":
            name = abaqus2fistr.getValName(line, "NAME")
            isDelete = "no"
            for pairName in pairNames:
                if pairName == name:
                    isDelete = "yes"
                    break
            if isDelete == "no":
                newLines.append(line)
            else:
                i += 1
        else:
            newLines.append(line)
        i += 1
    writeFistrModelMsh(newLines)
    #cnt側を修正
    lines = readFistrModelCnt()
    #contact削除
    cntLines = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CONTACT")] == "!CONTACT":
            if deleteSp(line).split(",")[0] == "!CONTACT":
                name = deleteSp(lines[i+1]).split(",")[0]
                flag = 0
                for pairName in pairNames:
                    if name == pairName:
                        flag = 1
                        break
                if flag == 0:
                    cntLines.append(lines[i])
                else:
                    i += 1
            else:
                cntLines.append(lines[i])
        else:
            cntLines.append(lines[i])
        i += 1
    writeFistrModelCnt(cntLines)

#
#  getContactContInCntMsh
#    [slave, master, interaction, nTol, tTol, coeff, factor]を取得
def getContactContInCntMsh(name):
    pairNames = getContactPairNameInMsh(name)
    conts = getContactContInCnt(name)
    ans = pairNames + conts
    return ans

#
#  getContactContInCnt
#    contactの内容を取得
def getContactContInCnt(name):
    coeffs = ["0.0", "0.0"]
    interaction = "FSLID"
    nTol = "1.0e-5"
    tTol = "1.0e-3"
    lines = readFistrModelCnt()
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CONTACT")] == "!CONTACT":
            if deleteSp(line).split(",")[0] == "!CONTACT":
                words = deleteSp(lines[i+1]).split(",")
                if words[0] == name:
                    coeffs = words[1:]
                    interW = abaqus2fistr.getValName(line, "INTERACTION")
                    nTolW = abaqus2fistr.getValName(line, "NTOL")
                    tTolW = abaqus2fistr.getValName(line, "TTOL")
                    if interW != "":
                        interaction = interW
                    if nTolW != "":
                        nTol = nTolW
                    if tTolW != "":
                        tTol = tTolW
                    break
        i += 1
    conts = [interaction] + [nTol] + [tTol] + coeffs
    return conts

#
#  getContactPairNameInMsh
#    contactPairの[slave, master]名を取得
def getContactPairNameInMsh(name):
    lines = readFistrModelMsh()
    pairNames = ["", ""]
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CONTACT PAIR")] == "!CONTACT PAIR":
            if abaqus2fistr.getValName(line, "NAME") == name:
                pairNames = deleteSp(lines[i+1]).split(",")
                break
        i += 1
    return pairNames

#
#  getContactAlgoInCnt
#    contactの内容を取得
def getContactAlgoInCnt():
    lines = readFistrModelCnt()
    algo = "ALAGRANGE"
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CONTACT_ALGO")] == "!CONTACT_ALGO":
            algo = abaqus2fistr.getValName(line, "TYPE")
            break
        i += 1
    return algo

#
#  getContactGroupInCnt
#    CONTACTのgroupを取得
def getContactGroupInCnt(lines):
    grp = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CONTACT")] == "!CONTACT":
            if deleteSp(line).split(",")[0] == "!CONTACT":
                i += 1
                grp.append(deleteSp(lines[i]).split(",")[0])
        i += 1
    return grp

#
#  getCentGroupInCnt
#    CENT（遠心力）のgroupを取得
def getCentGroupInCnt(lines):
    cents = getDloadTypeGroupInCnt(lines, "CENT")
    return cents

#
#  getVloadGroupInCnt
#    VLOAD（体積力）のgroupを取得
def getVloadGroupInCnt(lines):
    vload = getDloadTypeGroupInCnt(lines, "BX")
    if len(vload) == 0:
        vload = getDloadTypeGroupInCnt(lines, "BY")
        if len(vload) == 0:
            vload = getDloadTypeGroupInCnt(lines, "BZ")
    return vload

#
#  getDloadGroupInCnt
#    DLOAD（面荷重）のgroupを取得
def getDloadGroupInCnt(lines):
    dload = getDloadTypeGroupInCnt(lines, "S")
    return dload

#  getDloadTypeGroupInCnt
def getDloadTypeGroupInCnt(lines, bndType):
    dload = []
    i = 1
    while i < len(lines):
        line = lines[i]
        if line[:len("!DLOAD")] == "!DLOAD":
            i += 1
            line = deleteSp(lines[i])
            words = line.split(",")
            if words[1] == bndType:
                sgrp = words[0]
                dload.append(sgrp)
        i += 1
    return dload

#
#  getCloadGroupInCnt
#    CLOAD（荷重拘束）のgroupを取得
def getCloadGroupInCnt(lines):
    cload = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!CLOAD")] == "!CLOAD":
            i += 1
            line = deleteSp(lines[i])
            ngrp = lines[i].split(",")[0]
            cload.append(ngrp)
        i += 1
    return cload

#
#  getAnalysisTypeInCnt
#    解析の種類を取得
def getAnalysisTypeInCnt(lines):
    anaType = "STATIC"
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!SOLUTION")] == "!SOLUTION":
            anaType = abaqus2fistr.getValName(line, "TYPE")
            break
        i += 1
    return anaType

#
#  getBoundaryGroupInCnt
#    boundary（変位拘束）のgroupを取得
def getBoundaryGroupInCnt(lines):
    boundary = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!BOUNDARY")] == "!BOUNDARY":
            i += 1
            line = deleteSp(lines[i])
            ngrp = lines[i].split(",")[0]
            boundary.append(ngrp)
        i += 1
    return boundary

#
#  writeCentContInCnt
#    CENTを書き換え
def writeCentContInCnt(grp, addLines):
    lines = readFistrModelCnt()
    lines = deleteDloadNames(lines, "CENT", [grp])
    lines = addDloadContNames(lines, [grp], addLines)
    writeFistrModelCnt(lines)

#
#  writeTemperatureContInCnt
#    TEMPERATUREを書き換え
def writeTemperatureContInCnt(addLines):
    lines = readFistrModelCnt()
    lines = deleteTemperatureCont(lines)
    lines = addTemperatureCont(lines, addLines)
    writeFistrModelCnt(lines)

#
#  writeStepContInCnt
#    STEPを書き換え
def writeStepContInCnt(addLines):
    lines = readFistrModelCnt()
    lines = deleteStepCont(lines)
    lines = addStepCont(lines, addLines)
    writeFistrModelCnt(lines)

#
#  writeGravContInCnt
#    GRAVを書き換え
def writeGravContInCnt(grp, addLines):
    lines = readFistrModelCnt()
    lines = deleteGravContNames(lines, [grp])
    lines = addDloadContNames(lines, [grp], addLines)
    writeFistrModelCnt(lines)

#
#  writeVloadContInCnt
#    VLOADを書き換え
def writeVloadContInCnt(grp, addLines):
    lines = readFistrModelCnt()
    lines = deleteVloadContNames(lines, [grp])
    lines = addDloadContNames(lines, [grp], addLines)
    writeFistrModelCnt(lines)

#
#  writeDloadContInCnt
#    DLOADを書き換え
def writeDloadContInCnt(grp, addLines):
    lines = readFistrModelCnt()
    lines = deleteDloadContNames(lines, [grp])
    lines = addDloadContNames(lines, [grp], addLines)
    writeFistrModelCnt(lines)

#  addStepCont
def addStepCont(lines, addLines):
    ip = pyFistr.getInsertPointInCnt(lines, "!STEP", ["STEP", "Step", "step"])
    lines = lines[:ip] + addLines + lines[ip:]
    return lines

# addTemperatureCont
def addTemperatureCont(lines, addLines):
    #insertPointを取得
    ip = 0
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[0] == "#":
            if line.find("Temperature") >= 0:
                if lines[i+1][:3] == "###":
                    ip = i + 2
                else:
                    ip = i + 1
                break
        i += 1
    newLines = lines[:ip] + addLines + lines[ip:]
    return newLines


#  addDloadContNames
def addDloadContNames(lines, names, addLines):

    def getAddNames(names):
        addNames = []
        for name in names:
            flag = 0
            for bnd in cloadDefine:
                if name == bnd:
                    flag = 1
                    break
            if flag == 0:
                addNames.append(name)
        return addNames

    def getInsertPointer(lines):
        iP = 0
        i = 0
        while i < len(lines):
            line = lines[i]
            isDload = (line[:len("!DLOAD")] == "!DLOAD")
            isType = False
            if isDload == True:
                nextLine = deteSp(lines[i+1])
                bndType = nextLine.split(",")[1]
                if bndType == "S":
                    isType = True
            if isDload and isType:
                iP = i
                break
            else:
                if line[0] == "#":
                    n1 = line.find("BOUNDARY")
                    n2 = line.find("Boundary")
                    n3 = line.find("boundary")
                    if n1>=0 or n2>=0 or n3>=0:
                        if lines[i+1][:3] == "###":
                            iP = i + 2
                        else:
                            iP = i + 1
                        break
            i += 1
        return iP

    def remakeGRPID(lines):
        grpId = 1
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!DLOAD")] == "!DLOAD":
                line = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                grpId += 1
            newLines.append(line)
            i += 1
        return newLines

    addNames = getAddNames(names)
    iP = getInsertPointer(lines)
    lines = lines[:iP] + addLines + lines[iP:]
    lines = remakeGRPID(lines)
    return lines


#
#  writeCloadContInCnt
#    CLOADを書き換え
def writeCloadContInCnt(ngrp, cloadCont):

    def skipThisBoundary(lines, ngrp, i):
        p = i
        i += 1
        while i < len(lines):
            cname = deleteSp(lines[i]).split(",")[0]
            if cname != ngrp:
                p = i - 1
                break
            i += 1
        return p

    def deleteBoundary(lines, ngrp):
        ip = 0
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!CLOAD")] == "!CLOAD":
                newLines.append(line)
                cname = deleteSp(lines[i+1]).split(",")[0]
                if cname == ngrp:
                    ip = i+1
                    i = skipThisBoundary(lines, ngrp, i+1)
            else:
                newLines.append(line)
            i += 1
        return [newLines, ip]

    def createAddCont(name, bndCont):
        newConts = []
        for bnd in bndCont:
            line = name + ", " + bnd
            newConts.append(line)
        return newConts

    lines = readFistrModelCnt()
    [lines, ip] = deleteBoundary(lines, ngrp)
    addConts = createAddCont(ngrp, cloadCont)
    newLines = lines[:ip] + addConts + lines[ip:]
    writeFistrModelCnt(newLines)


#
#  writeBoundaryContInCnt
#    BOUNDARYを書き換え
def writeBoundaryContInCnt(ngrp, bndCont):

    def skipThisBoundary(lines, ngrp, i):
        p = i
        i += 1
        while i < len(lines):
            cname = deleteSp(lines[i]).split(",")[0]
            if cname != ngrp:
                p = i - 1
                break
            i += 1
        return p

    def deleteBoundary(lines, ngrp):
        ip = 0
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!BOUNDARY")] == "!BOUNDARY":
                newLines.append(line)
                cname = deleteSp(lines[i+1]).split(",")[0]
                if cname == ngrp:
                    ip = i+1
                    i = skipThisBoundary(lines, ngrp, i+1)
            else:
                newLines.append(line)
            i += 1
        return [newLines, ip]

    def createAddCont(name, bndCont):
        newConts = []
        for bnd in bndCont:
            line = name + ", " + bnd
            newConts.append(line)
        return newConts

    lines = readFistrModelCnt()
    [lines, ip] = deleteBoundary(lines, ngrp)
    addConts = createAddCont(ngrp, bndCont)
    newLines = lines[:ip] + addConts + lines[ip:]
    writeFistrModelCnt(newLines)

#
#  writeDefaultCentInCnt
#    defaultのCENTをファイルに書き込み
def writeDefaultCentInCnt(names):
    lines = readFistrModelCnt()
    lines = deleteCentWithoutNames(lines, names)
    lines = addDefaultCentNames(lines, names)
    writeFistrModelCnt(lines)

#
#  writeDefaultVloadInCnt
#    defaultのVLOADをファイルに書き込み
def writeDefaultVloadInCnt(names):
    lines = readFistrModelCnt()
    lines = deleteVloadWithoutNames(lines, names)
    lines = addDefaultVloadNames(lines, names)
    writeFistrModelCnt(lines)

#
#  writeDefaultDloadInCnt
#    defaultのDLOADをファイルに書き込み
def writeDefaultDloadInCnt(names):
    lines = readFistrModelCnt()
    lines = deleteDloadWithoutNames(lines, names)
    lines = addDefaultDloadNames(lines, names)
    writeFistrModelCnt(lines)

#
#  writeDefaultCloadInCnt
#    defaultのCLOADをファイルに書き込み
def writeDefaultCloadInCnt(names):
    lines = readFistrModelCnt()
    lines = deleteCloadWithoutNames(lines, names)
    lines = addDefaultCloadNames(lines, names)
    writeFistrModelCnt(lines)

#
#  addDefaultCentNames
#    defaultのCENTを設定
#    新しく追加されたgroupのみdefaultの内容を追加する
def addDefaultCentNames(lines, names):
    #追加するnameを取得
    addNames = pyFistr.getNoGroups(names, centDefine)
    #追加するlineを作成
    addLines = []
    for name in addNames:
        addLines += ["!DLOAD, GRPID=1\n"]
        addLines += [name + ", CENT, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0\n"]
    #挿入する場所を取得
    ip = pyFistr.getInsertPointInCnt(lines, "!DLOAD", ["BOUNDARY", "Boundary", "boundary"])
    lines = lines[:ip] + addLines + lines[ip:]
    #GRPIDを修正
    lines = pyFistr.remakeGRPID(lines, "!DLOAD")
    return lines

#
#  addDefaultVloadNames
#    defaultのVLOADを設定
#    新しく追加されたgroupのみdefaultの内容を追加する
def addDefaultVloadNames(lines, names):

    def getAddNames(names):
        addNames = []
        for name in names:
            flag = 0
            for bnd in vloadDefine:
                if name == bnd:
                    flag = 1
                    break
            if flag == 0:
                addNames.append(name)
        return addNames

    def getInsertPointer(lines):
        iP = 0
        i = 0
        while i < len(lines):
            line = lines[i]
            isDload = (line[:len("!DLOAD")] == "!DLOAD")
            isType = False
            if isDload == True:
                nextLine = deteSp(lines[i+1])
                bndType = nextLine.split(",")[1]
                if bndType == "BX" or bndType == "BY" or bndType == "BZ":
                    isType = True
            if isDload and isType:
                iP = i
                break
            else:
                if line[0] == "#":
                    n1 = line.find("BOUNDARY")
                    n2 = line.find("Boundary")
                    n3 = line.find("boundary")
                    if n1>=0 or n2>=0 or n3>=0:
                        if lines[i+1][:3] == "###":
                            iP = i + 2
                        else:
                            iP = i + 1
                        break
            i += 1
        return iP

    def createAddBoundary(addNames):
        addLines = []
        for name in addNames:
            addLines += ["!DLOAD, GRPID=1\n"]
            addLines += [name + ", BX, 0.0\n"]
            addLines += [name + ", BY, 0.0\n"]
            addLines += [name + ", BZ, 0.0\n"]
        return addLines

    def remakeGRPID(lines):
        grpId = 1
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!DLOAD")] == "!DLOAD":
                line = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                grpId += 1
            newLines.append(line)
            i += 1
        return newLines

    
    addNames = getAddNames(names)
    iP = getInsertPointer(lines)
    addLines = createAddBoundary(addNames)
    lines = lines[:iP] + addLines + lines[iP:]
    lines = remakeGRPID(lines)
    return lines


#
#  addDefaultDloadNames
#    defaultのDLOADを設定
#    新しく追加したgroupのみdefaultの内容を追加する。
def addDefaultDloadNames(lines, names):

    def getAddNames(names):
        addNames = []
        for name in names:
            flag = 0
            for bnd in dloadDefine:
                if name == bnd:
                    flag = 1
                    break
            if flag == 0:
                addNames.append(name)
        return addNames

    def getInsertPointer(lines):
        iP = 0
        i = 0
        while i < len(lines):
            line = lines[i]
            isDload = (line[:len("!DLOAD")] == "!DLOAD")
            isType = False
            if isDload == True:
                nextLine = deteSp(lines[i+1])
                bndType = nextLine.split(",")[1]
                if bndType == "S":
                    isType = True
            if isDload and isType:
                iP = i
                break
            else:
                if line[0] == "#":
                    n1 = line.find("BOUNDARY")
                    n2 = line.find("Boundary")
                    n3 = line.find("boundary")
                    if n1>=0 or n2>=0 or n3>=0:
                        if lines[i+1][:3] == "###":
                            iP = i + 2
                        else:
                            iP = i + 1
                        break
            i += 1
        return iP

    def createAddBoundary(addNames):
        addLines = []
        for name in addNames:
            addLines += ["!DLOAD, GRPID=1\n"]
            addLines += [name + ", S, 0.0\n"]
        return addLines

    def remakeGRPID(lines):
        grpId = 1
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!DLOAD")] == "!DLOAD":
                line = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                grpId += 1
            newLines.append(line)
            i += 1
        return newLines

    
    addNames = getAddNames(names)
    iP = getInsertPointer(lines)
    addLines = createAddBoundary(addNames)
    lines = lines[:iP] + addLines + lines[iP:]
    lines = remakeGRPID(lines)
    return lines

#
#  addDefaultCloadNames
#    defaultのCLOADを設定
def addDefaultCloadNames(lines, names):

    def getAddNames(names):
        addNames = []
        for name in names:
            flag = 0
            for bnd in cloadDefine:
                if name == bnd:
                    flag = 1
                    break
            if flag == 0:
                addNames.append(name)
        return addNames

    def getInsertPointer(lines):
        iP = 0
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!CLOAD")] == "!CLOAD":
                iP = i
                break
            else:
                if line[0] == "#":
                    n1 = line.find("BOUNDARY")
                    n2 = line.find("Boundary")
                    n3 = line.find("boundary")
                    if n1>=0 or n2>=0 or n3>=0:
                        if lines[i+1][:3] == "###":
                            iP = i + 2
                        else:
                            iP = i + 1
                        break
            i += 1
        return iP

    def createAddBoundary(addNames):
        addLines = []
        for name in addNames:
            addLines += ["!CLOAD, GRPID=1\n"]
            addLines += [name + ", 1, 0.0\n"]
            addLines += [name + ", 2, 0.0\n"]
            addLines += [name + ", 3, 0.0\n"]
        return addLines

    def remakeGRPID(lines):
        grpId = 1
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!CLOAD")] == "!CLOAD":
                line = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                grpId += 1
            newLines.append(line)
            i += 1
        return newLines

    addNames = getAddNames(names)
    iP = getInsertPointer(lines)
    addLines = createAddBoundary(addNames)
    lines = lines[:iP] + addLines + lines[iP:]
    lines = remakeGRPID(lines)
    return lines

#
#  writeDefaultBoundaryInCnt
#    defaultのBOUNDARYをファイルに書き込み
def writeDefaultBoundaryInCnt(names):
    lines = readFistrModelCnt()
    lines = deleteBoundaryWithoutNames(lines, names)
    lines = addDefaultBoundaryNames(lines, names)
    writeFistrModelCnt(lines)

#
#  addDefaultBoundaryNames
#    defaultのBOUNDARYを設定
def addDefaultBoundaryNames(lines, names):

    def getAddNames(names):
        addNames = []
        for name in names:
            flag = 0
            for bnd in bndDefine:
                if name == bnd:
                    flag = 1
                    break
            if flag == 0:
                addNames.append(name)
        return addNames

    def getInsertPointer(lines):
        iP = 0
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!BOUNDARY")] == "!BOUNDARY":
                iP = i
                break
            else:
                if line[0] == "#":
                    n1 = line.find("BOUNDARY")
                    n2 = line.find("Boundary")
                    n3 = line.find("boundary")
                    if n1>=0 or n2>=0 or n3>=0:
                        if lines[i+1][:3] == "###":
                            iP = i + 2
                        else:
                            iP = i + 1
                        break
            i += 1
        return iP

    def createAddBoundary(addNames):
        addLines = []
        for name in addNames:
            addLines += ["!BOUNDARY, GRPID=1\n"]
            addLines += [name + ", 1, 1, 0.0\n"]
            addLines += [name + ", 2, 2, 0.0\n"]
            addLines += [name + ", 3, 3, 0.0\n"]
        return addLines

    def remakeGRPID(lines):
        grpId = 1
        newLines = []
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!BOUNDARY")] == "!BOUNDARY":
                line = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                grpId += 1
            newLines.append(line)
            i += 1
        return newLines

    addNames = getAddNames(names)
    iP = getInsertPointer(lines)
    addLines = createAddBoundary(addNames)
    lines = lines[:iP] + addLines + lines[iP:]
    lines = remakeGRPID(lines)
    return lines

#
#  deleteStepCont
#    STEPを削除
def deleteStepCont(lines):
    newLines = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!STEP")] == "!STEP":
            #「#」または「!」までskipする
            i += 1
            while i < len(lines) and not (lines[i][0] == "!" or lines[i][0] == "#"):
                i += 1
            newLines.append(lines[i])
        else:
            newLines.append(line)
        i += 1
    return newLines

#
#  deleteTemperatureCont
#    TEMPERATUREを削除
def deleteTemperatureCont(lines):
    newLines = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!TEMPERATURE")] == "!TEMPERATURE":
            i += 2
        elif line[:len("!REFTEMP")] == "!REFTEMP":
            i += 2
        else:
            newLines.append(line)
            i += 1
    return newLines

#
#  deleteGravContNames
#    指定したGRAVのgroupを削除
def deleteGravContNames(lines, names):
    lines = deleteDloadNames(lines, "GRAV", names)
    return lines
    

#
#  deleteVloadContNames
#    指定したVLOADのgroupを削除
def deleteVloadContNames(lines, names):
    lines = deleteDloadNames(lines, "BX", names)
    return lines

#
#  deleteDloadContNames
#    指定したDLOADのgroupを削除
def deleteDloadContNames(lines, names):
    lines = deleteDloadNames(lines, "S", names)
    return lines

#
#  deleteDloadNames
#    指定したgroupを削除
def deleteDloadNames(lines, bndType, names):

    def isDeleteName(bndName, names):
        flag = 0
        for name in names:
            if bndName == name:
                flag = 1
                break
        if flag == 1:
            return True
        else:
            return False

    def skipNextHeader(lines, i):
        ans = 0
        while i < len(lines):
            if lines[i][0] == "!" or lines[i][0] == "#":
                ans = i - 1
                break
            i += 1
        return ans

    bndNames = []
    newLines = []
    i = 0
    while i < len(lines):
        if lines[i][:len("!DLOAD")] == "!DLOAD":
            line = deleteSp(lines[i+1])
            if line.split(",")[1] == bndType:
                bndName = line.split(",")[0]
                bndNames.append(bndName)
                if isDeleteName(bndName, names) == True:
                    i += 1
                    i = skipNextHeader(lines, i)
                else:
                    newLines.append(lines[i])
            else:
                newLines.append(lines[i])
        else:
            newLines.append(lines[i])
        i += 1
    return newLines

#
#  deleteCentWithoutNames
#    設定以外のCENTを削除
def deleteCentWithoutNames(lines, names):
    global centDefine
    [lines, bndNames] = deleteAllDloadWithoutNames(lines, "CENT", names)
    centDefine = bndNames
    return lines

#
#  deleteVloadWithoutNames
#    設定以外のVLOADを削除
def deleteVloadWithoutNames(lines, names):
    global vloadDefine
    [lines, bndNames] = deleteAllDloadWithoutNames(lines, "BX", names)
    if len(bndNames) == 0:
        [lines, bndNames] = deleteAllDloadWithoutNames(lines, "BY", names)
        if len(bndNames) == 0:
            [lines, bndNames3] = deleteAllDloadWithoutNames(lines, "BZ", names)
    vloadDefine = bndNames
    return lines

#
#  deleteDloadWithoutNames
#    設定以外のDLOADを削除
def deleteDloadWithoutNames(lines, names):
    global dloadDefine
    [lines, bndNames] = deleteAllDloadWithoutNames(lines, "S", names)
    dloadDefine = bndNames
    return lines

#  deleteAllDloadWithoutNames
def deleteAllDloadWithoutNames(lines, bndType, names):
    setNames = getDloadTypeGroupInCnt(lines, bndType)
    delNames = []
    for setName in setNames:
        flag = 0
        for name in names:
            if setName == name:
                flag = 1
                break
        if flag == 0:
            delNames.append(setName)
    lines = deleteDloadNames(lines, bndType, delNames)
    return [lines, setNames]
    

#
#  deleteCloadWithoutNames
#    設定以外のCLOADを削除
def deleteCloadWithoutNames(lines, names):
    global cloadDefine
    [lines, bndNames] = deleteBndWithoutNames(lines, "!CLOAD", names)
    cloadDefine = bndNames
    return lines


def deleteBndWithoutNames(lines, bnd, names):

    def isDeleteName(bndName, names):
        flag = 0
        for name in names:
            if bndName == name:
                flag = 1
                break
        if flag == 0:
            return True
        else:
            return False

    def skipNextHeader(lines, i):
        ans = 0
        while i < len(lines):
            if lines[i][0] == "!" or lines[i][0] == "#":
                ans = i - 1
                break
            i += 1
        return ans

    bndNames = []
    newLines = []
    i = 0
    while i < len(lines):
        if lines[i][:len(bnd)] == bnd:
            line = deleteSp(lines[i+1])
            bndName = line.split(",")[0]
            bndNames.append(bndName)
            if isDeleteName(bndName, names) == True:
                i += 1
                i = skipNextHeader(lines, i)
            else:
                newLines.append(lines[i])
        else:
            newLines.append(lines[i])
        i += 1
    return [newLines, bndNames]


#
#  deleteBoundaryWithoutNames
#    設定以外のBOUNDARYを削除
def deleteBoundaryWithoutNames(lines, names):
    global bndDefine

    def isDeleteName(bndName, names):
        flag = 0
        for name in names:
            if bndName == name:
                flag = 1
                break
        if flag == 0:
            return True
        else:
            return False

    def skipNextHeader(lines, i):
        ans = 0
        while i < len(lines):
            if lines[i][0] == "!" or lines[i][0] == "#":
                ans = i - 1
                break
            i += 1
        return ans

    bndNames = []
    newLines = []
    i = 0
    while i < len(lines):
        if lines[i][:len("!BOUNDARY")] == "!BOUNDARY":
            line = deleteSp(lines[i+1])
            bndName = line.split(",")[0]
            bndNames.append(bndName)
            if isDeleteName(bndName, names) == True:
                i += 1
                i = skipNextHeader(lines, i)
            else:
                newLines.append(lines[i])
        else:
            newLines.append(lines[i])
        i += 1
    bndDefine = bndNames
    return newLines

#
#  writeFistrModelFile
#    材料物性値をファイルに書き込み
def writeFistrModelFile(elGroup, matName, elastic, matModel, matType, matHard, data):

    def isElGroup(line, elGroup):
        ans = False
        if line[:len("!SECTION")] == "!SECTION":
            egrp = abaqus2fistr.getValName(line, "EGRP")
            if egrp == elGroup:
                ans = True
        return ans

    #meshファイル修正
    lines = readFistrModelMsh()
    #!SECTION部の材料名変更
    n = 0; nLine = 0
    while n < len(lines):
        line = lines[n]
        if line[0] == "!":
            if isElGroup(line, elGroup) == True:
                newLine = abaqus2fistr.setValName(line, "MATERIAL="+matName)
                lines[n] = newLine
                nLine = n
                break
        n += 1
    #!MATERIAL部の追加部作成
    newMat =  ["!MATERIAL, NAME="+matName+", ITEM=3\n"]
    newMat += ["!ITEM=1, SUBITEM=2\n"]
    newMat += [elastic[0] + ", " + elastic[1] + "\n"]
    newMat += ["!ITEM=2, SUBITEM=1\n"]
    newMat += [elastic[2] + "\n"]
    newMat += ["!ITEM=3, SUBITEM=1\n"]
    newMat += [elastic[3] + "\n"]
    #!MATERIAL部（定義部）を修正
    newLines = addOrExchangeMaterialInMesh(lines, matName, newMat)
    #FistrModel.mshを保存
    writeFistrModelMsh(newLines)

    #制御ファイル修正
    lines = readFistrModelCnt()
    newMat =  ["!MATERIAL, NAME="+matName+"\n"]
    newMat += ["!ELASTIC, TYPE=ISOTROPIC\n"]
    newMat += [elastic[0] + ", " + elastic[1] + "\n"]
    newMat += ["!DENSITY\n"]
    newMat += [elastic[2] + "\n"]
    newMat += ["!EXPANSION_COEFF, DEPENDENCIES=0\n"]
    newMat += [elastic[3] + "\n"] 
    if matModel != "ELASTIC":
        newLine = "!" + matModel
        if matType != "":
            newLine += ", YIELD=" + matType
            if matHard != "":
                newLine += ", HARDEN=" + matHard
        newLine += "\n"
        newMat += [newLine]
        #PLASTICの場合塑性データ追加
        if matModel == "PLASTIC":
            newMat += data
    #!MATERIAL部（定義部）を修正
    lines = addOrExchangeMaterialInCnt(lines, matName, newMat)
    #Fistr.cntを保存
    writeFistrModelCnt(lines)

#
#  addOrExchangeMaterialInCnt
#    材料定義部を追加or入れ替える
def addOrExchangeMaterialInCnt(lines, matName, newMat):
    lines = deleteDefineMaterialInCnt(lines, matName)
    lines = addDefineMaterialInCnt(lines, matName, newMat)
    return lines

#  addDefineMaterialInCnt
#    cntファイルに材料定義を追加
def addDefineMaterialInCnt(lines, matName, newMat):

    def skipMaterialHeader(lines):
        ans = 0
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len("!MATERIAL")] == "!MATERIAL":
                ans = i
                break
            else:
                if line[0] == "#":
                    n1 = line.find("MATERIAL")
                    n2 = line.find("Material")
                    n3 = line.find("material")
                    if n1>= 0 or n2>=0 or n3>=0:
                        i += 1
                        if lines[i][:3] == "###":
                            ans = i + 1
                        else:
                            ans = i
                        break
            if ans != 0:
                break
            i += 1
        return ans

    i = skipMaterialHeader(lines)
    newLines = lines[:i] + newMat + lines[i:]
    return newLines

#  deleteDefineMaterialInCnt
#    cntファイルから材料定義部を削除
def deleteDefineMaterialInCnt(lines, matName):

    def nextHeader(lines, i):
        while lines[i][0] != "!" and i < len(lines):
            i += 1
        if i < len(lines):
            line = deleteSp(lines[i])
            name = line.split(",")[0][1:]
        else:
            name = ""
        return [name, i]

    def isMatHeader(name):
        ans = False
        if name == "DENSITY" or name == "EXPANSION_COEFF":
            ans = True
            return ans
        for [matModel, matTypes, matHards] in materialType:
            if name == matModel:
                ans = True
                break
        return ans

    def skipNextMaterialHeader(lines, i):
        n = i
        [name, i] = nextHeader(lines, i)
        while isMatHeader(name) == True and i < len(lines):
            n = i
            i += 1
            [name, i] = nextHeader(lines, i)
        #数字以外までskip
        i = n + 1
        while i < len(lines):
            line = deleteSp(lines[i])
            if line[0] >= "0" and line[0] <= "9":
                i += 1
            else:
                n = i
                break
        return n

    newLines = []
    flag = 0
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!MATERIAL")] == "!MATERIAL":
            name = abaqus2fistr.getValName(line, "NAME")
            if name == matName:
                newLines = lines[:i]
                i += 1
                i = skipNextMaterialHeader(lines, i)
                newLines += lines[i:]
                flag = 1
                break
        i += 1
    if flag == 0:
        newLines = lines
    return newLines

#
#  addOrExchangeMaterialInMesh
#    材料定義部を追加or入れ替える
def addOrExchangeMaterialInMesh(lines, matName, newMat):
    lines = deleteDefineMaterialInMesh(lines, matName)
    lines = addDefineMaterialInMesh(lines, matName, newMat)
    return lines

#  addDefineMaterialInMesh
#    meshファイルに材料定義を追加
def addDefineMaterialInMesh(lines, matName, newMat):
    newLines = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!SECTION")] == "!SECTION":
            name = abaqus2fistr.getValName(line, "MATERIAL")
            if name == matName:
                newLines = lines[:i]
                newLines += newMat
                newLines += lines[i:]
                break
        i += 1
    if i < len(lines):
        return newLines
    else:
        return lines

#  deleteDefineMaterialInMesh
#    meshファイルから材料定義部を削除
def deleteDefineMaterialInMesh(lines, matName):
    nItem = ""
    newLines = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!MATERIAL")] == "!MATERIAL":
            name = abaqus2fistr.getValName(line, "NAME")
            if name == matName:
                newLines = lines[:i]
                nItem = abaqus2fistr.getValName(line, "ITEM")
                keyword = "!ITEM=" + nItem
                i += 1
                while i < len(lines) and lines[i][:len(keyword)] != keyword:
                    i += 1
                if i < len(lines):
                    newLines += lines[i+2:]
        i += 1
    if nItem != "":
        return newLines
    else:
        return lines

#
#  getMaterialElasticValue
#    物性値（弾性）をDBから取得
def getMaterialElasticValue(matName):
    fileName = materialDBName
    data = pyFistr.readCsvData(fileName)
    matData = data[2:]
    vals = []
    for words in matData:
        if matName == words[0]:
            vals = words[1:5]   #[young, poisson, density, linearexp]
    return vals

#
#  readMeshGetEachMaterialName
#    group名と材料名を取得
def readMeshGetEachMaterialName():
    lines = readFistrModelMsh()
    ans = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!SECTION")] == "!SECTION":
            elGroup = abaqus2fistr.getValName(line, "EGRP")
            matName = abaqus2fistr.getValName(line, "MATERIAL")
            ans.append([elGroup, matName])
        i += 1
    return ans

#
#  readGetMaterialModel
#    材料モデル、タイプを取得
def readGetMaterialModel():

    def checkMaterialType(line):
        ans = False
        for matGroup in materialType[1:]:
            matType = matGroup[0]
            if line[1:len(matType)+1] == matType:
                ans = True
                break
        return ans

    matName = ""
    matModel="ELASTIC"; matType=""; matHard=""
    lines = readFistrModelCnt()
    i = 0
    flag = 0
    while i < len(lines):
        line = lines[i]
        if checkMaterialType(line) == True:
            matModel = (deleteSp(line)).split(",")[0][1:]
            matType = abaqus2fistr.getValName(line, "YIELD")
            matHard = abaqus2fistr.getValName(line, "HARDEN")
            flag += 1
        if flag == 2:
            break
        i += 1
    return [matModel, matType, matHard]

#  addContactGroupInTreeData
def addContactGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 7)

#  addTempGroupInTreeData
def addTempGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 6)

#  addCentGroupInTreeData
def addCentGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 5)

#  addGravGroupInTreeData
def addGravGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 4)

#  addVloadGroupInTreeData
def addVloadGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 3)

#  addDloadGroupInTreeData
#    TreeDataにDLOADを追加
def addDloadGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 2)

#  addCloadGroupInTreeData
#    TreeDataにCloadを追加
def addCloadGroupInTreeData(names):
    addBoundaryGroupInTreeData(names, 0, 1, 1)

#  addBndGroupInTreeData
#    TreeDataにbndを追加
def addBndGroupInTreeData(names):
    global treeBoundary
    treeBnd = treeBoundary[0][1][0]        #BOUNDARYを取得
    if type(treeBnd) == str or type(treeBnd) == unicode:
        treeBoundary[0][1][0] = [treeBnd, names]
    else:
        treeBoundary[0][1][0][1] = names
        treeBoundary[0][1][0][1].sort()
    makeTreeData()

#  addBoundaryGroupInTreeData
def addBoundaryGroupInTreeData(names, t1, t2, t3):
    global treeBoundary
    treeBnd = treeBoundary[t1][t2][t3]
    if type(treeBnd) == str or type(treeBnd) == unicode:
        treeBoundary[t1][t2][t3] = [treeBnd, names]
    else:
        treeBoundary[t1][t2][t3][1] = names
        treeBoundary[t1][t2][t3][1].sort()
    makeTreeData()

#
#  addStepGroupInTreeData
#
def addStepGroupInTreeData(names):
    global treeStep
    if type(treeStep[0]) == str or type(treeStep[0]) == unicode:
        treeStep = [[treeStep[0], names]]
    else:
        treeStep[0][1] = names
        treeStep[0][1].sort()
    makeTreeData()

#
#  addMatGroupInTreeData
#    TreeDataをroot以下に追加
def addMatGroupInTreeData(names):
    global treeMaterial
    if type(treeMaterial[0]) == str or type(treeMaterial[0]) == unicode:
        treeMaterial = [[treeMaterial[0], names]]
    else:
        treeMaterial[0][1] = names
        treeMaterial[0][1].sort()
    makeTreeData()

#
#  makeIniTreeData
#    初期のTreeDataを作成
def makeIniTreeData():
    global treeData, treeRoot, treeFistrModel, treeMaterial, treeBoundary, treeStep
    global treeSolution
    global treeSolver, treePost, matDefine, bndDefine
    #初期のTreeDataを取得
    [treeRoot, treeFistrModel, treeSolution, treeMaterial, treeBoundary, treeStep,
        treeSolver, treePost] = initialTreeParts()
    #材料定義を追加
    if len(matDefine) > 0:
        mat = []
        for names in matDefine:
            mat.append(names[0])
        treeMaterial = [[treeMaterial[0], mat]]
    #BOUNDARYを追加
    if len(bndDefine) > 0:
        treeBnd = treeBoundary[0][1][0]
        bnd = bndDefine
        bnd.sort()
        treeBoundary[0][1][0] = [treeBnd, bnd]
    #CLOAD
    if len(cloadDefine) > 0:
        treeBnd = treeBoundary[0][1][1]
        bnd = cloadDefine
        bnd.sort()
        treeBoundary[0][1][1] = [treeBnd, bnd]
    #DLOAD
    if len(dloadDefine) > 0:
        treeBnd = treeBoundary[0][1][2]
        bnd = dloadDefine
        bnd.sort()
        treeBoundary[0][1][2] = [treeBnd, bnd]
    #VLOAD
    if len(vloadDefine) > 0:
        treeBnd = treeBoundary[0][1][3]
        bnd = vloadDefine
        bnd.sort()
        treeBoundary[0][1][3] = [treeBnd, bnd]
    #GRAV
    if len(gravDefine) > 0:
        treeBnd = treeBoundary[0][1][4]
        bnd = gravDefine
        bnd.sort()
        treeBoundary[0][1][4] = [treeBnd, bnd]
    #CENT
    if len(centDefine) > 0:
        treeBnd = treeBoundary[0][1][5]
        bnd = centDefine
        bnd.sort()
        treeBoundary[0][1][5] = [treeBnd, bnd]
    #TEMPERATURE
    if len(tempDefine) > 0:
        treeBnd = treeBoundary[0][1][6]
        bnd = tempDefine
        bnd.sort()
        treeBoundary[0][1][6] = [treeBnd, bnd]
    #CONTACT
    if len(contactDefine) > 0:
        treeBnd = treeBoundary[0][1][7]
        bnd = contactDefine
        bnd.sort()
        treeBoundary[0][1][7] = [treeBnd, bnd]
    #STEP
    if len(stepDefine) > 0:
        treeStep = [[treeStep[0], stepDefine]]
    makeTreeData()
    return

#
#  makeTreeData
#    TreeDataを作成する
def makeTreeData():
    global treeData
    treeData = []
    treeData += treeRoot
    items = []
    items += treeFistrModel
    items += treeSolution
    items += treeMaterial
    items += treeBoundary
    items += treeStep
    items += treeSolver
    items += treePost
    treeData += [items]

#
#  getFistrModelMeshData
#    FistrModeのメッシュデータ読み込み
def getFistrModelMeshData():
    fileName = currDir + "/FistrModel.msh"
    if len(glob.glob(fileName)) != 0:
        lines = readFistrModelMsh()
        cont = getMeshParameter(lines)
    else:
        cont = ""
    return cont

#  getMeshParameter
#    mesh内容を取得
def getMeshParameter(lines):
    global meshGroup, matDefine
    #FistrModel.mshからparameterListを取得
    parameterList = getMeshParameterList(lines)
    #parameterListから数量を取得
    paraNum = getParameterNumber(parameterList)
    #meshGroup作成
    meshGroup = createMeshGroup(paraNum)
    #材料定義を取得
    matDefine = createMaterialDefine(lines)
    #出力内容を作成
    cont = createParameterCont(paraNum)
    return cont

#  createMaterialDefine
#    材料定義を取得
def createMaterialDefine(lines):
    matDefine = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[0] == "!":
            if line[:len("!SECTION")] == "!SECTION":
                egrp = abaqus2fistr.getValName(line, "EGRP")
                matName = abaqus2fistr.getValName(line, "MATERIAL")
                if matName != "":
                    matDefine.append([egrp, matName])
        i += 1
    return matDefine

#  createMeshGroup
def createMeshGroup(paraNum):
    #EGRP
    elGroup = []
    for para in paraNum[2]:
        name = para[0].split()[1]
        elGroup.append(name)
    #SGRP
    surGroup = []
    for para in paraNum[3]:
        surGroup.append(para[0].split()[1])
    #NGRP
    ndGroup = []
    for para in paraNum[4]:
        ndGroup.append(para[0].split()[1])
    ans = [elGroup, surGroup, ndGroup]
    return ans

#  createParameterCont
def createParameterCont(paraNum):
    cont = ""
    #nodeを作成
    cont += paraNum[0][0] + " " + str(paraNum[0][1]) + "\n"
    #elementを作成
    for line in paraNum[1]:
        cont += line[0] + " " + str(line[1]) + "\n"
    cont += "\n"
    #elementGroupを作成
    for line in paraNum[2]:
        cont += line[0] + " " + str(line[1]) + "\n"
    #surfaceGroupを作成
    for line in paraNum[3]:
        cont += line[0] + " " + str(line[1]) + "\n"
    #nodeGroupを作成
    for line in paraNum[4]:
        cont += line[0] + " " + str(line[1]) + "\n"
    return cont

#  getParameterNumber
def getParameterNumber(parameter):
    paraNum = [[], [], [], [], []]
    #node数を取得
    num = 0
    for line in parameter[0]:
        n = int(abaqus2fistr.getValName(line, "NUM"))
        num += n
    paraNum[0] = ["nodes", num]
    #element数を取得
    for line in parameter[1]:
        num = int(abaqus2fistr.getValName(line, "NUM"))
        typeName = abaqus2fistr.getValName(line, "TYPE")
        paraNum[1] += [["elements type:"+typeName, num]]
    #elementGroup数を取得
    for line in parameter[1]:
        grpName = abaqus2fistr.getValName(line, "EGRP")
        if grpName != "":
            num = int(abaqus2fistr.getValName(line, "NUM"))
            paraNum[2] += [["EGRP "+grpName, num]]
    for line in parameter[2]:
        grpName = abaqus2fistr.getValName(line, "EGRP")
        num = int(abaqus2fistr.getValName(line, "NUM"))
        paraNum[2] += ["EGRP "+grpName, num]
    #surfaceGroupの取得
    for line in parameter[3]:
        grpName = abaqus2fistr.getValName(line, "SGRP")
        num = int(abaqus2fistr.getValName(line, "NUM"))
        paraNum[3] += [["SGRP "+grpName, num]]
    #nodeGroupの取得
    for line in parameter[0]:
        grpName = abaqus2fistr.getValName(line, "NGRP")
        if grpName != "":
            num = int(abaqus2fistr.getValName(line, "NUM"))
            paraNum[4] += [["NGRP "+grpName, num]]
    for line in parameter[4]:
        grpName = abaqus2fistr.getValName(line, "NGRP")
        num = int(abaqus2fistr.getValName(line, "NUM"))
        paraNum[4] += [["NGRP "+grpName, num]]
    return paraNum

#  getMeshParameter
def getMeshParameterList(lines):
    meshList = [[], [], [], [], []]
    item = ""
    header = ""
    data = []
    n = 0
    maxLines = len(lines)
    while n < maxLines:
        line = lines[n]
        if line[0] == "!":
            if len(data) > 0:
                meshList = saveDataList(item, header, data, meshList)
            #nodeの取得
            if line[:len("!NODE")] == "!NODE":
                header = line
                item = "NODE"
                length = 4
                data = []
            #要素の取得
            elif line[:len("!ELEMENT")] == "!ELEMENT":
                typeName = abaqus2fistr.getValName(line, "TYPE")
                #四面体1次
                if typeName == "341":
                    header = line
                    item = "ELEMENT"
                    length = 5
                    data = []
                #四面体2次
                elif typeName == "342":
                    header = line
                    item = "ELEMENT"
                    length = 11
                    data = []
                #六面体1次
                elif typeName == "361":
                    header = line
                    item = "ELEMENT"
                    length = 9
                    data = []
                #六面体2次
                elif typeName == "362":
                    header = line
                    item = "ELEMENT"
                    length = 21
                    data = []
                #五面体1次
                elif typeName == "351":
                    header = line
                    item = "ELEMENT"
                    length = 7
                    data = []
                #五面体2次
                elif typeName == "352":
                    header = line
                    item = "ELEMENT"
                    length = 16
                    data = []
                #以外の要素
                else:
                    item = ""
            #要素groupの取得
            elif line[:len("!EGROUP")] == "!EGROUP":
                header = line
                item = "EGROUP"
                length = 1
                data = []
            #面groupの取得
            elif line[:len("!SGROUP")] == "!SGROUP":
                header = line
                item = "SGROUP"
                length = 2
                data = []
            #節点groupの取得
            elif line[:len("!NGROUP")] == "!NGROUP":
                header = line
                item = "NGROUP"
                length = 1
                data = []
            else:
                item = ""
            n += 1
        else:
            if item != "":
                [data, n] = getDataList(lines, n, item, length)
            else:
                n += 1
    if len(data) > 0:
        meshList = saveDataList(item, header, data, meshList)
    return meshList

#
#  getDataList
#    data部を取得
def getDataList(lines, n, item, length):
    data = []
    maxLines = len(lines)
    if item == "NODE":
        while n < maxLines and lines[n][0] != "!":
            line = lines[n]
            words = line.split(",")
            if len(words) >= 4:
                a = [int(words[0]), float(words[1]), float(words[2]), float(words[3])]
            else:
                a = ["", "", "", ""]
            data += [a]
            n += 1
    else:
        while n < maxLines and lines[n][0] != "!":
            line = lines[n]
            words = line.split("\n")[0].split(",")
            for word in words:
                if len(word) > 0:
                    data += [int(word)]
            n += 1
        ans = []
        line = []
        p = 0
        for a in data:
            if p < length:
                line.append(a)
            else:
                ans.append(line)
                p = 0
                line = []
                line.append(a)
            p += 1
        ans.append(line)
        data = ans
    return [data, n]

#
#  saveDataList
#    取得したdataListを保存
def saveDataList(item, header, data, meshList):
    if item == "NODE":
        num = str(len(data))
        header = abaqus2fistr.setValName(header, "NUM="+num)
        meshList[0].append(header)
    elif item == "ELEMENT":
        num = str(len(data))
        header = abaqus2fistr.setValName(header, "NUM="+num)
        meshList[1].append(header)
    elif item == "EGROUP":
        num = str(len(data))
        header = abaqus2fistr.setValName(header, "NUM="+num)
        meshList[2].append(header)
    elif item == "SGROUP":
        num = str(len(data))
        header = abaqus2fistr.setValName(header, "NUM="+num)
        meshList[3].append(header)
    elif item == "NGROUP":
        num = str(len(data))
        header = abaqus2fistr.setValName(header, "NUM="+num)
        meshList[4].append(header)
    return meshList

#
#  checkFistrModelMeshFile
#    headerからPARTNAMEとNUMを削除
def checkFistrModelMeshFile():

    def deletePartNum(line):
        newLine = line
        if line.find("PARTNAME=") >= 0 or line.find("NUM="):
            words = deleteSp(line).split(",")
            newWords = []
            for word in words:
                if word[:len("PARTNAME=")] != "PARTNAME=":
                    if word[:len("NUM=")] != "NUM=":
                        newWords.append(word)
            newLine = ", ".join(newWords) + "\n"
        return newLine

    lines = readFistrModelMsh()
    i = 0
    while i < len(lines):
        if lines[i][0] == "!":
            line = deletePartNum(lines[i])
            lines[i] = line
        i += 1
    writeFistrModelMsh(lines)


#
#  initialize
#
def initialize():
    global currDir, istrData
    istrData = getSavedData()

#  getSaveData
#    保存したデータの読み込み
def getSavedData():
    dataDir = os.getenv("easyIstrPath")
    fileName = dataDir + "/data/" + istrDataFileName
    if len(glob.glob(fileName)) == 0:
        cont = createInitialData()
        f=open(fileName, "w"); f.write(cont); f.close()
    else:
        f=open(fileName); cont=f.read(); f.close()
    ans = []
    lines = cont.split("\n")
    for line in lines:
        words = line.split()
        if len(words) > 1:
            if words[0][0] != "#":
                ans.append(line)
    return ans


#  createInitialData
def createInitialData():
    currDir = os.getcwd()
    cont = """#
#  easyIstr用data
#

# FistrModel
convertOpt unv2fistr
currDir """ + currDir + """

# material
materialDB ~/REVOCAP_PrePost-1.6.08/Fstr/mat.csv

"""
    return cont

#
#  saveIstrData
#    Istrのセットデータを保存
def saveIstrData(setData):

    def changeLine(newLine, conts):
        item = newLine.split()[0]
        i = 0
        while i < len(conts):
            line = conts[i]
            lineItems = line.split()
            if len(lineItems) > 1:
                if item == lineItems[0]:
                    conts[i] = newLine + "\n"
                    break
            i += 1 
        return conts

    fileName = os.getenv("easyIstrPath") + "/data/" + istrDataFileName
    f=open(fileName); conts=f.readlines(); f.close()
    for newLine in setData:
        conts = changeLine(newLine, conts)
    f=open(fileName, "w")
    for line in conts:
        f.write(line)
    f.close()


#
#  deleteSp
#    空白とCRを削除して返す
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara > " ":
            newLine += chara
    return newLine

#  readFistrModelMsh
#    meshファイルを読み込み
def readFistrModelMsh():
    fileName = currDir + "/FistrModel.msh"
    f=open(fileName); lines=f.readlines(); f.close()
    return lines

#  readFistrModelCnt
#    cntファイルを読み込み
def readFistrModelCnt():
    fileName = currDir + "/FistrModel.cnt"
    if len(glob.glob(fileName)) == 0:
        return []
    f=open(fileName); lines=f.readlines(); f.close()
    return lines

#  writeFistrModelMsh
#    meshファイルの書き込み
def writeFistrModelMsh(lines):
    fileName = currDir + "/FistrModel.msh"
    f=open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()

#  writeFistrModelCnt
#    cntファイルの書き込み
def writeFistrModelCnt(lines):
    fileName = currDir + "/FistrModel.cnt"
    f=open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()


if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name

    app = wx.PySimpleApp(0)
    wx.InitAllImageHandlers()
    frame_1 = MyFrame(None, wx.ID_ANY, "")
    app.SetTopWindow(frame_1)
    frame_1.Show()
    app.MainLoop()
